/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.basis;

import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.zauber.Repraesentation;
import de.bernhardjung.dsaprogramm.zauber.Zauber;

public class ElementMitNameImpl implements ElementMitName {

    /**
     * Prueft ob es sich bei dem uebergebenen String um die Id eines Elements
     * mit Name handelt.
     * 
     * @param string der ueberprueft werden soll.
     * @return {@code true}, wenn es sich um eine ID handelt.
     */
    public static final boolean istId( final String string ) {

        if ( string.startsWith( "Kulturkunde." ) ) {

            return Kulturkunde.istKulturkundeId( string );

        } else if ( string.startsWith( "Repraesentation." ) ) {

            return Repraesentation.istRepraesentationId( string );

        } else if ( string.startsWith( "Ruestung." ) ) {

            return Ruestung.istRuestungId( string );

        } else if ( string.startsWith( "Talentgruppe." ) ) {

            return Talentgruppe.istTalentgruppeId( string );

        } else if ( string.startsWith( "Talent." ) ) {

            return Talent.istTalentId( string );

        } else if ( string.startsWith( "Zauber." ) ) {

            return Zauber.istZauberId( string );

        } else {
            
            return false;
        }
    }
    
    /**
     * Liefert das ElemntMitId zu einer ID. Funktioniert nicht fuer alle
     * IDs.
     * 
     * @return Liefert das Element mit Name zu der uebergebenen ID. Liefert 
     *          {@code null}, wenn das Element nicht existiert.
     */
    public static final ElementMitName getElement( final String id ) {

        if ( id.startsWith( "Kulturkunde." ) ) {

            return Kulturkunde.getKulturkunde( id );

        } else if ( id.startsWith( "Repraesentation." ) ) {

            return Repraesentation.getRepraesentation( id );

        } else if ( id.startsWith( "Ruestung." ) ) {

            return Datenspeicher.getInstanz().getRuestung( id );

        } else if ( id.startsWith( "Talentgruppe." ) ) {

            return Talentgruppe.getTalentgruppe( id );

        } else if ( id.startsWith( "Talent." ) ) {

            return Talent.getTalent( id );

        } else if ( id.startsWith( "Zauber." ) ) {

            return Zauber.getZauber( id );

        } else {
            
            return null;
        }
    }

    /**
     * Die Id des Elements. Sie dient dazu Regelelemente wie Talente oder 
     * Eigenschaften identifiezieren zu können, wenn sie z.B. abgespeichert 
     * werden.
     */
    private final String id;
    
    /**
     * Unter diesem Schluessel kann es eine Kuerzel wie z.B. "LE" fuer 
     * Lebensenergie, in der Lokalisierungsdatei geben.
     */
    private final String lokalisierungsschluesselKuerzel;
    
    /**
     * Unter diesem Schluessel sollte der Name des Elements in der 
     * Lokalisierungsdatei stehen.
     */
    private final String lokalisierungsschluesselName;
    
    /**
     * Der Name des Elements. Wenn der Name aus der Lokalisierungsdatei
     * verwendet wird ist dieser Wert {@code null}.
     */
    protected String name;
    
    /**
     * @param dieId des Elements.
     */
    public ElementMitNameImpl( final String dieId ) {
        
        id = dieId;
        
        lokalisierungsschluesselKuerzel = dieId + KUERZEL;
        
        lokalisierungsschluesselName = dieId + NAME;
        
        name = null;
    }

    /**
     * @return Die Id des Elements.
     */
    public final String getId() {
        
        return id;
    }
    
    /**
     * @return Das Kuerzel fuer den Namen des Elements, falls es ein solches
     *          Kuerzel gibt. Sonst wird irgendein String geliefert.
     */
    public final String getKuerzel() {
        
        if ( null == lokalisierungsschluesselKuerzel ) {
            
            return "Kuerzel";
            
        } else if ( I18n.hatLokalisierungstext( 
                lokalisierungsschluesselKuerzel ) ) {
            
            return I18n.i18n( lokalisierungsschluesselKuerzel );
        
        } else {
            
            return lokalisierungsschluesselKuerzel;
        }
    }
    
    /**
     * @return Der Schluessel fuer das Kuerzel des Elements in der 
     *          Lokalisierungsdatei.
     */
    public String getLokalisierungsschluesselKuerzel() {
        
        return lokalisierungsschluesselKuerzel;
    }
    
    /**
     * @return Der Schluessel ueber den der Name des Elements aus der 
     *          Lokalisierungsdatei geladen wird.
     */
    public final String getLokalisierungsschluesselName() {
        
        return lokalisierungsschluesselName;
    }
    
    /**
     * @return Den Namen des Elements.
     */
    public String getName() {
        
        if ( null != name ) {
            
            return name;
        
        } else if ( 
                I18n.hatLokalisierungstext( lokalisierungsschluesselName ) ) {
            
            return I18n.i18n( lokalisierungsschluesselName );
        
        } else {
            
            return lokalisierungsschluesselName;
        }
    }
    
    /**
     * @param neuerName des Elements. Es kann {@code null} oder ein leerer 
     *          String uebergeben werden, dann wird versucht einen Namen aus
     *          der Lokalisierungsdatei zu laden. Entspricht der uebergebene
     *          Name der Lokalisierung des Namens diese Elements, so wird
     *          immer die Lokalisierung des Namens verwendet.
     */
    public final void setName( String neuerName ) {
        
        if ( ( null == neuerName ) || neuerName.trim().equals( "" ) ) {
            
            name = null;
        
        } else if ( I18n.hatLokalisierungstext( 
                lokalisierungsschluesselName ) ) {
            
            final String lokalisierung = 
                I18n.i18n( lokalisierungsschluesselName );
            
            if ( lokalisierung.equals( neuerName ) ) {
                
                name = null;
            
            } else {
                
                name = neuerName.trim();
            }
        
        } else {
            
            name = neuerName.trim();
        }
    }
}
