/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Util;

public class Haarfarbe {

    /**
     * Sortiert die Haarfarben alphabetisch nach Namen.
     */
    public static final Comparator< Haarfarbe > NAME_COMPARATOR =
        new Comparator< Haarfarbe >() {

            public int compare( final Haarfarbe h1, final Haarfarbe h2 ) {
                
                return Util.compare( h1.getName(), h2.getName() );
            }
    };
    
    /** Eine Liste aller Haarfarben. */
    private static final Vector< Haarfarbe > HAARFARBEN = 
        new Vector< Haarfarbe >();

    /** Eine Map in der jede Haarfarbe unter ihrer Id gespeichert ist. */
    private static final Map< String, Haarfarbe > ID_HAARFARBE_MAP =
        new HashMap< String, Haarfarbe >();

    public static final Haarfarbe BLOND = new Haarfarbe( "blond" );
    public static final Haarfarbe BRAUN = new Haarfarbe( "braun" );
    public static final Haarfarbe WEISS = new Haarfarbe( "weiss" );

    static {
        final String[] ids = {
                "blauschwarz",
                "blond",
                "braun",
                "dunkelblond",
                "dunkelbraun",
                "dunkelgrau",
                "feuerrot",
                "hellblond",
                "hellbraun",
                "hellgrau",
                "kupferrot",
                "mittelblond",
                "mittelbraun",
                "rot",
                "rotblond",
                "salzweiss",
                "schwarz",
                "silbern",
                "silberweiss",
                "weiss",
                "weissblond" };

        for ( final String id : ids ) {

            final Haarfarbe haarfarbe = new Haarfarbe( id );

            HAARFARBEN.add( haarfarbe );

            // HAARFARBE_ID_MAP.put( farbe, id );

            ID_HAARFARBE_MAP.put( id, haarfarbe );
        }        
    }

    /** @return eine alphabetisch sortierte Liste mit Haarfarben. */
    public static final Vector< Haarfarbe > getHaarfarben() {
        
        Collections.sort( HAARFARBEN, NAME_COMPARATOR );
        
        return new Vector< Haarfarbe >( HAARFARBEN );
    }

    /**
     * @param idOderHaarfarbe
     *            Hier kann die ID einer Haarfarbe oder direkt der Name einer
     *            Haarfarbe uebergeben werden. Haarfarben mit ID koennen
     *            Lokalisiert werden, wenn sie dem Programm bekannt sind.
     * @return die Haarfarbe zur gesuchten Id.
     */
    public static final Haarfarbe getHaarfarbe( final String idOderHaarfarbe ) {

        if ( ! ID_HAARFARBE_MAP.containsKey( idOderHaarfarbe ) ) {
            
            final Haarfarbe haarfarbe = new Haarfarbe( idOderHaarfarbe );
            
            ID_HAARFARBE_MAP.put( idOderHaarfarbe, haarfarbe );
            
            return haarfarbe;
        
        } else {
            
            return ID_HAARFARBE_MAP.get( idOderHaarfarbe );
        }
    }

    private final String id;
    
    public Haarfarbe( final String dieId ) {
        
        id = dieId;
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj ) {
            
            return false;
        }
        
        if ( ! getClass().equals( obj.getClass() ) ) {
            
            return false;
        }
        
        final Haarfarbe haarfarbe = (Haarfarbe) obj;
        
        return getId().equals( haarfarbe.getId() );
    }
    
    /**
     * @return Die ID die auch zur Lokalisierung verwendet wird.
     */
    public String getId() {
        
        return id;
    }
    
    /**
     * @return Die Bezeichnung der Haarfarbe.
     */
    public String getName() {
        
        if ( I18n.hatLokalisierungstext( id ) ) {
            
            return I18n.i18n( id );
        
        } else {
            
            return id;
        }
    }
    
    @Override
    public int hashCode() {
        
        return id.hashCode();
    }
    
    @Override
    public String toString() {
        
        return getName();
    }
}
