/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Set;

import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basis.varianten.VarianteListener;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.basiswerte.BasiswertWert;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.eigenschaften.EigenschaftWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWertListe;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWerte;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.talente.TalentWerte;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Beobachtbar;
import de.bernhardjung.dsaprogramm.util.Beobachter;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWerte;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;
import de.bernhardjung.dsaprogramm.zauber.ZauberWerte;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

public class Held extends Observable implements Beobachter, VarianteListener {

    /** Die Anzahl Fernkampfwaffen, die ein Held maximal tragen kann. */
    public static final int ANZAHL_FERNKAMPFWAFFEN = 3;

    /** Die Anzahl Nahkampfwaffen, die ein Held maximal tragen kann. */
    public static final int ANZAHL_NAHKAMPFWAFFEN = 5;

    /** Die Anzahl Ruestungsteile, die ein Held maximal tragen kann. */
    public static final int ANZAHL_RUESTUNGSTEILE = 5;

    /** Die Anzahl Schilde, die ein Held maximal tragen kann. */
    public static final int ANZAHL_SCHILDE = 3;

    /** Wird an die Observer geschickt, wenn die Augenfarbe veraendert wird. */
    public static final Object AUGENFARBE = new Object();

    /** Wird an die Obeserver geschickt, wenn die Haarfarbe veraendert wird. */
    public static final Object HAARFARBE = new Object();

    /** Wird an die Obeserver geschickt, wenn die Rasse geaendert wird. */
    public static final Object RASSE = new Object();

    /** Ein Zaehler fuer die vom Benutzer aktivierten Talente. */
    private int aktivierteTalente;

    /** Ein Zaehler fuer die vom Benutzer aktivierten Zauber. */
    private int aktivierteZauber;

    /** Das Alter des Helden in Jahren. */
    private int alter;

    /** Die AP die der Held schon zum Steigern aufgewendet hat. */
    private int apEingesetzt;

    /** Die AP die dem Helden insgesamt zum Steigern zu Verfuegung stehen. */
    private int apGesamt;

    /** Die Augenfarbe des Helden. */
    private Augenfarbe augenfarbe;

    /** Eine Beschreibung des Aussehens des Helden. */
    private String aussehen;

    /** Die Basiswerte des Helden. */
    private final List< BasiswertWert > basiswerte;

    /** Die Eigenschaftswerte des Helden. */
    private final List< EigenschaftWert > eigenschaften;

    /** Die Fernkampfwaffen des Helden. */
    private final List< FernkampfwaffeWert > fernkampfwaffen;

    /** Der Geburtstage des Helden. */
    private String geburtstag;

    /** Das Geschlecht des Helden/der Heldin. */
    private Geschlecht geschlecht;

    /**
     * Alle Gelaendekunden die der Held erhalten hat.
     * Die Werte sind in der Reihenfolge wie der Held sie erhalten
     * hat mit der ersten Gelaendekunde am Postion 0.
     * Mit dieser Liste kann festgestellt werden welches die erste
     * Gelaendekunde ist die der Held erhalten hat. Diese ist teurer als die
     * anderen.
     */
    private final SonderfertigkeitWertListe gelaendekunden;

    /** Das Gwicht des Helden in Stein/kg. */
    private int gewichtInKg;

    /** Die Groesse des Helden in Halbfingern/cm. */
    private int groesseInCm;

    /** Die Haarfarbe des Helden. */
    private Haarfarbe haarfarbe;

    /** Die Hauszauber des Helden. */
    private final Set< Zauber > hauszauber;

    /** Informationen zu Familie, Herkunft und Hintergrund des Helden. */
    private String hintergrund;

    /** Die Kultur des Helden. */
    private Kultur kultur;

    /** Die Leittalente eines elfischen Helden. */
    private final Set< Talent > leittalente;

    /** Die Leitsonderfertigkeiten eines elfischen Helden. */
    private final SonderfertigkeitWerte leitsonderfertigkeiten;

    /** Die Leitzauber eines elfischen Helden. */
    private final Set< Zauber > leitzauber;

    /** Die Muttersprache des Helden. */
    private Sprache muttersprache;

    /** Die Nahkampfwaffen des Helden. */
    private final List< NahkampfwaffeWert > nahkampfwaffen;

    /** Der Name des Helden. */
    private String name;

    /**
     * Die Varianten der Sonderfertigkeit Ortskenntnis die der Held erhalten
     * hat. Die Werte sind in der Reihenfolge in der der Held sie erhalten hat
     * mit der ersten Ortskenntnis an Position 0.
     * Mit dieser Liste kann man feststellen welche Ortskenntnis der Held
     * als erste erhalten hat. Diese Ortskenntnis ist teurer als die anderen.
     */
    private final SonderfertigkeitWertListe ortskenntnisse;

    /** Die Profession des Helden. */
    private Profession profession;

    /** Die Rasse des Helden. */
    private Rasse rasse;

    /** Die Ruestungen, die der Held traegt. */
    private final List< RuestungWert > ruestungen;

    /** Die Schilde die der Held traegt. */
    private final List< SchildWert > schilde;

    /** Informationen ueber den Stand des Helden. */
    private String stand;

    /**
     * Alle Sonderfertigkeiten die der Held erhalten hat.
     */
    private final SonderfertigkeitWerte sonderfertigkeiten;

    /**
     * Die Sonderfertigkeiten von denen der Held alle Varianten verbilligt
     * erhaelt.
     */
    private final Set< Sonderfertigkeit > sonderfertigkeitenVerbilligt;

    /**
     * Die Sonderfertigkeiten von denen der Held nur einzelne Varianten
     * verbilligt erhaelt. Aller hier gespeicherten Sonderfertigkeiten kosten
     * den Helden nur noch 50%.
     */
    private final SonderfertigkeitWerte sonderfertigkeitenVerbilligtVarianten;

    /**
     * Die Sonderfertigkeiten die dem Held durch Rasse, Kultur und Profession
     * vorgegeben sind.
     */
    private final SonderfertigkeitWerte sonderfertigkeitenVorgegeben;

    /**
     * Die Sonderfertigkeiten die dem Held durch Rasse, Kultur und Profession
     * vorgegeben sind, die er aber nicht besitzt.
     */
    private final SonderfertigkeitWerte sonderfertigkeitenFehlend;

    /**
     * Alle Talentwerte die der Held erhalten hat.
     * Diese Liste ist die vollstaendinge Liste aller Talentwerte.
     * Sie enthaelt zu jedem Talent stehts den aktuellen Talentwert.
     */
    private final TalentWerte talente;

    /**
     * Die Talente die vorgegeben sind, die der Held aber nicht besitzt, weil
     * der Benutzer sie deaktiviert hat.
     */
    private final Set< Talent > talenteFehlend;

    /** Die Talentwerte die der Held durch seine Profession erhalten hat. */
    private final TalentWerte talenteProfession;

    /**
     * Die Basis-Talente und die durch Rasse, Kultur und Profession
     * vorgegebenen Talentwerte.
     */
    private final TalentWerte talenteVorgegeben;

    /**
     * Ein Flag, das gesetzt wird, um zu verhindern, dass die Methode 
     * {@link #varianteGeaendert(SonderfertigkeitWert, Variante, Variante)} bei
     * einer Aenderung mehrfach aufgerufen wird.   
     */
    private boolean varianteGeaendertAktiv;
    
    /**
     * Eine Profession, z.B. Streuner, erhaelt automatisch Verbindungen im Wert
     * von SO x Punkten. Hier werden die so finanzierten Verbindungen
     * gespeichert die der Benutzer ausgewaehlt hat.
     */
    private final VorNachteilWerte verbindungen;

    /** Der Titel des Helden, so er denn von adligem Gebluet ist. */
    private String titel;

    /** Die Vor-/Nachteile des Helden. */
    private final VorNachteilWerte vorNachteile;

    /**
     * Die Nachteile, die der Held, durch Einsatz von AP, abgebaut hat.
     */
    private final VorNachteilWerte vorNachteileAbgebaut;

    /**
     * Die Vor-/Nachteile die fuer den Held vorgegeben sind, die er aber nicht
     * besitzt, weil der Benutzer sie .
     */
    private final VorNachteilWerte vorNachteileFehlend;

    /** Die durch Rasse, Kultur oder Profession Vor-/Nachteile des Helden. */
    private final VorNachteilWerte vorNachteileVorgegeben;

    /** Gibt an ob der Held gerade erschaffen oder gesteigert wird. */
    private boolean wirdErschaffen;

    /**
     * Die Zauber die der Held besitzt.
     */
    private final ZauberWerte zauber;

    /**
     * Die fehlenden Zauber die der Held eigentlich besitzen sollte, weil
     * sie vorgegeben sind.
     */
    private final ZauberWerte zauberFehlend;

    /**
     * Die Zauber die durch Rasse, Kultur und Profession vorgegeben sind.
     */
    private final ZauberWerte zauberVorgegeben;

    /** Die Zweitsprache des Helden */
    private Sprache zweitsprache;

    public Held() {

        aktivierteTalente = 0;

        aktivierteZauber = 0;

        alter = 29;

        apEingesetzt = 0;

        apGesamt = 0;

        augenfarbe = Augenfarbe.BRAUN;

        aussehen = "";

        basiswerte = new ArrayList< BasiswertWert >( 9 );
        for ( final Basiswert basiswert : Basiswert.getBasiswerte() ) {

            basiswerte.add( new BasiswertWert( this, basiswert ) );
        }

        eigenschaften = new ArrayList< EigenschaftWert >( 9 );
        for ( final Eigenschaft eigenschaft :
                                    Eigenschaft.getEigenschaftenMitSo() ) {

            eigenschaften.add( new EigenschaftWert(this, eigenschaft) );
        }

        fernkampfwaffen =
            new ArrayList< FernkampfwaffeWert >( ANZAHL_FERNKAMPFWAFFEN );

        geburtstag = "1. Praios, 1 Hal";

        gelaendekunden = new SonderfertigkeitWertListe();

        geschlecht = Geschlecht.MAENNLICH;

        gewichtInKg = 78;

        groesseInCm = 182;

        haarfarbe = Haarfarbe.BRAUN;

        hauszauber = new HashSet< Zauber >();

        hintergrund = "";

        kultur = Kultur.LEERE_KULTUR;

        leittalente = new HashSet< Talent >();

        leitsonderfertigkeiten = new SonderfertigkeitWerte();

        leitzauber = new HashSet< Zauber >();

        nahkampfwaffen =
            new ArrayList< NahkampfwaffeWert >( ANZAHL_NAHKAMPFWAFFEN );

        name = "";

        ortskenntnisse = new SonderfertigkeitWertListe();

        profession = Profession.LEERE_PROFESSION;

        rasse = Rasse.LEERE_RASSE;

        ruestungen = new ArrayList< RuestungWert >( ANZAHL_RUESTUNGSTEILE );

        schilde = new ArrayList< SchildWert >( ANZAHL_SCHILDE );

        stand = "";

        sonderfertigkeiten = new SonderfertigkeitWerte( this );

        sonderfertigkeitenVerbilligt = new HashSet< Sonderfertigkeit >();

        sonderfertigkeitenVerbilligtVarianten = new SonderfertigkeitWerte( this );

        sonderfertigkeitenVorgegeben = new SonderfertigkeitWerte();

        sonderfertigkeitenFehlend = new SonderfertigkeitWerte();

        talente = new TalentWerte( this );

        talenteFehlend = new HashSet< Talent >();

        talenteProfession = new TalentWerte();

        talenteVorgegeben = new TalentWerte();

        titel = "";

        varianteGeaendertAktiv = false;
        
        verbindungen = new VorNachteilWerte( this );

        vorNachteile = new VorNachteilWerte( this );

        vorNachteileAbgebaut = new VorNachteilWerte();

        vorNachteileFehlend = new VorNachteilWerte();

        vorNachteileVorgegeben = new VorNachteilWerte();

        wirdErschaffen = true;

        zauber = new ZauberWerte( this );

        zauberFehlend = new ZauberWerte( this );

        zauberVorgegeben = new ZauberWerte();

        // Die Basis-Talente duerfen erst eingefuegt werden wenn talente und
        // vorgegebeneTalente initialisiert wurden, damit sie auch in beide
        // Listen eingetragen werden.
        for ( final Talent basisTalent : Talent.getBasisTalente() ) {
            einfuegenVorgegeben( new TalentWert( basisTalent, 0 ) );
        }
    }

    /**
     * Erzeugt eine Kopie des uebergebenen Helden. Man erhält einen komplett 
     * neuen Held mit exakt den selben Werten wie der Alte.
     * Das einzige, das nicht kopiert wird ist die Liste der Observer des
     * Helden.
     */
    public Held( final Held held ) {
        
        aktivierteTalente = held.aktivierteTalente;
        
        aktivierteZauber = held.aktivierteZauber;
        
        alter = held.alter;
        
        apEingesetzt = held.apEingesetzt;
        
        apGesamt = held.apGesamt;
        
        augenfarbe = held.augenfarbe;
        
        aussehen = held.aussehen;
        
        basiswerte = BasiswertWert.deepCopy( held.basiswerte );
    
        eigenschaften = EigenschaftWert.deepCopy( held.eigenschaften );
    
        fernkampfwaffen = FernkampfwaffeWert.deepCopy( held.fernkampfwaffen );

        geburtstag = held.geburtstag;
        
        gelaendekunden = held.gelaendekunden.clone();
        
        geschlecht = held.geschlecht;
        
        gewichtInKg = held.gewichtInKg;
        
        groesseInCm = held.groesseInCm;
        
        haarfarbe = held.haarfarbe;
        
        hauszauber = new HashSet< Zauber >( held.hauszauber );
        
        hintergrund = held.hintergrund;
        
        kultur = held.kultur;
        
        leitsonderfertigkeiten = held.leitsonderfertigkeiten.clone();
        
        leittalente = new HashSet< Talent >( held.leittalente );
        
        leitzauber = new HashSet< Zauber >( held.leitzauber );
        
        muttersprache = held.muttersprache;
        
        nahkampfwaffen = NahkampfwaffeWert.deepCopy( held.nahkampfwaffen );
        
        name = held.name;
        
        ortskenntnisse = held.ortskenntnisse.clone();
        
        profession = held.profession;
        
        rasse = held.rasse;
        
        ruestungen = RuestungWert.deepCopy( held.ruestungen );
        
        schilde = SchildWert.deepCopy( held.schilde );
        
        sonderfertigkeiten = held.sonderfertigkeiten.clone();
        
        sonderfertigkeitenFehlend = held.sonderfertigkeitenFehlend.clone();
        
        sonderfertigkeitenVerbilligt = new HashSet< Sonderfertigkeit >(
                held.sonderfertigkeitenVerbilligt );
        
        sonderfertigkeitenVerbilligtVarianten =
            held.sonderfertigkeitenVerbilligtVarianten.clone();
        
        sonderfertigkeitenVorgegeben =
            held.sonderfertigkeitenVorgegeben.clone();
        
        stand = held.stand;
        
        talente = held.talente.clone();
        
        talenteFehlend = new HashSet< Talent >( held.talenteFehlend );
        
        talenteProfession = held.talenteProfession.clone();
        
        talenteVorgegeben = held.talenteVorgegeben.clone();
        
        titel  = held.titel;
        
        varianteGeaendertAktiv = false;
        
        verbindungen = held.verbindungen.clone();
        
        vorNachteile = held.vorNachteile.clone();
        
        vorNachteileAbgebaut = held.vorNachteileAbgebaut.clone();
        
        vorNachteileFehlend = held.vorNachteileFehlend.clone();
        
        vorNachteileVorgegeben = held.vorNachteileVorgegeben.clone();
        
        wirdErschaffen = held.wirdErschaffen;
        
        zauber = held.zauber.clone();
        
        zauberFehlend = held.zauberFehlend.clone();
        
        zauberVorgegeben = held.zauberVorgegeben.clone();
        
        zweitsprache = held.zweitsprache;
    }

    /**
     * Baut einen Nachteil ab und berechnet dem Helden die Abbaukosten.
     *
     * @see #istAbgebaut(VorNachteilWert)
     * @param wert
     *            der abgebaut werden soll.
     */
    public final void abbauen( final VorNachteilWert wert ) {

        vorNachteileAbgebaut.einfuegen( wert );

        final VorNachteilWert heldenwert = getWert( wert );

        addiereApEingesetzt( heldenwert.getAbbaukosten( this ) );

        entfernen( wert );
    }

    /**
     *
     * @param ap die zu den eingesetzten AP addiert werden sollen.
     */
    public final void addiereApEingesetzt( final int ap ) {

        if ( 0 != ap ) {

            apEingesetzt += ap;

            apEingesetzt = Math.max( 0, apEingesetzt );

            notifyObservers();
        }
    }

    /**
     * @param ap die zu den Gesamt-AP addiert werden sollen.
     */
    public final void addiereApGesamt( final int ap ) {

        if ( 0 != ap ) {

            apGesamt += ap;

            apGesamt = Math.max( 0, apGesamt );

            notifyObservers();
        }
    }

    /**
     * Aktiviert ein Talent und berechnet dem Helden die entstehenden
     * Aktivierungskosten.
     *
     * @param talent
     *            das aktiviert werden soll.
     * @param lernmethode
     *            die zum Aktivieren verwendet werden soll.
     * @throws IllegalStateException
     *             wenn das Talent schon aktiviert ist.
     */
    public final void aktivieren(
            final Talent talent,
            final Lernmethode lernmethode ) {

        if ( hat( talent ) &&  getWert( talent ).istAktiviert() ) {

            throw new IllegalStateException( talent.getName() +
                    " ist schon aktiviert." );
        }

        if ( hat( talent )
          && ! getWert( talent ).istAktiviert() ) {
            // Sonderfall Spezial-Talent mit negativer vorgegebener Stufe:
            // Das Talent ist nur noch als aktivert zu markieren und auf
            // Stufe 0 zu setzen.
            final TalentWert wert = getWert( talent );

            wert.setAktiviert( true );

            int kosten = talent.getAktivierungskosten( this, lernmethode );

            wert.setAktivierungskosten( kosten );

            kosten += wert.setStufe( this, 0, lernmethode );

            apEingesetzt += kosten;

            notifyObservers( talent );

        } else if ( ! hat( talent ) ) {

            final int stufe;

            final int aktivierungskosten;

            if ( hatVorgegeben( talent ) ) {

                stufe = getMindeststufe( talent );

                aktivierungskosten = 0;

            } else {

                stufe = 0;

                aktivierungskosten =
                    talent.getAktivierungskosten( this, lernmethode );

            }

            apEingesetzt += aktivierungskosten;

            einfuegen( new TalentWert( talent, stufe, aktivierungskosten ) );
        }
    }

    /**
     * Aktiviert einen Zauber und berechnet dem Held die AP-Kosten die dadurch
     * entstehen. Der Zauber hat nach dem Aktivieren die Stufe 0 oder, falls es
     * eine Mindeststufe gibt, die Mindeststufe. Die mit dem {@code wert}
     * uebergebenen Stufe wird ignoriert.
     *
     * @param wert
     *            der aktiviert werden soll.
     * @param lernmethode
     *            die zum Aktivieren angewendet wird.
     */
    public final void aktivieren(
            final ZauberWert wert,
            final Lernmethode lernmethode ) {

        if ( hat( wert ) ) {

            throw new IllegalStateException( wert.getName() + " ist schon " +
            		"aktiviert." );
        }

        if ( hatVorgegeben( wert ) ) {

            einfuegen( getVorgegeben( wert ) );

        } else {

            final int aktivierungskosten =
                wert.getAktivierungskosten( this, lernmethode );

            apEingesetzt += aktivierungskosten;

            final ZauberWert neuerWert = wert.clone();

            neuerWert.setStufe( 0 );

            neuerWert.setAktivierungskosten( aktivierungskosten );

            einfuegen( neuerWert );
        }
    }

    /**
     * Deaktiviert ein Talent und berechnet dem Helden ggf. die AP, die dadurch
     * frei werden.
     *
     * @param talent das deaktivert werden soll.
     */
    public final void deaktivieren( final Talent talent ) {

        if ( hatVorgegeben( talent )
          && ( getMindeststufe( talent ) < 0 )
          && ( ! talent.istBasisTalent() ) ) {
            // Sonderfall Spezial-Talent mit negativer vorgegebener Stufe:
            // Das Talent wird nicht entfernt, sonder nur als deaktivert
            // markiert. Die Stufe wird auf den vorgegebenen negativen Wert
            // gesetzt oder bleibt auf 0 wenn sie gerade 0 ist.
            final TalentWert wert = getWert( talent );

            wert.setAktiviert( false );

            int kosten = - wert.getAktivierungskosten();

            wert.setAktivierungskosten( 0 );

            int stufe = wert.getStufe( this );

            if ( 0 < stufe ) {
                // Aktuell ist die Stufe positiv, da das Talent deaktiviert
                // werden soll wird es auf die negative vorgegebene Stufe
                // gestellt.
                stufe = getMindeststufe( talent );
            }

            kosten += wert.setStufe( this, stufe, Lernmethode.KEINE );

            apEingesetzt += kosten;

            notifyObservers( talent );

        } else {
            // Der normale Fall. Das Talent wird einfach entfernt.
            final TalentWert wert = getWert( talent );

            apEingesetzt -= wert.getApKosten();

            entfernen( talent );
        }

    }

    /**
     * Deaktiviert einen Zauber und berechnet dem Helden ggf. die AP, die
     * dadurch frei werden.
     *
     * @param wert der deaktivert werden soll.
     */
    public final void deaktivieren( final ZauberWert wert ) {

        if ( hat( wert ) ) {

            final int apKosten = getWert( wert ).getApKosten();

            apEingesetzt -= apKosten;

            entfernen( wert );
        }
    }

    /**
     * Fuegt dem Helden eine Fernkampfwaffe hinzu. Ein Held kann maximal
     * {@link #ANZAHL_FERNKAMPFWAFFEN}
     * Fernkampfwaffen tragen. Versuche mehr Waffen einzufuegen werden
     * ignoriert. Es kann kein Objekt zweimal eingefuegt werden.
     *
     * @param waffe
     *            die der Held erhalten soll.
     */
    public final void einfuegen( final FernkampfwaffeWert waffe ) {

        if ( ! fernkampfwaffen.contains( waffe )
          && ( fernkampfwaffen.size() < ANZAHL_FERNKAMPFWAFFEN ) ) {

            fernkampfwaffen.add( waffe );
        }
    }

    /**
     * Fuegt dem Helden eine Nahkampfwaffe hinzu. Ein Held kann maximal
     * {@link #ANZAHL_NAHKAMPFWAFFEN}
     * Nahkampfwaffen besitzen. Versuche mehr Waffen einzufuegen werden
     * ignoriert. Es kann kein Objekt zweimal eingefuegt werden.
     *
     * @param waffe
     *            die der Held erhalten soll.
     */
    public final void einfuegen( final NahkampfwaffeWert waffe ) {

        if ( ! nahkampfwaffen.contains( waffe )
          && ( nahkampfwaffen.size() < ANZAHL_NAHKAMPFWAFFEN ) ) {

            nahkampfwaffen.add( waffe );
        }
    }

    /**
     * Fuegt dem Helden eine Ruestung hinzu. Ein Held kann maximal
     * {@link #ANZAHL_RUESTUNGSTEILE}
     * Ruestungsteile tragen. Versuche mehr Ruestungen einzufuegen werden
     * ignoriert. Es kann kein Objekt zweimal eingefuegt werden.
     *
     * @param wert
     *            die Ruestung, die der Held tragen soll.
     */
    public final void einfuegen( final RuestungWert wert ) {

        if ( ! ruestungen.contains( wert )
          && ( ruestungen.size() < ANZAHL_RUESTUNGSTEILE ) ) {

            ruestungen.add( wert );

            wert.setHeld( this );

            notifyObservers( RuestungWert.BE );
        }
    }

    /**
     * Fuegt dem Helden einen Schild hinzu. Ein Held kann maximal
     * {@link #ANZAHL_SCHILDE}
     * Schilde tragen. Versuche mehr Schilde einzufuegen werden
     * ignoriert. Es kann kein Objekt zweimal eingefuegt werden.
     *
     * @param schild
     *            den der Held tragen soll.
     */
    public final void einfuegen( final SchildWert schild ) {

        if ( ! schilde.contains( schild )
          && ( schilde.size() < ANZAHL_SCHILDE ) ) {

            schilde.add( schild );
        }
    }

    /**
     * Fuegt dem Helden eine Sonderfertigkeit hinzu. Wenn der Held die
     * Sonderfertigkeit schon besitzt macht die Methode nichts. Diese Methode
     * kann bei der Erschaffung und beim Steigern verwendet werden. Evtl.
     * Lernkosten (@link {@link SonderfertigkeitWert#setLernkosten(int)})
     * werden dem Helden berechnet.
     *
     * @param wert
     *            den der Held erhalten soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegen( SonderfertigkeitWert wert ) {

        if ( ! hat( wert ) ) {

            if ( sonderfertigkeitenFehlend.enthaelt( wert ) ) {

                wert = sonderfertigkeitenFehlend.getWert( wert );

                sonderfertigkeitenFehlend.entfernen( wert );
            }

            sonderfertigkeiten.einfuegen( wert );

            final Sonderfertigkeit sonderfertigkeit = wert.getSonderfertigkeit();

            // In den Listen ortskenntnisse und gelaendekunden soll das selbe
            // Objekt gespeichert sein wie in sonderfertigkeiten, damit der Eintrag
            // in den Listen automatisch aktualisiert wird wenn eine Veraenderung
            // am Wert in sonderfertigkeiten durchgefuehrt wird.
            if ( Sonderfertigkeit.ORTSKENNTNIS == sonderfertigkeit ) {

                ortskenntnisse.add( sonderfertigkeiten.getWert( wert ) );

            } else if ( sonderfertigkeit.istGelaendekunde() ) {

                gelaendekunden.add( sonderfertigkeiten.getWert( wert ) );

            }

            apEingesetzt += wert.getLernkosten();

            notifyObservers( sonderfertigkeit );
        }
    }

    /**
     * Fuegt dem Helden einen Talentbonus hinzu. Wenn der Held das Talent noch
     * nicht besitzt erhaelt er es neu, wenn er es schon besitzt wird die
     * unmodifizierte Stufe von {@code wert} zum bestehenden Talentwert addiert.
     * Diese Methode ist fuer die Erschaffung gedacht. Zum Aktivieren von
     * Talenten waehrend des Steigerns stehen extra Methoden zur Verfuegung.
     *
     * @see #aktivieren(Talent, Lernmethode)
     * @see #deaktivieren(Talent)
     * @see #setStufe(Talent, int, Lernmethode)
     * @param wert
     *            den der Held erhalten soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegen( final TalentWert wert ) {

        final Talent talent = wert.getTalent();

        // Das Hochzaehlen muss vor dem Einfuegen geschehen, denn beim
        // einfuegen werden die Observer benachrichtigt und die lesen
        // dann noch den alten Wert aus.
        if ( ! hatVorgegeben( talent ) ) {
            // Das Talent ist nicht vorgegeben, also hat es der Benutzer
            // aktiviert.
            ++aktivierteTalente;
        }

        talenteFehlend.remove( talent );

        talente.einfuegen( wert );
    }

    /**
     * Fuegt dem Helden einen Vor-/Nachteil hinzu. Wenn der Held den Wert schon
     * besitzt, so macht die Methode nichts. Wenn der Held den Wert schon
     * besitzt und der Wert eine Stufe hat, wird seine Stufe zur Stufe des
     * Wertes, den der Held schon besitzt, hinzu addiert.
     *
     * @param wert
     *            den der Held erhalten soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegen( VorNachteilWert wert ) {

        if ( vorNachteileFehlend.enthaelt( wert ) ) {

            final int stufe = wert.getStufe();

            wert = vorNachteileFehlend.getWert( wert );

            wert.setStufe( stufe );

            vorNachteileFehlend.entfernen( wert );
        }

        if ( vorNachteileAbgebaut.enthaelt( wert ) ) {

            vorNachteileAbgebaut.entfernen( wert );

            addiereApEingesetzt( - wert.getAbbaukosten( this ) );
        }

        vorNachteile.einfuegen( wert );

        wert.eingefuegt( this );

        notifyObservers( wert.getVorNachteil() );
    }

    /**
     * Fuegt dem Helden einen Zauber hinzu. Wenn der Held das Zauber noch
     * nicht besitzt erhaelt er ihn neu, wenn er ihn schon besitzt wird die
     * Stufe von {@code wert} zur bestehenden Stufe addiert.
     *
     * @param wert
     *            den der Held erhalten soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegen( final ZauberWert wert ) {

        // Das Hochzaehlen muss vor dem Einfuegen geschehen, denn beim
        // einfuegen werden die Observer benachrichtigt und die lesen
        // dann noch den alten Wert aus.
        if ( ! hatVorgegeben( wert ) ) {
            // Der Zauber ist nicht vorgegeben, also hat ihn der Benutzer
            // aktiviert.
            ++aktivierteZauber;
        }

        zauberFehlend.entfernen( wert );

        zauber.einfuegen( wert );
    }

    /**
     * Fuegt dem Helden mehrere Hauszauber hinzu. Der Held erhaelt
     * die Zauber dadurch nicht, sie werden nur als Hauszauber markiert.
     *
     * @see #hatHauszauber(Zauber)
     * @param zauber
     *            die als Hauszauber markiert werden soll.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenHauszauber( final Collection< Zauber > zauber ) {

        if ( null == zauber ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        hauszauber.addAll( zauber );
    }

    /**
     * Fuegt dem Helden ein Hauszauber hinzu. Der Held erhaelt den
     * Zauber dadurch nicht, er wird nur als Hauszauber markiert.
     *
     * @see #hatHauszauber(Zauber)
     * @param zauber
     *            der als Hauszauber markiert werden soll.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenHauszauber( final Zauber zauber ) {

        if ( null == zauber ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        hauszauber.add( zauber );
    }

    /**
     * Fuegt dem Held Leitsonderfertigkeiten hinzu. Der Held erhaelt die
     * Sonderfertigkeiten dadurch nicht, sie werden nur als
     * Leitsonderfertigkeiten * markiert.
     *
     * @see #hatLeitsonderfertigkeit(SonderfertigkeitWert)
     * @param werte
     *            die zu Leitsonderfertigkeiten erklaert werden sollen.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenLeitsonderfertigkeiten(
            final Collection< SonderfertigkeitWert > werte ) {

        for ( final SonderfertigkeitWert wert : werte ) {

            leitsonderfertigkeiten.einfuegen( wert );
        }
    }

    /**
     * Fuegt dem Helden ein Leittalent hinzu. Der Held erhaelt das Talent
     * dadurch nicht, es wird nur als Leittalent markiert. Leittalente werden
     * fuer elfische Helden benoetigt.
     *
     * @see #hatLeittalent(Talent)
     * @param leittalent
     *            das der Held erhalten soll.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenLeittalent( final Talent leittalent ) {

        if ( null == leittalent ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        leittalente.add( leittalent );
    }

    /**
     * Fuegt dem Helden einige Leittalente hinzu. Der Held erhaelt die Talente
     * dadurch nicht, sie werden nur als Leittalente markiert. Leittalente
     * werden fuer elfische Helden benoetigt.
     *
     * @see #hatLeittalent(Talent)
     * @param talente
     *            die als Leittalente markiert werden sollen.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenLeittalente(
            final Collection< Talent > talente ) {

        if ( null == talente ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        leittalente.addAll( talente );
    }

    /**
     * Fuegt dem (elfischen) Helden mehrere Leitzauber hinzu. Der Held erhaelt
     * die Zauber dadurch nicht, sie werden nur als Leitzauber markiert.
     *
     * @see #hatLeitzauber(Zauber)
     * @param zauber
     *            die als Leitzauber markiert werden soll.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenLeitzauber( final Collection< Zauber > zauber ) {

        if ( null == zauber ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        leitzauber.addAll( zauber );
    }

    /**
     * Fuegt dem (elfischen) Helden einen Leitzauber hinzu. Der Held erhaelt den
     * Zauber dadurch nicht, er wird nur als Leitzauber markiert.
     *
     * @see #hatLeitzauber(Zauber)
     * @param zauber
     *            der als Leitzauber markiert werden soll.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenLeitzauber( final Zauber zauber ) {

        if ( null == zauber ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        leitzauber.add( zauber );
    }

    /**
     * Fuegt dem Helden einen Talentbonus hinzu den den er durch seine
     * Profession erhalten hat. Wenn der Held das Talent noch nicht besitzt,
     * dann erhaelt er es neum wenn er es schon besitzt dann wird die
     * unmodifizierte Stufe von {@code wert} zum bestehenden Talentwert addiert.
     *
     * @param wert
     *            den der Held durch seine Profession erhalten hat.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenProfession( final TalentWert wert ) {
        talenteProfession.einfuegen( wert );
        einfuegenVorgegeben( wert );
    }

    /**
     * Fuegt dem Held eine Sonderfertigkeit als verbilligt hinzu. Fuer
     * verbilligte Sonderfertigkeiten muss der Held nur die Haelfte der Kosten
     * zahlen.
     *
     * @see #hatVerbilligt(SonderfertigkeitWert)
     * @param sonderfertigkeit
     *            die verbilligt wird.
     */
    public final void einfuegenVerbilligt(
            final Sonderfertigkeit sonderfertigkeit ) {

        sonderfertigkeitenVerbilligt.add( sonderfertigkeit );
    }

    /**
     * Fuegt dem Held eine Sonderfertigkeit als verbilligt hinzu. Fuer
     * verbilligte Sonderfertigkeiten muss der Held nur die Haelfte der Kosten
     * zahlen.
     *
     * @see #hatVerbilligt(SonderfertigkeitWert)
     * @param wert
     *            der verbilligt wird.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenVerbilligt( final SonderfertigkeitWert wert ) {
        sonderfertigkeitenVerbilligtVarianten.einfuegen( wert );
    }

    /**
     * Eine Profession kann automatisch Verbindungen im Wert von SO x Punkten
     * vergeben. Mit dieser Methode kann dem Helden ein so finanzierter Vorteil
     * hinzugefuegt werden.
     *
     * @see #entfernenVerbindungen(VorNachteilWert)
     * @param wert
     *            den der Held erhalten soll.
     * @throws IllegalArgumentException
     *             wenn es sich beim Wert nicht um den Vorteil Verbindungen
     *             handelt.
     */
    public final void einfuegenVerbindungen(
            final VorNachteilWert wert ) {

        if ( ! wert.getVorNachteil().equals( VorNachteil.VERBINDUNGEN ) ) {

            throw new IllegalArgumentException( String.format(
                    "Es wurd ein Wert vom Typ %1$s uebergeben. " +
                    "Es muss sich jedoch um den Vorteil Verbindungen handeln",
                    wert.getVorNachteil().getName() ) );
        }

        vorNachteileVorgegeben.einfuegen( wert );

        verbindungen.einfuegen( wert );

        if ( ! hat( wert ) ) {

            einfuegen( wert );

        } else {

            final int stufe = wert.getStufe();

            final int aktuelleStufe = getWert( wert ).getStufe();

            if ( aktuelleStufe < stufe ) {

                getWert( wert ).setStufe( stufe );
            }
        }
    }

    /**
     * Fuegt dem Helden eine Sonderfertigkeit, als durch Rasse, Kultur oder
     * Profession vorgegeben, hinzu. Wenn der Held den Wert schon besitzt macht
     * die Methode nichts.
     *
     * @param wert
     *            den der Held erhalten soll.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenVorgegeben( final SonderfertigkeitWert wert ) {
        sonderfertigkeitenVorgegeben.einfuegen( wert );
        einfuegen( wert );
    }

    /**
     * Fuegt dem Helden einen Talentbonus, als durch Rasse, Kultur oder
     * Profession vorgegeben, hinzu. Auch Basis-Talent werden auf diese Weise
     * als vorgegeben markiert. Die Stufe des Talents steigt um die
     * unmodifizierte Stufe von {@code wert}.
     *
     * @see #hatVorgegeben(Talent)
     * @param wert
     *            den der Held erhalten soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenVorgegeben( final TalentWert wert ) {
        talenteVorgegeben.einfuegen( wert );
        einfuegen( wert );
    }

    /**
     * Fuegt dem Helden einen Vor-/Nachteil, als durch Rasse, Kultur oder
     * Profession vorgegeben, hinzu.
     *
     * @see #hatVorgegeben(VorNachteilWert)
     * @param wert
     *            den der Held erhalten soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenVorgegeben( final VorNachteilWert wert ) {
        vorNachteileVorgegeben.einfuegen( wert );
        einfuegen( wert );
    }

    /**
     * Fuegt dem Helden einen Zauber, als durch Rasse, Kultur oder
     * Profession vorgegeben, hinzu. Die Stufe des Zaubers steigt um die
     * unmodifizierte Stufe von {@code wert}.
     *
     * @see #hatVorgegeben(Zauber)
     * @param wert
     *            den der Held erhalten soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void einfuegenVorgegeben( final ZauberWert wert ) {
        zauberVorgegeben.einfuegen( wert );
        einfuegen( wert );
    }

    /**
     * Entfernt eine Waffe vom Helden.
     *
     * @param waffe
     *            die vom Helden entfernt werden soll.
     */
    public final void entfernen( final FernkampfwaffeWert waffe ) {

        fernkampfwaffen.remove( waffe );
    }

    /**
     * Entfernt eine Waffe vom Helden.
     *
     * @param waffe
     *            die vom Helden entfernt werden soll.
     */
    public final void entfernen( final NahkampfwaffeWert waffe ) {

        nahkampfwaffen.remove( waffe );
    }

    /**
     * Entfernt ein Ruestungsteil vom Helden.
     *
     * @param ruestung
     *            die entfernt werden soll.
     */
    public final void entfernen( final RuestungWert ruestung ) {

        if (  ruestungen.remove( ruestung ) )  {

            notifyObservers( RuestungWert.BE );

            ruestung.setHeld( null );
        }
    }

    /**
     * Entfernt einen Schild vom Helden.
     *
     * @param schild
     *            der entfernt werden soll.
     */
    public final void entfernen( final SchildWert schild ) {

        schilde.remove( schild );
    }

    /**
     * Entfernt eine Sonderfertigkeit vom Helden. Wenn der Held die
     * Sonderfertigkeit nicht besitzt macht die Methode nichts. Evtl. AP die
     * dabei frei werden {@link SonderfertigkeitWert#getLernkosten()}, werden
     * dem Helden berechnet. Die Methode kann bei der Erschaffung und beim
     * Steigern verwendet werden.
     *
     * @param wert
     *            der entfernt werden soll.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final void entfernen( final SonderfertigkeitWert wert ) {

        if ( hat( wert ) ) {

            final Sonderfertigkeit sonderfertigkeit = wert.getSonderfertigkeit();

            if ( Sonderfertigkeit.ORTSKENNTNIS == sonderfertigkeit ) {

                ortskenntnisse.remove( wert );

            } else if ( sonderfertigkeit.istGelaendekunde() ) {

                gelaendekunden.remove( wert );

            }

            if ( hatVorgegeben( wert ) ) {

                sonderfertigkeitenFehlend.einfuegen( getVorgegeben( wert ) );

            }

            apEingesetzt -= getWert( wert ).getLernkosten();

            sonderfertigkeiten.entfernen( wert );
        }
    }

    /**
     * Entfernt ein Talent vom Helden. Es besitzt ggf noch immer seinen Status
     * als vorgegebenes Talent. Diese Methode ist fuer die Heldenerschaffung
     * gedacht. Fuer das Steigern gibt es extra Methoden.
     *
     * @see #deaktivieren(Talent)
     * @param talent
     *            das entfernt werden soll.
     */
    public final void entfernen( final Talent talent ) {

        // Das Runterzaehlen muss vor dem Entfernen geschehen,
        // denn beim Entfernen werden die Observer benachrichtigt und die
        // rufen dann noch den alten Wert ab.
        if ( ! hatVorgegeben( talent ) ) {
            // Das Talent war nicht vorgegeben also hat es der Benutzer
            // aktiviert.
            --aktivierteTalente;
        }

        talente.entfernen( talent );

        if ( hatVorgegeben( talent ) ) {
            talenteFehlend.add( talent );
        }
    }

    /**
     * Enternt einen Vor-/Nachteil vom Helden. Wenn der Held den Wert nicht
     * besitzt, so macht die Methode nichts.
     *
     * @param wert
     *            der entfernt werden soll.
     */
    public final void entfernen( final VorNachteilWert wert ) {

        if ( hat( wert ) ) {

            final VorNachteilWert entfernterWert =
                vorNachteile.entfernen( wert );

            if ( hatVorgegeben( wert ) ) {

                final VorNachteilWert vorgegeben = getVorgegeben( wert );

                entfernterWert.setVarianteAenderbar(
                        vorgegeben.istVarianteAenderbar() );

                vorNachteileFehlend.einfuegen( entfernterWert );
            }

            entfernterWert.entfernt( this );

            notifyObservers( entfernterWert.getVorNachteil() );
        }
    }

    /**
     * Entfernt einen Zauber vom Helden. Es besitzt ggf noch immer seinen Status
     * als vorgegebenen Zauber.
     *
     * @param wert
     *            das entfernt werden soll.
     */
    public final void entfernen( final ZauberWert wert ) {

        // Das Runterzaehlen muss vor dem Entfernen geschehen,
        // denn beim Entfernen werden die Observer benachrichtigt und die
        // rufen dann noch den alten Wert ab.
        if ( ! hatVorgegeben( wert ) ) {
            // Der Zauber war nicht vorgegeben also hat ihn der Benutzer
            // aktiviert.
            --aktivierteZauber;

        } else {

            zauberFehlend.einfuegen( wert );

        }

        zauber.entfernen( wert );

    }

    /**
     * Entfernt den Bonus den ein Held durch die Profession auf ein Talent
     * erhalten hat. Das Talent gilt danach nicht mehr als durch die Profession
     * vorgegeben und ggf. als ueberhaupt nicht mehr vorgegeben.
     * Wenn der Held das Talent nur besitzt, weil er einen Bonus durch die
     * Profession darauf erhalten hat, so verliert er es komplett, auch wenn
     * das Talent durch den Benutzer gesteigert worden sein sollte.
     *
     * @param talent
     *            dessen Bonus entfernt werden soll.
     * @throws IllegalArgumentException
     *             wenn das Talent nicht durch die Profession vorgegeben ist.
     */
    public final void entfernenProfession( final Talent talent ) {

        if ( ! hatVorgegebenProf( talent ) ) {

            throw new IllegalArgumentException( String.format(
                    "Das Talent %1$s hat keinen Bonus durch die Profession " +
                    "erhalten.",
                    talent.getName() ) );
        }

        final int bonus;
        bonus = talenteProfession.getWert( talent ).getUnmodifizierteStufe();

        talenteProfession.entfernen( talent );

        // Der Bonus den der Held durch die Profession erhalten hat muss vom
        // Talent abgezogen werden.
        final TalentWert wert = new TalentWert( talent, -bonus );

        talenteVorgegeben.einfuegen( wert );
        talente.einfuegen( wert );

        final int vorgegebeneStufe =
            talenteVorgegeben.getWert( talent ).getUnmodifizierteStufe();

        if ( ! talent.istBasisTalent()
                && vorgegebeneStufe == 0 ) {
            // Das Talent gilt nur durch die Profession als vorgegeben.
            // Jetzt wo der Professions-Bonus weg ist ist es nicht laenger
            // vorgegeben.
            talenteVorgegeben.entfernen( talent );
            talenteFehlend.remove( talent );
            talente.entfernen( talent );
        }
    }

    /**
     * Entfernt einen Vorteil Verbindungen vom Helden, der mit Punkten
     * finanziert wurden, die man automatisch durch die Profession erhaelt.
     * Nach Aufruf der Methode besitzt der Held den Vorteil nicht mehr.
     *
     * @see #getVerbindungen(VorNachteilWert)
     * @see #hatVerbindungen(VorNachteilWert)
     * @see #setVerbindungenStufe(VorNachteilWert)
     * @param wert
     *            Die Verbindungen die vom Helden entfernt werden sollen.
     * @throws IllegalArgumentException
     *             wenn es sich nicht um Verbindunge handelt die ueber die
     *             Punkte der Profession finanziert wurden.
     */
    public final void entfernenVerbindungen(
            final VorNachteilWert wert ) {

        if ( ! verbindungen.enthaelt( wert ) ) {

            throw new IllegalArgumentException( String.format(
                    "Bei %1$s handelt es sich nicht um Verbindungen die " +
                    "ueber die Profession finanziert wurden.",
                    wert.getName() ) );
        }

        vorNachteileVorgegeben.entfernen( wert );

        verbindungen.entfernen( wert );

        entfernen( wert );
    }

    /**
     * Entfernt eine durch Rasse, Kultur oder Profession vorgegebene
     * Sonderfertigkeit vom Helden.
     *
     * @param sonderfertigkeit
     *            die nicht laenger als vorgegeben gelten soll.
     * @throws IllegalArgumentException
     *             wenn die Sonderfertigkeit nicht vorgegeben ist.
     */
    public final void entfernenVorgegeben(
            final Sonderfertigkeit sonderfertigkeit ) {

        if ( ! sonderfertigkeitenVorgegeben.enthaelt( sonderfertigkeit ) ) {

            throw new IllegalArgumentException( String.format(
                    "Die Sonderfertigkeit %1$s ist nicht vorgegeben",
                    sonderfertigkeit.getName() ) );
        }

        sonderfertigkeitenVorgegeben.entfernen( sonderfertigkeit );

        if ( Sonderfertigkeit.ORTSKENNTNIS == sonderfertigkeit ) {

            ortskenntnisse.clear();

        } else if ( sonderfertigkeit.istGelaendekunde() ) {

            gelaendekunden.clear();

        }

        sonderfertigkeiten.entfernen( sonderfertigkeit );
    }

    /**
     * Entfernt ein Talent vom Helden und nimmt ihm den Status als vorgegeben,
     * ausser es handelt sich um ein Basis-Talent. Bei einem Basis-Talent wird
     * die Stufe des Talents um den vorgegebenen Bonus gesenkt.
     *
     * @param talent
     *            das nicht laenger vorgegeben sein soll.
     * @throws IllegalArgumentException
     *             wenn das Talent kein vorgegebenes Talent ist.
     */
    public final void entfernenVorgegeben( final Talent talent ) {

        if ( ! hatVorgegeben( talent ) ) {

            throw new IllegalArgumentException( String.format(
                    "Das Talent %1$s ist kein vorgegebenes Talent.",
                    talent.getName() ) );
        }

        final int bonus;
        bonus = talenteVorgegeben.getWert( talent ).getUnmodifizierteStufe();

        talenteVorgegeben.entfernen( talent );
        talenteProfession.entfernen( talent );

        if ( ! talent.istBasisTalent() ) {
            // Es handelt sich um ein Spezial-Talent, das der Held nur besitzt,
            // weil es vorgegeben ist.
            talente.entfernen( talent );
        } else {

            talente.einfuegen( new TalentWert( talent, -bonus ) );
        }
    }

    /**
     * Entfernt einen durch Rasse, Kultur oder Profession vorgegebenen
     * Vor-/Nachteil vom Helden. Nach Aufruf der Methode besitzt der Held den
     * Vor-/Nachteil nicht mehr und er gilt nicht mehr als vorgegeben.
     *
     * @param vorNachteil
     *            der nicht laenger als vorgegeben gelten soll.
     * @throws IllegalArgumentException
     *             wenn der Vor-/Nachteil nicht vorgegeben ist.
     */
    public final void entfernenVorgegeben(
            final VorNachteil vorNachteil ) {

        if ( ! vorNachteileVorgegeben.enthaelt( vorNachteil ) ) {

            throw new IllegalArgumentException( String.format(
                    "Der Vor-/Nachteil %1$s ist nicht vorgegeben",
                    vorNachteil.getName() ) );
        }

        vorNachteileVorgegeben.entfernen( vorNachteil );

        vorNachteile.entfernen( vorNachteil );
    }

    /**
     * Entfernt einen Zauber vom Helden und nimmt ihm den Status als
     * vorgegeben.
     *
     * @param wert
     *            Der Zauber der nicht laenger vorgegeben sein soll.
     * @throws IllegalArgumentException
     *             wenn der Zauber nicht vorgegeben ist.
     */
    public final void entfernenVorgegeben( final ZauberWert wert ) {

        if ( ! hatVorgegeben( wert ) ) {

            throw new IllegalArgumentException( String.format(
                    "Der Zauber %1$s ist nicht vorgegeben.",
                    wert.getZauber().getName() ) );
        }

        zauberVorgegeben.entfernen( wert );
        zauberFehlend.entfernen( wert );
        zauber.entfernen( wert );
    }

    /**
     * @return das Alter des Helden in Jahren.
     */
    public final int getAlter() {
        return alter;
    }

    /**
     * @return die Anzahl der Talente die vom Benutzer aktivert wurden.
     */
    public final int getAnzahlAktivierteTalente() {
        return aktivierteTalente;
    }

    /**
     * @return die Anzahl der Zauber die vom Benutzer aktivert wurden.
     */
    public final int getAnzahlAktivierteZauber() {
        return aktivierteZauber;
    }

    /**
     * Liefer die zum Steigern eingesetzten AP. Diese Methode kann nicht
     * waehrend der Heldenerschaffung eingesetzt werden.
     *
     * @return die zum Steigern verwendeten AP.
     */
    public final int getApEingesetzt() {
        return apEingesetzt;
    }

    /**
     * Liefert die AP, die der Held insgesamt erhalten hat. Diese Methode kann
     * nicht waehrend der Heldenerschaffung verwendet werden.
     *
     * @return die Gesamtzahl an AP, die der Held bisher erhalten hat.
     */
    public final int getApGesamt() {
        return apGesamt;
    }

    /**
     * Liefert die AP die der Held noch zum Steigern verwenden kann. Diese
     * Methode kann nicht waehrend der Heldenerschaffung verwendet werden.
     *
     * @return die Anzahl AP, die dem Held zum Steigern noch zur Verfuegung
     *         stehen.
     */
    public final int getApGuthaben() {
        return ( apGesamt - apEingesetzt );
    }

    /**
     * @return die Attacke in Raufen.
     */
    public final int getAttackeRaufen() {
        return getAttackeWaffenlos(
                Talent.RAUFEN,
                Sonderfertigkeit.HAMMERFAUST );
    }

    /**
     * @return die Attacke in Ringen.
     */
    public final int getAttackeRingen() {
        return getAttackeWaffenlos(
                Talent.RINGEN,
                Sonderfertigkeit.UNAUER_SCHULE );
    }

    /**
     *
     * @param talent eine waffenlose Kampftechnik.
     * @param kampfstil ein waffenloser Kampfstil.
     * @return die Attacke in einer Waffenlosen Kampftechnik.
     */
    private int getAttackeWaffenlos(
            final Talent talent,
            final Sonderfertigkeit kampfstil ) {

        int at = getStufe( Basiswert.ATB );

        at += getWert( talent ).getAttacke();

        if ( hat( kampfstil ) ) {
           ++at;
        }

        int be = getBehinderung();

        be /= 2;

        at -= be;

        return at;
    }

    /** @return die Augenfarbe des Helden. */
    public final Augenfarbe getAugenfarbe() {
        return augenfarbe;
    }

    /** @return eine kurze Beschreibung des Aussehns des Helden. */
    public final String getAussehen() {
        return aussehen;
    }

    /** @return den Ausweichenwert des Helden. */
    public final int getAusweichen() {

        int ausweichen = getStufe( Basiswert.PAB );

        for ( final Sonderfertigkeit s : new Sonderfertigkeit[] {
                Sonderfertigkeit.AUSWEICHEN_I,
                Sonderfertigkeit.AUSWEICHEN_II,
                Sonderfertigkeit.AUSWEICHEN_III
        } ) {

            if ( hat( s ) ) {

                ausweichen += 3;
            }
        }

        ausweichen -= getBehinderung();

        return ausweichen;
    }

    /**
     * @return die Behinderung des Helden, entsprechend der getragenen
     *         Ruestungen.
     */
    public final int getBehinderung() {

        int be = 0;

        final SonderfertigkeitWert ruestgewI = new SonderfertigkeitWert( 
                Sonderfertigkeit.RUESTUNGSGEWOEHNUNG_I,
                new Variante() );

        // Wirkt einen Ruestungsgewoehnung oder nicht?
        boolean ruestungsgewoehnung = false;

        for ( final RuestungWert ruestung : ruestungen ) {

            be += ruestung.getBe();

            ruestgewI.setVariante( ruestung.getRuestung() );

            if ( hat( ruestgewI ) ) {

                ruestungsgewoehnung = true;
            }
        }

        if ( ruestungsgewoehnung
          || hat( Sonderfertigkeit.RUESTUNGSGEWOEHUNG_II ) ) {

            --be;
        }

        return be;
    }

    /**
     * @return die Sonderfertigkeiten die dem Held durch Rasse, Kultur und
     *         Profession vorgegeben sind, die er aber nicht besitzt (weil vom
     *         Benutzer abgewaehlt).
     */
    public final
    List< SonderfertigkeitWert > getFehlendeSonderfertigkeiten() {
        return sonderfertigkeitenFehlend.getWerte();
    }

    /**
     * @return die Talent die der Held eigentlich besitzen muss, weil es sich
     *         um Basis-Talente oder vorgegebene Talente handelt, die aber vom
     *         Benutzer deaktiviert wurden.
     */
    public final List< Talent > getFehlendeTalente() {
        return new ArrayList< Talent >( talenteFehlend );
    }

    /**
     * @return die Vor-/Nachteile die durch Kultur oder Profession vorgegeben
     *         sind, die er aber nicht besitzt (weil vom Benutzer abgewaehlt).
     */
    public final List< VorNachteilWert > getFehlendeVorNachteile() {
        return vorNachteileFehlend.getWerte();
    }

    /**
     * @return die Zauber die der Held eigentlich besitzen muss, weil es sich um
     *         vorgegebene Zauber handelt, die aber vom Benutzer deaktiviert
     *         wurden.
     */
    public final List< ZauberWert > getFehlendeZauber() {
        return zauberFehlend.getZauber();
    }

    /**
     * @return die Fernkampfwaffen des Helden.
     */
    public final List< FernkampfwaffeWert > getFernkampfwaffen() {
        return new ArrayList< FernkampfwaffeWert >( fernkampfwaffen );
    }

    /** @return der Geburstag des Helden. */
    public final String getGeburtstag() {
        return geburtstag;
    }

    /** @return das Geschlecht des Helden/der Heldin. */
    public final Geschlecht getGeschlecht() {
        return geschlecht;
    }

    /**
     * @return die Geschwindigkeit des Helden in Metern pro Sekunde.
     */
    public final int getGeschwindigkeit() {

        int gs;

        final int ge = getStufe( Eigenschaft.GE );

        if ( 10 >= ge ) {

            gs = 7;

        } else if ( 16 <= ge ) {

            gs = 9;

        } else {

            gs = 8;

        }

        if ( hat( VorNachteil.ZWERGENWUCHS ) ) {

            gs -= 2;
        }

        if ( hat( VorNachteil.EINBEINIG ) ) {

            gs -= 3;
        }

        if ( hat( VorNachteil.LAHM ) ) {

            gs -= 1;
        }

        if ( hat( VorNachteil.FLINK ) ) {

            gs += 1;
        }

        return gs;
    }

    /** @return das Gewicht des Helden in Stein bzw. kg. */
    public final int getGewicht() {
        return gewichtInKg;
    }

    /**
     * @return die GP, die der Held durch Nachteile und schlechte Eigenschaften
     *         erhalten hat.
     */
    public final GpNachteile getGpNachteile() {

        int nachteile = 0;

        int schlechteEigenschaften = 0;

        for ( final VorNachteilWert wert : getVorNachteile() ) {

            final VorNachteil vorNachteil = wert.getVorNachteil();

            if ( ! vorNachteil.istVorteil() ) {

                final double kosten = wert.getGpKosten( this );

                if ( kosten < 0 ) {

                    nachteile += kosten;

                    if ( vorNachteil.istSchlechteEigenschaft() ) {

                        schlechteEigenschaften += kosten;
                    }
                }
            }
        }

        nachteile *= -1;

        schlechteEigenschaften *= -1;

        return new GpNachteile( nachteile,
                                schlechteEigenschaften );
    }

    /** @return die Koerpergroesse des Helden in Halbfingern bzw. cm. */
    public final int getGroesse() {
        return groesseInCm;
    }

    /** @return die Haarfarbe des Helden. */
    public final Haarfarbe getHaarfarbe() {
        return haarfarbe;
    }

    /** @return die Hauszauber des Helden. */
    public Collection< Zauber > getHauszauber() {
        return new ArrayList< Zauber >( hauszauber );
    }

    /**
     * @return Informationen zu Familie, Herkunft und Hintergrund des Helden.
     */
    public final String getHintergrund() {
        return hintergrund;
    }

    /**
     * Liefert den Index der angibt die wievielte Variante der Sonderfertigkeit
     * der Wert fuer den Helden ist. Hat der Held den Wert schon gibt der Index
     * an als wievielten er ihn erhalten hat. Hat der Held den Wert noch nicht,
     * so gibt der Index an die wievielte Variante der Sonderfertigkeit der Wert
     * fuer den Helden waere.
     * <dl>
     * <dt>Beispiel:</dt>
     * <dd> Hat der Held erst Talentspezialisierung Klettern (Freiklettern) und
     * dann Talentspezialisierung Klettern (Bergsteigen) erhalten hat die
     * erstere den Index 1 und die zweitere den Index 2. Wird die Methode jetzt
     * nach dem Index von Talentspezialisierung Klettern (Baeume) gefragt, so
     * liefert sie 3. </dd>
     * </dl>
     *
     * @param wert
     *            dessen Index ermittelt werden soll.
     * @return den Index des Wertes. Der Index wird von 1 an gezaehlt.
     */
    public final int getIndex( final SonderfertigkeitWert wert ) {
        return sonderfertigkeiten.getIndex( wert );
    }

    /**
     *
     * @return
     */
    public final int getInitiative() {

        int ini = getStufe( Basiswert.INI );

        if ( hat( Sonderfertigkeit.KAMPFREFLEXE ) ) {

            ini += 4;
        }

        ini -= getBehinderung();

        return ini;
    }

    /** @return die Kultur des Helden. */
    public final Kultur getKultur() {
        return kultur;
    }

    /**
     * Liefert eine Leitsonderfertigkeit.
     *
     * @param wert der gesucht wird.
     * @return den Wert der die Leitsonderfertigkeit repraesentiert.
     *         Anderungen am Wert wirken sich auf den Helden aus.
     */
    public final SonderfertigkeitWert getLeitsonderfertigkeit(
            final SonderfertigkeitWert wert ) {

        return leitsonderfertigkeiten.getWert( wert );
    }

    /**
     * @return die Leitsonderfertigkeiten des Helden. Aenderungen an den Werte
     *         wirken sich auf den Helden aus.
     */
    public final Collection< SonderfertigkeitWert> getLeitsonderfertigkeiten() {
        return leitsonderfertigkeiten.getWerte();
    }

    /**
     * @return alle Leittalente des Helden.
     */
    public final Collection< Talent > getLeittalente() {
        return new ArrayList< Talent >( leittalente );
    }

    /**
     * @return alle Leitzauber des Helden.
     */
    public final Collection< Zauber > getLeitzauber() {
        return new ArrayList< Zauber >( leitzauber );
    }

    /**
     * Gibt die Stufe in einem Talent an die ein Held mindestens habe muss.
     * Eine solche Mindeststufe kann z.B. durch einen Bonus durch die Kultur
     * oder einen Vorteil entstehen.
     *
     * @param talent
     *            dessen Mindeststufe gesucht ist.
     * @return die durch Rasse, Kultur und Profession und Vor-/Nachteile
     *         vorgegebene Mindesstufe Stufe im Talent.
     */
    public final int getMindeststufe( final Talent talent ) {

        if ( hatVorgegeben( talent ) ) {

            return talenteVorgegeben.getWert( talent ).getStufe( this );

        } else {

            return talent.getModifikation( this );

        }
    }

    /**
     * Gibt die Stufe in einem Zauber an die ein Held mindestens habe muss.
     * Eine solche Mindeststufe entsteht durch Boni aus Rasse, Kultur und
     * Profession
     *
     * @see #hatVorgegeben(Zauber)
     * @param wert
     *            dessen Mindeststufe gesucht ist.
     * @return die durch Rasse, Kultur und Profession und Vor-/Nachteile
     *         vorgegebene Mindesstufe Stufe im Talent.
     * @throws IllegalArgumentException
     *             wenn der Zauber nicht vorgegeben ist.
     */
    public final int getMindeststufe( final ZauberWert wert ) {

        if ( ! hatVorgegeben( wert ) ) {

            throw new IllegalArgumentException( String.format(
                    "Der Zauber %1$s ist nicht vorgegeben", wert ));
        }

        return getVorgegeben( wert ).getStufe();
    }

    /**
     * @return die Muttersprache des Helden, oder {@code null}, wenn der Held
     *         noch keine Muttersprache besitzt.
     */
    public final Sprache getMuttersprache() {
        return muttersprache;
    }

    /**
     * @return die Nahkampfwaffen des Helden.
     */
    public final List< NahkampfwaffeWert > getNahkampfwaffen() {
        return new ArrayList< NahkampfwaffeWert >( nahkampfwaffen );
    }

    /** @return der Name des Helden. */
    public final String getName() {
        return name;
    }

    /**
     * @return die Parade in Raufen.
     */
    public final int getParadeRaufen() {
        return getParadeWaffenlos(
                Talent.RAUFEN,
                Sonderfertigkeit.HAMMERFAUST );
    }

    /**
     * @return die Parade in Ringen.
     */
    public final int getParadeRingen() {
        return getParadeWaffenlos(
                Talent.RINGEN,
                Sonderfertigkeit.UNAUER_SCHULE );
    }

    /**
     *
     * @param talent eine waffenlose Kampftechnik.
     * @param kampfstil ein waffenloser Kampfstil.
     * @return die Parade in einer Waffenlosen Kampftechnik.
     */
    private int getParadeWaffenlos(
            final Talent talent,
            final Sonderfertigkeit kampfstil ) {

        int pa = getStufe( Basiswert.PAB );

        pa += getWert( talent ).getParade( this );

        if ( hat( kampfstil ) ) {
           ++pa;
        }

        int be = getBehinderung();

        be = ( be + 1 ) / 2; // +1 zum Aufrunden.

        pa -= be;

        return pa;
    }

    /** @return die Profession des Helden. */
    public final Profession getProfession() {
        return profession;
    }

    /** @return die Rasse des Helden. */
    public final Rasse getRasse() {
        return rasse;
    }

    /** Die Ruestungen des Helden. */
    public final List< RuestungWert > getRuestungen() {
        return new ArrayList< RuestungWert >( ruestungen );
    }

    /**
     * @return den Ruestungsschutz des Helden.
     */
    public final int getRuestungsschutz() {

        int rsTorso = 0;

        for ( final RuestungWert ruestung : ruestungen ) {

            if ( ! ruestung.getRuestung().istZusatzruestung() ) {

                rsTorso += ruestung.getRs();
            }
        }

        int rsZusatz = 0;

        for ( final RuestungWert ruestung : ruestungen ) {

            final Ruestung typ = ruestung.getRuestung();

            if ( typ.istZusatzruestung()
              && ( typ.getMindestTorsoRs() <= rsTorso ) ) {

                rsZusatz += ruestung.getRs();
            }
        }

        return rsTorso + rsZusatz;
    }

    /**
     * @return die Schilde des Helden.
     */
    public final List< SchildWert > getSchilde() {
        return new ArrayList< SchildWert >( schilde );
    }

    /**
     * @return alle Sonderfertigkeiten die der Held besitzt. Es handelt sich um
     *         die Objekte die im Helden gespeichert sind. Jede Veraenderung an
     *         ihnen wirkt sich direkt auf den Helden aus. Die Liste selbst ist
     *         unabhaengig vom Helden und kann ohne Seiteneffekte veraendert
     *         werden.
     */
    public final List< SonderfertigkeitWert > getSonderfertigkeiten() {
        return sonderfertigkeiten.getWerte();
    }

    /** @return Informationene ueber den Stand des Helden. */
    public final String getStand() {
        return stand;
    }

    /**
     * @param basiswert
     *            dessen Stufe gesucht ist.
     * @return die Stufe des Helden in dem Basiswert.
     */
    public final int getStufe( final Basiswert basiswert ) {
       return getWert( basiswert ).getStufe();
    }

    /**
     * @param eigenschaft
     *            deren Stufe gesucht ist.
     * @return die Stufe des Helden in der Eigenschaft.
     */
    public final int getStufe( final Eigenschaft eigenschaft ) {
        return getWert( eigenschaft ).getStufe();
    }

    /**
     * @see #hat(Talent)
     * @param talent
     *            dessen Stufe gefragt ist.
     * @return die Stufe des Talents, so der Held es besitzt.
     * @throws NullPointerException
     *             wenn der Held das Talent nicht besitzt.
     */
    public final int getStufe( final Talent talent ) {
        return talente.getWert( talent ).getStufe( this );
    }

    /**
     * @return eine Liste aller Talente die der Held besitzt. Die Liste enthaelt
     *         auch Spezial-Talente mit negativer Stufe.
     */
    public final List< Talent > getTalente() {
        return talente.getTalente();
    }

    /**
     * @param talentgruppe
     *            deren Talente gesucht sind.
     * @return alle Talente der uebergebenen Talentgruppe, die der Held besitzt.
     *         Die Liste enthaelt auch Spezial-Talente mit negativer Stufe.
     */
    public final List< Talent > getTalente(
            final Talentgruppe talentgruppe ) {

        return talente.get( talentgruppe );
    }

    /** @return den oder die Titel des Helden. */
    public final String getTitel() {
        return titel;
    }

    /**
     * @see #getVerbilligteVarianten()
     * @return eine Liste der Sonderfertigkeiten von denen der Held alle
     *         Varianten verbilligt erwerben kann (so sie Varianten haben). Es
     *         gibt dann noch die Sonderfertigkeiten die nur in Varianten
     *         verbilligt sind, dies koenne ueber
     *         {@link #getVerbilligteVarianten()} abgerufen werden.
     */
    public final List< Sonderfertigkeit > getVerbilligteSonderfertigkeiten() {
        return new ArrayList< Sonderfertigkeit >(sonderfertigkeitenVerbilligt);
    }

    /**
     * @return das Objekt das die Informationen ueber die verbilligte Variante
     *         speichert. Wenn man die Variante des Objekts aendert gilt diese
     *         andere Variante fuer den Helden als verbilligt.
     *         <p>
     *         Die Methode liefert {@code null}, wenn {@code wert} nicht
     *         verbilligt wird.
     */
    public final SonderfertigkeitWert getVerbilligteVariante(
            final SonderfertigkeitWert wert ) {
        return sonderfertigkeitenVerbilligtVarianten.getWert( wert );
    }

    /**
     * @see #getVerbilligteSonderfertigkeiten()
     * @return eine Liste von Sonderfertigkeiten von denen der Held bestimmte
     *         Varianten verbilligt erlernen kann.
     */
    public final List< SonderfertigkeitWert > getVerbilligteVarianten() {
        return sonderfertigkeitenVerbilligtVarianten.getWerte();
    }

    /**
     * Liefert einen Vorteil Verbindungen der mit den x Punkten, die die
     * Profession automatisch vergibt, finanziert wurde. Die Stufe des Wertes
     * gibt an wie viele Punkte aus den x Punkten auf den Wert verteilt wurden.
     * <strong>Der Wert darf nicht manipuliert werden, dazu gib es spezielle
     * Methoden wie {@link #setVerbindungenStufe(VorNachteilWert)} </strong>
     *
     * @see #entfernenVerbindungen(VorNachteilWert)
     * @see #hatVerbindungen(VorNachteilWert)
     * @see #setVerbindungenStufe(VorNachteilWert)
     * @param wert
     *            fuer den festgestellt werden so wie viele der Punkte aus der
     *            Profession auf ihn verteilt wurden.
     * @return den gesuchten Wert oder {@code null}, wenn es den Wert nicht
     *         gibt.
     */
    public final VorNachteilWert getVerbindungen(
            final VorNachteilWert wert ) {

        return verbindungen.getWert( wert );
    }

    /**
     * @return die Anzahl der Punkte die auf die Eigenschaften verteilt worden
     *         sind. SO nicht mitgerechnet.
     */
    public final int getVerteilteEigenschaftspunkte() {

        int verteilt = 0;

        for ( final Eigenschaft eigenschaft : Eigenschaft.getEigenschaften() ){

            verteilt += getWert( eigenschaft ).getBasisstufe();
        }

        return verteilt;
    }

    /**
     * Liefert den vorgegebenen Wert des Helden.
     *
     * @param wert
     *            der gesucht ist.
     * @return den vorgegebenen Wert des Helden. Es handelt sich um das Objekt
     *         das die Daten ueber den vorgegebenen Wert des Helden speichert.
     *         Aenderungen an ihm veraendern den vorgegebenen Wert des Helden.
     *         <strong>Diese Veraenderungen werden aber nicht automatisch an die
     *         Observer gemeldet.</strong>
     */
    public final SonderfertigkeitWert getVorgegeben(
            final SonderfertigkeitWert wert ) {

        return sonderfertigkeitenVorgegeben.getWert( wert );
    }

    /**
     * Liefert den vorgegebenen Wert eines Talents.
     *
     * @param talent
     *            dessen vorgegebener Wert gesucht ist.
     * @return den vorgegebenen Wert des Helden. Es handelt sich um das Objekt,
     *         das im Held gespeichert ist. Aenderungen an ihm wirken sich auf
     *         den vorgegebenen Talentwert des Helden aus.
     */
    public final TalentWert getVorgegeben( final Talent talent ) {

        return talenteVorgegeben.getWert( talent );
    }

    /**
     * Liefert den vorgegebenen Wert des Helden.
     *
     * @param wert
     *            der gesucht ist.
     * @return den vorgegebenen Wert des Helden. Es handelt sich um das Objekt
     *         das die Daten ueber den vorgegebenen Wert des Helden speichert.
     *         Aenderungen an ihm veraendern den vorgegebenen Wert des Helden.
     *         <strong>Diese Veraenderungen werden aber nicht automatisch an die
     *         Observer gemeldet.</strong>
     */
    public final VorNachteilWert getVorgegeben(
            final VorNachteilWert wert ) {

        return vorNachteileVorgegeben.getWert( wert );
    }

    /**
     * Liefert die vorgegebenen Werte des Helden.
     *
     * @param z
     *            zu dem die vorgegebenen Werte gesucht sind.
     * @return die vorgegegbenen Werte des Helden. Es handelt sich um die
     *         Objekte, die im Helden gespeichert sind. Aenderungen an ihnen
     *         veraendern die vorgegebenen Werte des Helden.
     */
    public final List< ZauberWert > getVorgegeben( final Zauber z ) {

        if ( zauberFehlend.enthaelt( z ) )
            return zauberFehlend.getWerte( z );
        else
            return zauberVorgegeben.getWerte( z );
    }

    /**
     * Liefert den vorgegebenen Wert des Helden.
     *
     * @param wert
     *            zu dem der vorgegebene Wert gesucht ist.
     * @return den vorgegegbenen Wert des Helden. Es handelt sich um das Objekt,
     *         das im Helden gespeichert ist. Aenderungen an ihm veraendern den
     *         vorgegebenen Wert des Helden.
     */
    public final ZauberWert getVorgegeben( final ZauberWert wert ) {

        return zauberVorgegeben.getWert( wert );
    }

    /**
     * @see #hatVorgegebenProf(Talent)
     * @param talent
     *            dessen vorgegebener Wert gesucht ist.
     * @return den durch die Profession vorgegeben Wert.
     */
    public final TalentWert getVorgegebenProf( final Talent talent ) {
        return talenteProfession.getWert( talent );
    }

    /**
     * @return alle Vor-/Nachteile die der Held besitzt. Es handelt sich um die
     *         Objekte die im Helden gespeichert sind. Aenderungen an ihnen
     *         wirken sich direkt auf die Werte des Helden aus.
     */
    public final List< VorNachteilWert > getVorNachteile() {
        return vorNachteile.getWerte();
    }

    /**
     * @return den Wert des Basiswert. Es handelt sich um das Objekt, das den
     *         Wert des Helden darstellt. Aenderungen am Wert wirken sich direkt
     *         auf dem Helden aus. Der Werte ist dem Helden zugeordnet.
     *         Aenderungen an ihm werden ueber {@link #notifyObservers(Object)}
     *         gemeldet.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final BasiswertWert getWert( final Basiswert basiswert ) {
        return basiswerte.get( basiswert.getIndex() );
    }

    /**
     * @return den Wert der Eigenschaft. Es handelt sich um das Objekt, das den
     *         Wert des Helden darstellt. Aenderungen am Wert wirken sich direkt
     *         auf dem Helden aus. Der Werte ist dem Helden zugeordnet.
     *         Aenderungen an ihm werden ueber {@link #notifyObservers(Object)}
     *         gemeldet.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final EigenschaftWert getWert( final Eigenschaft eigenschaft ) {
        return eigenschaften.get( eigenschaft.getIndex() );
    }

    /**
     * Liefert den Wert eines Helden.
     *
     * @param wert
     *            der gesucht ist.
     * @return den Wert des Helden. {@code null}, wenn der Held den Wert nicht
     *         besitzt. Aenderungen am Wert wirken sich direkt auf den Helden
     *         aus. Der Werte ist dem Helden zugeordnet. Aenderungen an
     *         ihm werden ueber {@link #notifyObservers(Object)} gemeldet.
     */
    public final SonderfertigkeitWert getWert(
            final SonderfertigkeitWert wert ) {

        return sonderfertigkeiten.getWert( wert );
    }

    /**
     * @param talent
     *            dessen Wert gesucht ist.
     * @return den Wert des Talents oder {@code null} wenn der Held das Talent
     *         nicht besitzt. Es handelt sich um das selbe Objekt das im Helden
     *         gespeichert ist. Aenderungen am Wert wirken sich direkt auf den
     *         Helden aus. Der Werte ist dem Helden zugeordnet. Aenderungen an
     *         ihm werden ueber {@link #notifyObservers(Object)} gemeldet.
     */
    public final TalentWert getWert( final Talent talent ) {
        return talente.getWert( talent );
    }

    /**
     * Liefert den Wert eines Vor-/Nachteils.
     *
     * @param vorNachteil
     *            dessen Wert gesucht ist.
     * @return den Wert des Helden. Besitzt der Held mehrere Varianten des
     *         Vor-/Nachteils, so wird eine beliebige zurueckgegeben. Besitzt
     *         der Held den Vor-/Nachteil nicht, so liefert die Methode
     *         {@code null}. Aenderungen am Wert wirken sich direkt auf den
     *         Helden aus. Der Werte ist dem Helden zugeordnet. Aenderungen an
     *         ihm werden ueber {@link #notifyObservers(Object)} gemeldet.
     */
    public final VorNachteilWert getWert( final VorNachteil vorNachteil ) {
        return vorNachteile.getWert( vorNachteil );
    }

    /**
     * @param wert
     *            der gesucht ist.
     * @return den Wert des Zaubers oder {@code null} wenn der Held den Zauber
     *         nicht besitzt. Es handelt sich um das selbe Objekt das im Helden
     *         gespeichert ist. Aenderungen am Wert wirken sich direkt auf den
     *         Helden aus. Der Werte ist dem Helden zugeordnet. Aenderungen an
     *         ihm werden ueber {@link #notifyObservers(Object)} gemeldet.
     */
    public final ZauberWert getWert( final ZauberWert wert  ) {
        return zauber.getWert( wert );
    }

    /**
     * Liefert den Wert eines Helden.
     *
     * @param wert
     *            der gesucht ist.
     * @return den Wert des Helden. {@code null}, wenn der Held den Wert nicht
     *         besitzt. Aenderungen am Wert wirken sich direkt auf den Helden
     *         aus.
     */
    public final VorNachteilWert getWert(
            final VorNachteilWert wert ) {
        return vorNachteile.getWert( wert );
    }

    /**
     * Liefert alle Varianten eines Vor-/Nachteils.
     *
     * @param vorNachteil
     *            dessen Varianten gesucht sind.
     * @return alle Varianten des Vor-/Nachteils die der Held besitzt. Die Werte
     *         sind dem Helden zugeordnet. Aenderungen an ihnen werden ueber
     *         {@link #notifyObservers(Object)} gemeldet.
     */
    public final List< VorNachteilWert > getWerte(
            final VorNachteil vorNachteil ) {

        return vorNachteile.getWerte( vorNachteil );
    }

    /**
     * Liefert alle Varianten eines Zaubers.
     *
     * @param z
     *            dessen Varianten gesucht sind.
     * @return alle Varianten des Zaubers die der Held besitzt. Die Werte sind
     *         dem Helden zugeordnet. Aenderungen an ihnen werden ueber
     *         {@link #notifyObservers(Object)} gemeldet.
     */
    public final List< ZauberWert > getWerte( final Zauber z ) {

        return zauber.getWerte( z );
    }

    /**
     * @return eine Liste aller Zauber die der Held besitzt.
     */
    public final List< ZauberWert > getZauber() {
        return zauber.getZauber();
    }

    /**
     * @param zaubergruppe
     *            deren Zauber gesucht sind.
     * @return alle Zauber der uebergebenen Zaubergruppe, die der Held besitzt.
     */
    public final List< ZauberWert > getZauber(
            final Zaubergruppe zaubergruppe ) {

        return zauber.get( zaubergruppe );
    }

    /**
     * @see #hatZweitsprache()
     * @return die Zweitsprache des Helden oder {@code null} wenn der Held keine
     *         Zweitsprache hat.
     */
    public final Sprache getZweitsprache() {
        return zweitsprache;
    }

    /**
     * Prueft ob ein Held eine Sonderfertigkeit oder eine Variante der
     * Sonderfertigkeit besitzt.
     *
     * @param sonderfertigkeit
     *            fuer die geprueft werden soll ob der Held sie bestizt.
     * @return {@code true} wenn der Held die Sonderfertigkeit besitzt, sonst
     *         {@code false}.
     */
    public final boolean hat( final Sonderfertigkeit sonderfertigkeit ) {
        return sonderfertigkeiten.enthaelt( sonderfertigkeit );
    }

    /**
     * Prueft ob ein Held eine bestimmte Variante einer Sonderfertigkeit
     * besitzt.
     *
     * @param wert
     *            fuer den geprueft werden soll ob der Held ihn besitzt
     * @return {@code true} wenn der Held die Sonderfertigkeit besitzt, sonst
     *         {@code false}.
     */
    public final boolean hat( final SonderfertigkeitWert wert ) {
        return sonderfertigkeiten.enthaelt( wert );
    }

    /**
     * Prueft ob der Held ein bestimmtes Talent besitzt. Es wird nicht geprueft
     * ob das Talent auch aktiviert ist. Für Spezial-Talente mit negativer Stufe
     * oder Stufe 0, die nicht aktiviert sind, liefert die Methode ebenfalls
     * {@code true}.
     *
     * @param talent
     *            von dem festgestellt werden soll ob der Held es besitzt.
     * @return {@code true} wenn der Held das Talent besitzt, sonst
     *         {@code false}.
     */
    public final boolean hat( final Talent talent ) {
        return talente.enthaelt( talent );
    }

    /**
     * Prueft ob der Held einen bestimmten Vor-/Nachteil besitzt.
     *
     * @param vorNachteil
     *            von dem festgestellt werden soll ob der Held ihn besitzt.
     * @return {@code true}, wenn der Held den Vor-/Nachteil besitzt, sonst
     *         {@code false}.
     */
    public final boolean hat( final VorNachteil vorNachteil ) {
        return vorNachteile.enthaelt( vorNachteil );
    }

    /**
     * Prueft ob der Held einen bestimmten Vor-/Nachteil besitzt.
     *
     * @param wert
     *            von dem festgestellt werden soll ob der Held ihn besitzt.
     * @return {@code true}, wenn der Held den Vor-/Nachteil besitzt, sonst
     *         {@code false}.
     */
    public final boolean hat( final VorNachteilWert wert ) {
        return vorNachteile.enthaelt( wert );
    }

    /**
     * Prueft ob der Held ein bestimmtes Zauber besitzt.
     *
     * @param z
     *            von dem festgestellt werden soll ob der Held ihn besitzt.
     * @return {@code true} wenn der Held das Zauber besitzt, sonst
     *         {@code false}.
     */
    public final boolean hat( final Zauber z ) {
        return zauber.enthaelt( z );
    }

    /**
     * Prueft ob der Held ein bestimmtes Zauber besitzt.
     *
     * @param wert
     *            von dem festgestellt werden soll ob der Held ihn besitzt.
     * @return {@code true} wenn der Held das Zauber besitzt, sonst
     *         {@code false}.
     */
    public final boolean hat( final ZauberWert wert ) {
        return zauber.enthaelt( wert );
    }

    /**
     * Prueft ob die Werte dieses Helden sich von den Werten des uebergebenen
     * Helden unterscheiden.
     * 
     * @param held gegen den geprueft werden soll.
     * 
     * @return {@code true}, wenn sich die Werte der Helden unterscheiden, sonst
     *         {@code false}.
     */
    public final boolean hatAndereWerteAls( final Held held ) {
        
        return ! Util.equals(
                aktivierteTalente, held.aktivierteTalente,
                aktivierteZauber, held.aktivierteZauber,
                alter, held.alter,
                apEingesetzt, held.apEingesetzt,
                apGesamt, held.apGesamt,
                augenfarbe, held.augenfarbe,
                aussehen, held.aussehen,
                basiswerte, held.basiswerte,
                eigenschaften, held.eigenschaften,
                fernkampfwaffen, held.fernkampfwaffen,
                geburtstag, held.geburtstag,
                gelaendekunden, held.gelaendekunden,
                geschlecht, held.geschlecht,
                gewichtInKg, held.gewichtInKg,
                groesseInCm, held.groesseInCm,
                haarfarbe, held.haarfarbe,
                hauszauber, held.hauszauber,
                hintergrund, held.hintergrund,
                kultur, held.kultur,
                leitsonderfertigkeiten, held.leitsonderfertigkeiten,
                leittalente, held.leittalente,
                leitzauber, held.leitzauber,
                muttersprache, held.muttersprache,
                nahkampfwaffen, held.nahkampfwaffen,
                name, held.name,
                ortskenntnisse, held.ortskenntnisse,
                profession, held.profession,
                rasse, held.rasse,
                ruestungen, held.ruestungen,
                schilde, held.schilde,
                sonderfertigkeiten, held.sonderfertigkeiten,
                sonderfertigkeitenFehlend, held.sonderfertigkeitenFehlend,
                sonderfertigkeitenVerbilligt, held.sonderfertigkeitenVerbilligt,
                sonderfertigkeitenVorgegeben, held.sonderfertigkeitenVorgegeben,
                stand, held.stand,
                verbindungen, held.verbindungen,
                vorNachteile, held.vorNachteile,
                vorNachteileAbgebaut, held.vorNachteileAbgebaut,
                vorNachteileFehlend, held.vorNachteileFehlend,
                vorNachteileVorgegeben, held.vorNachteileVorgegeben,
                wirdErschaffen, held.wirdErschaffen,
                zauber, held.zauber,
                zauberFehlend, held.zauberFehlend,
                zauberVorgegeben, held.zauberVorgegeben,
                zweitsprache, held.zweitsprache
        );
    }
    
    /**
     * @param vorNachteil
     *            fuer den geprueft werden soll ob er fuer den Helden empfohlen
     *            ist.
     * @return {@code true}, wenn der Vor-/Nachteil fuer den Helden empfohlen
     *         ist, sonst {@code false}.
     */
    public final boolean hatEmpfohlenen( final VorNachteil vorNachteil ) {
        return rasse.hatEmpfohlenen( vorNachteil )
            || kultur.hatEmpfohlenen( vorNachteil )
            || profession.hatEmpfohlenen( vorNachteil );
    }

    /**
     * Stellt fest ob ein Zauber fuer den Helden ein Hauszauber ist.
     *
     * @param zauber
     *            fuer den geprueft werden soll ob er Hauszauber ist.
     * @return {@code true}, wenn es sich bei dem Zauber um einen Hauszauber
     *         handelt, sonst {@code false}.
     */
    public final boolean hatHauszauber( final Zauber zauber ) {
        return hauszauber.contains( zauber );
    }

    /**
     * Stellt fest ob eine Sonderfertigkeit fuer einen (elfischen) Helden ein
     * Leitsonderfertigkeit ist.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er Leitsonderfertigkeit ist.
     * @return {@code true}, wenn es sich um eine Leitsonderfertigkeit
     *         handelt, sonst {@code false}.
     */
    public final boolean hatLeitsonderfertigkeit(
            final SonderfertigkeitWert wert ) {

        return leitsonderfertigkeiten.enthaelt( wert );
    }

    /**
     * Stellt fest ob ein Talent fuer den (elfischen) Helden ein Leittalent ist.
     *
     * @param talent
     *            fuer das geprueft werden soll ob es Leittalent ist.
     * @return {@code true}, wenn es sich bei dem Talent um ein Leittalent
     *         handelt, sonst {@code false}.
     */
    public final boolean hatLeittalent( final Talent talent ) {
        return leittalente.contains( talent );
    }

    /**
     * Stellt fest ob ein Zauber fuer den (elfischen) Helden ein Leitzauber ist.
     *
     * @param zauber
     *            fuer den geprueft werden soll ob er Leitzauber ist.
     * @return {@code true}, wenn es sich bei dem Zauber um einen Leitzauber
     *         handelt, sonst {@code false}.
     */
    public final boolean hatLeitzauber( final Zauber zauber ) {
        return leitzauber.contains( zauber );
    }

    /**
     * Prueft ob eine Sprache die Muttersprache des Helden ist.
     *
     * @param sprache
     *            fuer die ueberprueft werden soll ob sie die Muttersprache des
     *            Helden ist.
     * @return {@code true} wenn die Sprache die Muttersprache des Helden ist,
     *         sonst {@code false}.
     */
    public final boolean hatMuttersprache( final Sprache sprache ) {
        return muttersprache == sprache;
    }

    /**
     * @param vorNachteil
     *            fuer den geprueft werden soll ob er fuer den Helden ungeeignet
     *            ist.
     * @return {@code true}, wenn der Vor-/Nachteil fuer den Helden ungeeignet
     *         ist, sonst {@code false}.
     */
    public final boolean hatUngeeigneten( final VorNachteil vorNachteil ) {
        return rasse.hatUngeeigneten( this, vorNachteil )
            || kultur.hatUngeeigneten( this, vorNachteil )
            || profession.hatUngeeigneten( this, vorNachteil );
    }

    /**
     * Prueft ob eine Sonderfertigkeit fuer einen Helden verbilligt zu erlernen
     * ist. Verbilligte Sonderfertigkeiten kosten nur 50%.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er verbilligt ist.
     * @return {@code true} wenn die Sonderfertigkeit verbilligt ist, sonst
     *         {@code false}.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final boolean hatVerbilligt( final SonderfertigkeitWert wert ) {

        final Sonderfertigkeit sonderfertigkeit = wert.getSonderfertigkeit();

        return sonderfertigkeitenVerbilligt.contains( sonderfertigkeit )
            || sonderfertigkeitenVerbilligtVarianten.enthaelt( wert );
    }

    /**
     * Prueft ob eine Variante einer Sonderfertigkeit als verbilligt gilt und
     * zwar ob nur diese Variante der Sonderfertigkeit als verbilligt gilt. Um
     * allgemein festzustellen ob eine Sonderfertigkeit verbilligt wird dient
     * die Methode {@link #hatVerbilligt(SonderfertigkeitWert)}.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er verbilligt wird.
     * @return {@code true} wenn die Variante als verbilligt gilt, sonst
     *         {@code false}.
     */
    public final boolean hatVerbilligtVariante(
            final SonderfertigkeitWert wert ) {

        return sonderfertigkeitenVerbilligtVarianten.enthaelt( wert );
    }

    /**
     * Eine Profession kann automatisch Verbindungen im Wert von SO x Punkten
     * vergeben. Fuer
     * einen Vor-/Nachteil kann mit dieser Methode geprueft werden ob es sich
     * dabei um Verbindungen handelt die (teilweise) mit diesen x Punkten
     * finanziert wurden.
     *
     * @see #entfernenVerbindungen(VorNachteilWert)
     * @see #getVerbindungen(VorNachteilWert)
     * @see #setVerbindungenStufe(VorNachteilWert)
     * @param wert
     *            der Vorteil, fuer den ueberprueft werden soll ob es sich dabei
     *            um Verbindungen handelt, die mit den x Punkten, die die
     *            Profession vergibt, finanziert wurden.
     * @return {@code true} wenn es sich um Verbindugnen handelt die mit den
     *         Punkten aus der Profession bezahlt wurden, sonst {@code false}.
     */
    public final boolean hatVerbindungen( final VorNachteilWert wert ) {

        return verbindungen.enthaelt( wert );
    }

    /**
     * Prueft ob der Held eine Sonderfertigkeit besitzt, die durch Rasse, Kultur
     * oder Profession vorgegeben ist.
     *
     * @param sonderfertigkeit
     *            fuer die geprueft werden soll ob sie vorgegeben ist.
     * @return {@code true} wenn die Sonderfertigkeit vorgegeben ist, sonst
     *         {@code false}.
     */
    public final boolean hatVorgegeben(
            final Sonderfertigkeit sonderfertigkeit ) {

        return sonderfertigkeitenVorgegeben.enthaelt( sonderfertigkeit );
    }

    /**
     * Prueft ob der Held eine Sonderfertigkeit besitzt, die durch Rasse, Kultur
     * oder Profession vorgegeben ist.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er vorgegeben ist.
     * @return {@code true} wenn der Wert vorgegeben ist, sonst {@code false}.
     */
    public final boolean hatVorgegeben( final SonderfertigkeitWert wert ) {
        return sonderfertigkeitenVorgegeben.enthaelt( wert );
    }

    /**
     * Prueft ob ein Vor-/Nachteil fuer den Helden durch Rasse, Kultur oder
     * Profession vorgegeben ist.
     *
     * @param vorNachteil
     *            fuer den geprueft werden soll ob er vorgegeben ist.
     * @return {@code true} wenn der Vor-/Nachteil vorgegeben ist, sonst
     *         {@code false}.
     */
    public final boolean hatVorgegeben( final VorNachteil vorNachteil ) {
        return vorNachteileVorgegeben.enthaelt( vorNachteil );
    }

    /**
     * Prueft ob ein Vor-/Nachteil fuer den Helden durch Rasse, Kultur oder
     * Profession vorgegeben ist.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er vorgegeben ist.
     * @return {@code true} wenn der Wert vorgegeben ist, sonst {@code false}.
     */
    public final boolean hatVorgegeben( final VorNachteilWert wert ) {
        return vorNachteileVorgegeben.enthaelt( wert );
    }

    /**
     * Prueft ob der Held ein bestimmtes Talent besitzen muss weil das Talent,
     * z.B. durch seine Rasse oder einen Vorteil, vorgegeben ist. Zu einem
     * vorgegebenen Talent kann ein Mindesstufe abgerufen werden siehe
     * {@link #getMindeststufe(Talent)}.
     *
     * @see #getMindeststufe(Talent)
     * @param talent
     *            von dem geprueft werden soll ob es vorgegeben ist.
     * @return {@code true} wenn der Held das Talent besitzen muss, sonst
     *         {@code false}.
     */
    public final boolean hatVorgegeben( final Talent talent ) {
        return talenteVorgegeben.enthaelt( talent );
    }

    /**
     * Prueft ob der Held einen Bonus auf ein Talent durch seine Profession
     * erhalten hat.
     *
     * @param talent
     *            von dem geprueft werden soll ob es durch die Profession einen
     *            Bonus erhalten hat.
     * @return {@code true} wenn der Held durch seine Profession einen Bonus auf
     *         das Talent erhalten hat, sonst {@code false}.
     */
    public final boolean hatVorgegebenProf( final Talent talent ) {
        return talenteProfession.enthaelt( talent );
    }

    /**
     * Prueft ob der Held einen bestimmten Zauber besitzen muss weil er,
     * z.B. durch seine Rasse oder einen Vorteil, vorgegeben ist.
     *
     * @param z
     *            von dem geprueft werden soll ob er vorgegeben ist.
     * @return {@code true}, wenn der Held den Zauber besitzen muss, sonst
     *         {@code false}.
     */
    public final boolean hatVorgegeben( final Zauber z ) {
        return zauberVorgegeben.enthaelt( z );
    }

    /**
     * Prueft ob der Held einen bestimmten Zauber besitzen muss weil er,
     * z.B. durch seine Rasse oder einen Vorteil, vorgegeben ist. Zu einem
     * vorgegebenen Zauber kann ein Mindesstufe abgerufen werden siehe
     * {@link #getMindeststufe(ZauberWert)}.
     *
     * @see #getMindeststufe(ZauberWert)
     * @param wert
     *            von dem geprueft werden soll ob er vorgegeben ist.
     * @return {@code true}, wenn der Held den Zauber besitzen muss, sonst
     *         {@code false}.
     */
    public final boolean hatVorgegeben( final ZauberWert wert ) {
        return zauberVorgegeben.enthaelt( wert );// || zauberFehlend.enthaelt( wert );
    }

    /**
     * Prueft ob der Held eine Zweitsprache besitzt.
     *
     * @return {@code true} wenn der Held eine Zweitsprache besitzt, sonst
     *         {@code false}.
     */
    public final boolean hatZweitsprache() {
        return null != zweitsprache;
    }

    /**
     * Prueft ob eine Sprache die Zweitsprache des Helden ist.
     *
     * @param sprache
     *            fuer die ueberprueft werden soll ob sie die Zweitsprache des
     *            Helden ist.
     * @return {@code true} wenn die Sprache die Zweitsprache des Helden ist,
     *         sonst {@code false}.
     */
    public final boolean hatZweitsprache( final Sprache sprache ) {
        return zweitsprache == sprache;
    }

    /**
     * Prueft ob der Held einen Nachteil abgebaut hat.
     *
     * @param wert
     *            fuer den geprueft werden soll, ob der Held ihn abgebaut hat.
     * @return {@code true}, wenn der Held den Wert abgebaut hat, sonst
     *         {@code false}.
     */
    public final boolean istAbgebaut( final VorNachteilWert wert ) {
        return vorNachteileAbgebaut.enthaelt( wert );
    }

    /**
     * Prueft ob ein Wert die erste Gelaendekunde ist die der Held erhalten hat
     * oder ob der Wert die erste Gelaendekunde fuer den Held waere. Mit andern
     * Worten die Methode liefert bei einer Gelaendekunde {@code true} wenn der
     * Wert die erste Gelaendekunde ist, die der Held erhalten hat und wenn der
     * Held noch keine Gelaendekunde besitzt.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er die (potentiell) erste
     *            Gelaendekunde fuer den Helden ist.
     * @return {@code true} wenn {@code wert} die erste Gelaendekunde ist die
     *         der Held erhalten hat oder die erste Gelaendekunde waere, wenn er
     *         sie erhalten wuerde, sonst {@code false}.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final boolean istErsteGelaendekunde(
            final SonderfertigkeitWert wert ) {

        if ( ! wert.getSonderfertigkeit().istGelaendekunde() ) {
            // wert ist gar keine Gelaendekunde.
            return false;

        }

        return 0 >= gelaendekunden.size() || gelaendekunden.get(0).equals(wert);
    }

    /**
     * Prueft ob ein Wert die erste Ortskenntnis ist die ein Held erhalten hat
     * oder ob der Wert die erste Ortskenntnis waere die der Held erhalten
     * wuerde. Mit andern Worten die Method liefert bei einer Ortskenntnis
     * {@code true} wenn der Wert die erste Ortskenntis ist oder wenn der Held
     * noch keine Ortskenntnis besitzt.
     *
     * @param wert
     *            fuer den geprueft werden soll ob er die erste Ortskennis ist.
     * @return {@code true} wenn {@code wert} die erste Ortskenntnis ist, die
     *         der Held erhalten hat bzw es die erste Ortskenntis waere wenn der
     *         Held sie erhalten wuerde, sonst {@code false}.
     * @throws NullPointerException
     *             wenn {@code null} uebergeben wird.
     */
    public final boolean istErsteOrtskenntnis(
            final SonderfertigkeitWert wert ) {

        return 0 >= ortskenntnisse.size() || ortskenntnisse.get(0).equals(wert);
    }

    /**
     * @return {@code true}, wenn der Held magiebegabt ist, sonst {@code false}.
     */
    public final boolean istZauberer() {
        return hat( VorNachteil.VOLLZAUBERER )
            || hat( VorNachteil.VIERTELZAUBERER )
            || hat( VorNachteil.VIERTELZAUBERER_UNBEWUSST );
    }
    
    /**
     * Wird von den Werten des Helden aufgerufen, wenn sie geaendert werden.
     */
    public void melden( Beobachtbar b, Object arg ) {
        
        notifyObservers( arg );
    }
    
    /**
     * Ruft erst {@link #setChanged()} auf und dann
     * {@code super.notifyObservers}.
     */
    @Override
    public final void notifyObservers() {
        setChanged();
        super.notifyObservers();
    }

    /**
     * Ruft erst {@link #setChanged()} auf und dann
     * {@code super.notifyObservers(Object)}.
     */
    @Override
    public final void notifyObservers(final Object arg) {
        setChanged();
        super.notifyObservers(arg);
    }

    /**
     * @param neuesAlter des Helden.
     */
    public final void setAlter( final int neuesAlter ) {

        if ( neuesAlter != alter ) {

            alter = neuesAlter;

            notifyObservers();
        }
    }

    /**
     * @param ap die der Held insgesamt erhalten hat.
     */
    public final void setApGesamt( final int ap ) {

        if ( ( ap >= 0 )
          && ( ap != apGesamt ) ) {

            apGesamt = ap;

            notifyObservers();
        }
    }

    /**
     * @param ap die der Held bisher zum Steigern eingesetzt hat.
     */
    public final void setApEingesetzt( final int ap ) {

        if ( ( ap >= 0 )
          && ( ap != apEingesetzt ) ) {

            apEingesetzt = ap;

            notifyObservers();
        }
    }

    /**
     * @param neueAugenfarbe
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setAugenfarbe( Augenfarbe neueAugenfarbe ) {
        if ( null == neueAugenfarbe ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        if ( ! augenfarbe.equals( neueAugenfarbe ) ) {

            augenfarbe = neueAugenfarbe;

            notifyObservers( AUGENFARBE );
        }
    }

    /**
     * @param neuesAussehen
     *            Ein Beschreibung des Aussehens des Helden.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setAussehen( String neuesAussehen ) {

        if ( null == neuesAussehen ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        neuesAussehen = neuesAussehen.trim();

        if ( ! aussehen.equals( neuesAussehen ) ) {

            aussehen = neuesAussehen;

            notifyObservers();
        }
    }

    /**
     * @param neuerGeburtstag
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setGeburtstag( String neuerGeburtstag ) {
        if ( null == neuerGeburtstag ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        neuerGeburtstag = neuerGeburtstag.trim();

        if ( ! geburtstag.equals( neuerGeburtstag ) ) {

            geburtstag = neuerGeburtstag;

            notifyObservers();
        }
    }

    /**
     * @param neuesGeschlecht
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setGeschlecht( final Geschlecht neuesGeschlecht ) {
        if ( null == neuesGeschlecht ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        if ( geschlecht != neuesGeschlecht ) {

            geschlecht = neuesGeschlecht;

            notifyObservers();
        }
    }

    /**
     * @param neuesGewichtInKg
     *            des Helde in kg bzw. Stein.
     */
    public final void setGewicht( final int neuesGewichtInKg ) {

        if ( gewichtInKg != neuesGewichtInKg ) {

            gewichtInKg = neuesGewichtInKg;

            notifyObservers();
        }
    }

    /**
     * @param neueGroesseInCm
     *            des Helden in cm bzw. Halbfinger.
     */
    public final void setGroesse( final int neueGroesseInCm ) {

        if ( groesseInCm != neueGroesseInCm ) {

            groesseInCm = neueGroesseInCm;

            notifyObservers();
        }
    }

    /**
     * @param neuerHintergrund
     *            Informationen ueber Familie, Herkunft und Hintergrund des
     *            Helden.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setHintergrund( String neuerHintergrund ) {

        if ( null == neuerHintergrund ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        neuerHintergrund = neuerHintergrund.trim();

        if ( ! hintergrund.equals( neuerHintergrund ) ) {

            hintergrund = neuerHintergrund;

            notifyObservers();
        }
    }

    /**
     * @param neueHaarfarbe
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setHaarfarbe( final Haarfarbe neueHaarfarbe ) {
        if ( null == neueHaarfarbe ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        if ( ! haarfarbe.equals( neueHaarfarbe ) ) {

            haarfarbe = neueHaarfarbe;

            notifyObservers( HAARFARBE );
        }
    }

    /**
     * Aendert die Kultur des Helden.
     *
     * @param neueKultur
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setKultur( final Kultur neueKultur ) {
        if ( null == neueKultur ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        if ( kultur != neueKultur ) {

            kultur = neueKultur;

            if ( wirdErschaffen && kultur.hatNameFactory() )
            {
                name = kultur.getNameFactory().erzeugeName( this );
            }
            
            notifyObservers();
        }
    }

    /**
     * Bestimmt die Muttersprache des Helden.
     *
     * @param dieMuttersprache
     *            des Helden.
     * @throws IllegalStateException
     *             wenn der Held schon eine Muttersprache besitzt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setMuttersprache(
            final Sprache dieMuttersprache ) {

        if ( null == dieMuttersprache ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        if ( null != muttersprache ) {
            throw new IllegalStateException( String.format(
                    "Die Muttersprache des Helden sollte zum zweiten mal " +
                    "initialisiert werden.\n" +
                    "bestehende Muttersprache: %1$s\n" +
                    "neue Muttersprache: %2$s",
                    muttersprache,
                    dieMuttersprache ) );
        }

        muttersprache = dieMuttersprache;

        einfuegenVorgegeben( new TalentWert( dieMuttersprache, 0 ) );
    }

    /**
     * @param neuerName
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setName( String neuerName ) {
        if ( null == neuerName ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        neuerName = neuerName.trim();

        if ( ! name.equals( neuerName ) ) {

            name = neuerName;

            notifyObservers();
        }
    }

    /**
     * Aendert die Profession des Helden.
     *
     * @param neueProfession
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setProfession( final Profession neueProfession ) {
        if ( null == neueProfession ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        if ( profession != neueProfession ) {

            profession = neueProfession;

            // Durch eine zeitaufwendige Profession kann sich das
            // Basisalter geaendert haben.
            alter = rasse.getAlter().auswuerfeln( this );

            notifyObservers();
        }
    }

    /**
     * Aendert die Rasse des Helden und aktualisiert Augenfarbe, Groesse,
     * Gewicht und Haarfarbe entsprechend der Rasse.
     *
     * @param neueRasse
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setRasse( final Rasse neueRasse ) {
        if ( null == neueRasse ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        if ( rasse != neueRasse ) {

            rasse = neueRasse;

            alter = rasse.getAlter().auswuerfeln( this );

            final String augenfarbeId = rasse.getAugenfarben().wuerfeln();
            
            augenfarbe = Augenfarbe.getAugenfarbe( augenfarbeId );

            groesseInCm = rasse.getGroesse().wuerfeln();

            gewichtInKg = rasse.getGewicht().berechnen( this );

            final String haarfarbeId = rasse.getHaarfarben().wuerfeln(); 
            
            haarfarbe = Haarfarbe.getHaarfarbe( haarfarbeId );

            notifyObservers( RASSE );
        }
    }

    /**
     * @param neuerStand
     *            Informationen ueber den Stand des Helden.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setStand( String neuerStand ) {
        if ( null == neuerStand ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        neuerStand = neuerStand.trim();

        if ( ! stand.equals( neuerStand ) ) {

            stand = neuerStand;

            notifyObservers();
        }
    }

    /**
     * Aendert die Stufe eines Talents und berechnet dem Helden die dafuer
     * noetigen AP.
     *
     * @param talent
     *            dessen Stufe geaendert werden soll.
     * @param stufe
     *            die das Talent erhalten soll.
     * @param lernmethode
     *            die verwendet wird, falls das Talent zum aendern der Stufe
     *            gesteigert werden soll.
     */
    public final void setStufe(
            final Talent talent,
            final int stufe,
            final Lernmethode lernmethode ) {

        final TalentWert wert = getWert( talent );

        final int kosten = wert.setStufe( this, stufe, lernmethode );

        apEingesetzt += kosten;

        notifyObservers( talent );
    }

    /**
     * Aendert die Stufe eines Nachteils und berechnet dem Helden die dafuer
     * noetigen AP.
     *
     * @param wert
     *            dessen Stufe geaendert werden soll.
     * @param stufe
     *            die der Nachteil erhalten soll.
     * @param lernmethode
     *            die verwendet wird, falls es sich um eine Schlechte
     *            Eigenschaft handelt, die gesenkt werden soll.
     */
    public final void setStufe(
            final VorNachteilWert wert,
            final int stufe,
            final Lernmethode lernmethode ) {

        final VorNachteilWert heldenwert = getWert( wert );

        final int kosten = heldenwert.setStufe( this, stufe, lernmethode );

        apEingesetzt += kosten;

        notifyObservers( wert.getVorNachteil() );
    }

    /**
     * Aendert die Stufe eines Zaubers und berechnet dem Helden die dafuer
     * noetigen AP.
     *
     * @param wert
     *            dessen Stufe geaendert werden soll.
     * @param stufe
     *            die der Zauber erhalten soll.
     * @param lernmethode
     *            die verwendet wird, falls der Zauber zum aendern der Stufe
     *            gesteigert werden muss.
     */
    public final void setStufe(
            final ZauberWert wert,
            final int stufe,
            final Lernmethode lernmethode ) {

        final ZauberWert heldenwert = getWert( wert );

        final int kosten = heldenwert.setStufe( this, stufe, lernmethode );

        apEingesetzt += kosten;

        notifyObservers( wert.getZauber() );
    }

    /**
     * @param neuerTitel
     *            des Helden. {@code null} ist nicht erlaubt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setTitel( String neuerTitel ) {
        if ( null == neuerTitel ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt" );
        }

        neuerTitel = neuerTitel.trim();

        if ( ! titel.equals( neuerTitel ) ) {

            titel = neuerTitel;

            notifyObservers();
        }
    }

    /**
     * Eine Profession kann Verbindungen im Wert von SO x Punkten automatisch
     * vergegben. Mit dieser Methode kann eingestellt werden wie viele Punkte
     * auf einen einzelnen Wert verteilt wurden.
     *
     * @see #entfernenVerbindungen(VorNachteilWert)
     * @see #hatVerbindungen(VorNachteilWert)
     * @see #getVerbindungen(VorNachteilWert)
     * @param wert
     *            Die Stufe dieses Wertes gibt die Punkte an, die aus den x
     *            Punkten des Profession, die auf den Wert verteilt wurden.
     * @throws NullPointerException
     *             wenn es sich bei {@code wert} nicht um Verbindungen handelt,
     *             die mit den Punkten der Profession bezahlt wurden.
     */
    public final void setVerbindungenStufe(
            final VorNachteilWert wert ) {

        final int stufe = wert.getStufe();

        vorNachteileVorgegeben.getWert( wert ).setStufe( stufe );

        verbindungen.getWert( wert ).setStufe( stufe );

        vorNachteile.getWert( wert ).setStufe( stufe );
    }

    /* *
     * Eien Profession kann automatisch x Punkte vergeben von denen
     * Verbindungen finanziert werden duerfen. Diese Methode aendert die
     * Variante eines so finanzierten Vorteils.
     *
     * @see #entfernenVerbindungen(VorNachteilWert)
     * @see #hatVerbindungen(VorNachteilWert)
     * @see #getVerbindungen(VorNachteilWert)
     * @see #setVerbindungenStufe(VorNachteilWert)
     * @param wert
     *            dessen Variante geaendert werden soll.
     * @param variante
     *            die neue Variante fuer den Wert.
     * @throws NullPointerException
     *             wenn es sich bei dem Wert nicht um Verbindungen handelt die
     *             mit den x Punkten finanziert wurden.
     * /
    public final void setVerbindungenVariante(
            final VorNachteilWert wert,
            final Object variante ) {

        vorNachteileVorgegeben.getWert( wert ).setVariante( variante );

        verbindungen.getWert( wert ).setVariante( variante );

        vorNachteile.getWert( wert ).setVariante( variante );
    }
*/
    /**
     * @param wirdErschaffen
     *            {@code true}, wenn der Held gerade erschaffen wird,
     *            {@code false}, wenn die Erschaffung vorueber ist.
     */
    public final void setWirdErschaffen( final boolean wirdErschaffen ) {
        this.wirdErschaffen = wirdErschaffen;
    }

    /**
     * Bestimmt die Zweitsprache des Helden.
     *
     * @param dieZweitsprache
     *            des Helden.
     * @throws IllegalStateException
     *             wenn der Held schon eine Zweitsprache besitzt.
     * @throws IllegalArgumentException
     *             wenn {@code null} uebergeben wird.
     */
    public final void setZweitsprache(
            final Sprache dieZweitsprache ) {

        if ( null == dieZweitsprache ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );

        }

        if ( null != zweitsprache ) {
            throw new IllegalStateException( String.format(
                    "Die Zweitsprache des Helden sollte zum zweiten mal " +
                    "initialisiert werden.\n" +
                    "bestehende Zweitsprache: %1$s\n" +
                    "neue Zweitsprache: %2$s",
                    zweitsprache,
                    dieZweitsprache ) );
        }

        zweitsprache = dieZweitsprache;

        einfuegenVorgegeben( new TalentWert( dieZweitsprache, 0 ) );
    }

    /**
     * Wird aufgerufen, wenn sich die Variante einer Sonderfertigkeit oder eines
     * anderen Wertes geaendert hat.
     */
    public final void varianteGeandert( 
            final Object sender, 
            final Variante alt, 
            final Variante neu ) {
        
        if ( varianteGeaendertAktiv ) {
            
            return;
        }
        
        varianteGeaendertAktiv = true;
        
        if ( sender instanceof SonderfertigkeitWert ) {
            
            final SonderfertigkeitWert wert = (SonderfertigkeitWert) sender;
            
            varianteGeaendert( wert, alt, neu );
        
        } else if ( sender instanceof VorNachteilWert ) {
            
            final VorNachteilWert wert = (VorNachteilWert) sender;
            
            varianteGeaendert( wert, alt, neu );
        
        } else if ( sender instanceof ZauberWert ) {
            
            final ZauberWert wert = (ZauberWert) sender;
            
            varianteGeaendert( wert, alt, neu );
        }
        
        varianteGeaendertAktiv = false;
    }
    
    /**
     * Wird aufgerufen, wenn sich die Variante einer Sonderfertigkeit geaendert
     * hat.
     * 
     * @param wertNeu Der Wert, in dem die Variante geaendert wurde.
     * @param varianteAlt Die alte Variante.
     * @param varianteNeu Die neue Variante.
     */
    private void varianteGeaendert(
            final SonderfertigkeitWert wertNeu,
            final Variante varianteAlt,
            final Variante varianteNeu ) {
        
        final SonderfertigkeitWert wertAlt = wertNeu.clone();
        wertAlt.setVariante( varianteAlt );
        
        // In irgendeiner Collection wurde der Wert geaendert, das muss jetzt
        // ueberall nachgezogen werden.
        
        if ( ! hat( wertNeu ) && hat( wertAlt ) ) {
            
            getWert( wertAlt ).setVariante( varianteNeu );
        }
        
        if ( ! hatVorgegeben( wertNeu ) && hatVorgegeben( wertAlt ) ) {

            getVorgegeben( wertAlt ).setVariante( varianteNeu );
        }

        if ( ! hatVerbilligtVariante( wertNeu )
                && hatVerbilligtVariante( wertAlt ) ) {

            getVerbilligteVariante( wertAlt ).setVariante( varianteNeu );
        }

        if ( ! hatLeitsonderfertigkeit( wertNeu ) 
                && hatLeitsonderfertigkeit( wertAlt ) ) {

            getLeitsonderfertigkeit( wertAlt ).setVariante( varianteNeu );
        }
    }
    
    /**
     * Wird aufgerufen, wenn sich die Variante eines Vor-/Nachteils geaendert
     * hat.
     * 
     * @param wertNeu Der Wert dessen Variante geaendert wurde.
     * @param varianteAlt Die Variante die der Wert zuvor hatte.
     * @param varianteNeu Die Variante die der Wert jetzt hat.
     */
    private void varianteGeaendert(
            final VorNachteilWert wertNeu,
            final Variante varianteAlt,
            final Variante varianteNeu ) {
        
        final VorNachteilWert wertAlt = wertNeu.clone();
        wertAlt.setVariante( varianteAlt );
        
        if ( ! hat( wertNeu ) && hat( wertAlt ) ) {
            
            getWert( wertAlt ).setVariante( varianteNeu );
        }
        
        if ( ! hatVerbindungen( wertNeu ) && hatVerbindungen( wertAlt ) ) {

            getVerbindungen( wertAlt ).setVariante( varianteNeu );
        } 
        
        if ( ! hatVorgegeben( wertNeu ) && hatVorgegeben( wertAlt ) ) {

            getVorgegeben( wertAlt ).setVariante( varianteNeu );
        }
    }
    
    /**
     * Wird aufgerufen,w enn sich die Variante eines Zaubers geaendert hat.
     * 
     * @param wertNeu Der Wert dessen Variante geaendert wurde.
     * @param varianteAlt Die Variante die der Wert bisher hatte.
     * @param varianteNeu Die neue Variante des Wertes.
     */
    private void varianteGeaendert(
            final ZauberWert wertNeu,
            final Variante varianteAlt,
            final Variante varianteNeu ) {

        final ZauberWert wertAlt = wertNeu.clone();
        wertAlt.setVariante( varianteAlt );
        
        if ( ! hatVorgegeben( wertNeu ) && hatVorgegeben( wertAlt ) ) {
            
            getVorgegeben(wertAlt).setVariante( varianteNeu );
        }
    }
    
    /**
     * @return {@code true}, wenn der Held gerade erschaffen wird, sonst
     *         {@code false}.
     */
    public final boolean wirdErschaffen() {
        return wirdErschaffen;
    }
}
