/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.ATTR_BE;
import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.ATTR_MINDEST_TORSO_RS;
import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.ATTR_NAME;
import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.ATTR_RS;
import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.ATTR_ZUSATZRUESTUNG;
import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.TAG_RUESTUNG;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public class Ruestung
extends ElementMitNameImpl {

    public static final String KETTENHEMD =
        I18n.i18n( "Ruestung.Kettenhemd.Name" );

    public static final String LANGES_KETTENHEMD =
        I18n.i18n( "Ruestung.LangesKettenhemd.Name" );

    /**
     * Prueft ob es sich bei dem String um die gueltige ID einer existierenden
     * Ruestung handelt.
     *  
     * @param string der ueberprueft werden soll.
     * @return {@code true}, wenn es sich um die ID einer Ruestung handelt.
     */
    public static final boolean istRuestungId( final String string ) {
        
        return Datenspeicher.getInstanz().enthaeltRuestung( string );
    }
    
    /** Die Behinderung. */
    private int be;

    /**
     * Gibt bei einer Zusatzruestung an wie hoch der Torso-RS sein muss, damit
     * der RS der Zusatzruestung angerechnet wird.
     */
    private int mindestTorsoRs;

    /** Der Ruestungsschutz. */
    private int rs;

    /** Gibt an ob es sich um eine Torsoruestung oder eine Zusatzruestung. */
    private boolean zusatzruestung;

    /**
     * Erzeugt eine Platzhalter Ruestung. Sie sollte nur verwendet werden, wenn
     * keine eingegebene Ruestung verfuegbar ist.
     */
    public Ruestung() {
        this( "RuestungDummy" );
    }
    
    public Ruestung( final String dieId ) {
        super( dieId );

        be = 1;

        mindestTorsoRs = 0;

        rs = 1;

        zusatzruestung = false;
    }

    @Override
    public boolean equals( final Object obj ) {
        
        if ( null == obj ) {
            
            return false;
        }

        if ( ! getClass().equals( obj.getClass() ) ) {

            return false;
        }
        
        Ruestung ruestung = (Ruestung) obj;
        
        return getId().equals( ruestung.getId() );
    }

    /**
     * @return die Behinderung
     */
    public final int getBe() {
        return be;
    }

    /**
     * @return Liefert bei einer Zusatzruestung, den RS den man am Torso
     *         besitzen muss, damit der RS der Zusatzruestung angerechnet
     *         wird.
     */
    public int getMindestTorsoRs() {
        return mindestTorsoRs;
    }

    /** @return den Ruestungsschutz. */
    public final int getRs() {
        return rs;
    }
    
    @Override
    public int hashCode() {
        
        return getId().hashCode();
    }

    /**
     * @return {@code true}, wenn es sich um einen Helm oder Ruestzeug handelt,
     *         {@code false}, wenn es sich um eine Torso-Ruestung handelt.
     */
    public final boolean istZusatzruestung() {
        return zusatzruestung;
    }

    /**
     * Aendert die Behinderung der Ruestung.
     *
     * @param neueBe Die neue Behinderung.
     */
    public final void setBe( final int neueBe ) {
        be = neueBe;
    }

    /**
     * @param torsoRs bei einer Zusatzruestung der RS, den man am Torso haben
     *        muss, um Nutzen aus der Zusatzruestung zu ziehen.
     */
    public void setMindestTorsoRs( final int torsoRs ) {
        mindestTorsoRs = torsoRs;
    }

    /**
     * Aendert den Ruestungsschutz der Ruestung.
     *
     * @param neuerRs Der neue Ruestungschutz.
     */
    public final void setRs( final int neuerRs ) {
        rs = neuerRs;
    }

    /**
     * @param istZusatzruestung
     *            {@code true}, wenn es sich um einen Helm oder Ruestzeug
     *            handelt, {@code false}, wenn es sich um eine Torso-Ruestung
     *            handelt.
     */
    public final void setZusatzruestung( final boolean istZusatzruestung ) {
        zusatzruestung = istZusatzruestung;
    }

    @Override
    public String toString() {
        return getName();
    }

    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_RUESTUNG );

        element.setAttribute( ATTR_BE, String.valueOf( be ) );

        element.setAttribute( ATTR_ID, getId() );

        if ( zusatzruestung ) {

            element.setAttribute( ATTR_MINDEST_TORSO_RS,
                                  String.valueOf( mindestTorsoRs ) );
        }

        if ( null != name ) {

            element.setAttribute( ATTR_NAME, name );

        }
        
        element.setAttribute( ATTR_RS, String.valueOf( rs ) );

        element.setAttribute( ATTR_ZUSATZRUESTUNG,
                              String.valueOf( zusatzruestung ) );

        return element;
    }
}
