/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.heldio;

import java.util.LinkedList;
import java.util.Queue;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

class ZauberTabelleDaten {

    /** Der Held dessen Zauber in die Tabelle eingefuegt werden. */
    public final Held held;

    /**
     * Die Zaubergruppen die noch in die Tabelle eingetragen werden muessen.
     */
    public final Queue< Zaubergruppe > zaubergruppen;

    /**
     * Die Zauber der aktuellen Gruppe die noch in die Tabelle
     * eingetragen werden muessen.
     */
    public final Queue< ZauberWert > zauber;

    /**
     * @param derHeld dessen Daten in die Tabelle eingetragen werden sollen.
     */
    ZauberTabelleDaten( final Held derHeld ) {

        held = derHeld;

        zaubergruppen = new LinkedList< Zaubergruppe >(
                Zaubergruppe.getZaubergruppen() );

        zauber = new LinkedList< ZauberWert >();

        sucheZaubergruppeMitZaubern();
    }

    /**
     * Sucht die naechste Gruppe deren Zauber in die Tabelle
     * einzutragen sind.
     */
    public final void naechsteZaubergruppe() {

        // Sicherheitshalber die Talentliste leeren.
        zauber.clear();

        // Die aktuelle Talentgruppe ist abgearbeitet, es soll zur folgenden
        // umgeschaltet werden.
        zaubergruppen.poll();

        sucheZaubergruppeMitZaubern();
    }

    /**
     * Durchlaeuft die Liste der Zaubergruppen und sucht die erste, in welcher
     * der Held Zauber besitzt. Die Zauber werden in {@link #zauber}
     * eingetragen.
     */
    private final void sucheZaubergruppeMitZaubern() {

        // So lange noch eine nicht bearbeitete Talentgruppe da ist
        // wird geprueft ob der Held Talente aus der Talentgruppe hat.
        while ( ! zaubergruppen.isEmpty() ) {

            final Zaubergruppe gruppe = zaubergruppen.peek();

            zauber.addAll( held.getZauber( gruppe ) );

            if ( zauber.isEmpty() ) {
                // Der Held besitzt keine Zauber in der Gruppe, die
                // Gruppe kann uebergangen werden.
                zaubergruppen.poll();

            } else {
                // Es gibt in dieser Gruppe Zauber.
                // Die naechste Gruppe ist somit gefunden.
                break;

            }
        }
    }

    /**
     * @see #hatNaechstenZauber()
     * @return liefert den naechsten Zauber welcher in die Tabelle einzutragen
     *      ist. Wenn kein Zauber mehr uebrig ist wird {@code null} geliefert.
     */
    public final ZauberWert getNaechstenZauber() {
        return zauber.poll();
    }

    /**
     * @see #hatZaubergruppe()
     * @return die Zaubergruppe deren Zauber von {@link #getNaechstenZauber()}
     *      geliefert werdern. Die Methode liefert {@code null} wenn alle
     *      Gruppen abgearbeitet wurden.
     */
    public final Zaubergruppe getZaubergruppe() {
        return zaubergruppen.peek();
    }

    /**
     * @return {@code true} wenn es in der aktuellen Zaubergruppe noch einen
     *      Zauber gibt, der in die Tabelle eingetragen werden muss, sonst
     *      {@code false}.
     */
    public final boolean hatNaechstenZauber() {
        return ! zauber.isEmpty();
    }

    /**
     * @return {@code true} wenn es noch eine Zaubergruppe gibt, deren Zauber
     *      in die Tabelle eingetragen werden muessen, sonst {@code false}.
     */
    public final boolean hatZaubergruppe() {
        return ! zaubergruppen.isEmpty();
    }
}
