/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.namen;

import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_LOKALISIERUNGSSCHLUESSEL;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_NAME;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.TAG_NAME;

import java.util.Comparator;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Util;

public class Name {

    public static final Comparator< Name > NAME_COMPARATOR =
        new Comparator< Name >() {

        public int compare( Name o1, Name o2 ) {
            
            return Util.compare( o1.toString(), o2.toString() );
        }
    };
    
    /**
     * Eine Id, ueber die der Name lokalisiert werden kann. Diese Angabe ist
     * optional, man kann einen Namen auch direkt eingeben, ohne dass er
     * lokalisiert werden kann, in dem Fall ist der Schluessel {@code null}.
     */
    private String lokalisierungsschluessel;

    /** Das eigentlichen Wort, das fuer den Namen verwendet wird. */
    private String name;

    public Name() {
        
        lokalisierungsschluessel = null;

        name = null;
    }

    /**
     * Sortiert Namen nach Name und Lokalisierungsschluessel.
     */
    public int compareTo( final Name other ) {
        
        int ergebnis = Util.compare( getName(), other.getName() );
        
        if ( ergebnis != 0 ) {
            
            return ergebnis;
        }
        
        String ls1 = getLokalisierungsschluessel();
        
        String ls2 = other.getLokalisierungsschluessel();
        
        if ( ( null == ls1 ) &&  ( null == ls2 ) ) {
            
            return 0;
        
        } else if ( null == ls1 ) {
            
            return 1;
            
        } else if ( null == ls2 ) {
            
            return -1;
            
        } else {
            
            return ls1.compareTo( ls2 );
            
        }
    }

    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime
                * result
                + ( ( lokalisierungsschluessel == null ) ? 0
                        : lokalisierungsschluessel.hashCode() );
        result = prime * result + ( ( name == null ) ? 0 : name.hashCode() );
        return result;
    }

    @Override
    public boolean equals( Object obj ) {
        if ( this == obj )
            return true;
        if ( obj == null )
            return false;
        if ( getClass() != obj.getClass() )
            return false;
        final Name other = (Name) obj;
        if ( lokalisierungsschluessel == null ) {
            if ( other.lokalisierungsschluessel != null )
                return false;
        } else if ( !lokalisierungsschluessel
                .equals( other.lokalisierungsschluessel ) )
            return false;
        if ( name == null ) {
            if ( other.name != null )
                return false;
        } else if ( !name.equals( other.name ) )
            return false;
        return true;
    }

    /**
     * @return Den Lokalisierungsschluessel, wenn es einen gibt. Wenn es keinen
     *         Schluessel gibt wird {@code null} geliefert.
     */
    public String getLokalisierungsschluessel() {
        
        return lokalisierungsschluessel;
    }
    
    /**
     * @return Den Namen. Es wird nie {@code null} geliefert.
     */
    public String getName() {
        
        if ( ( null != lokalisierungsschluessel ) ) {
            
            if ( I18n.hatLokalisierungstext( lokalisierungsschluessel ) ) {

                return I18n.i18n( lokalisierungsschluessel );
            
            } else {
                
                return lokalisierungsschluessel;
            }
        
        } else if ( null != name ) {
            
            return name;
        
        } else {
            
            return "???";
        }
    }
    
    /**
     * Legt den Loaklisierungsschluessel des Elements fest.
     * 
     * @param schluessel
     *            Ein Schluessel aus der Lokalisierungsdatei.
     */
    public void setLokalisierungsschluessel( String schluessel ) {
        
        lokalisierungsschluessel = schluessel;
    }

    /**
     * Legt den Namen fest, wenn dieser nicht ueber einen Schluessel aus der
     * Lokalisierungdatei geladen wird.
     * 
     * @param derName
     *            Der eigentliche Name.
     */
    public void setName( final String derName ) {
        
        name = derName;
    }

    /**
     * \return Den Namen.
     */
    @Override
    public String toString() {
        
        return getName();
    }
    
    /**
     * @return Die XML representation des Elements.
     */
    public IXMLElement toXml() {
        
        final IXMLElement element = new XMLElement( TAG_NAME );
        
        if ( null != lokalisierungsschluessel ) {
            
            element.setAttribute( 
                    ATTR_LOKALISIERUNGSSCHLUESSEL, 
                    lokalisierungsschluessel );
        
        } else if ( null != name ) {
            
            element.setAttribute( ATTR_NAME, name );
        }
        
        return element;
    }
}
