/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.namen;

import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_BEZEICHNUNG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.TAG_NAMENLISTE;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Random;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Util;

public class Namenliste {

    /**
     * Sortiert die Namenlisten alphabetisch nach Bezeichnung.
     */
    public static final Comparator< Namenliste > BEZEICHNUNG_COMPARATOR =
        new Comparator< Namenliste >() {
            
            public int compare( final Namenliste n1, final Namenliste n2 ) {
                
                return Util.compare( n1.getBezeichnung(), n2.getBezeichnung() );
            }
    };
    
    private static final String BEZEICHNUNG = ".Bezeichnung";
    
    /**
     * Die Bezeichnung der Liste z.B. "Vornamen (weiblich)".
     */
    private String bezeichnung;

    /** 
     * Eine Id zum identifizieren der Liste. Die Id muss nur unter den Listen
     * die zu einer Factory gehoeren eindeutig sein.
     */
    private String id;

    /**
     * Der Schluessel ueber den die Bezeichnung der Liste aus der 
     * Lokalisierungsdatei geladen wird. 
     */
    private final String lokalisierungsschluesselBezeichnung;
    
    /** Die eigentliche Liste der Namen. */
    private List< Name > namen;

    /** Ein Zufallselement, um einen Namen aus der Liste zu waehlen. */
    private Random random;
    
    public Namenliste( final String dieId ) {
        
        bezeichnung = null;

        id = dieId;
        
        lokalisierungsschluesselBezeichnung = id + BEZEICHNUNG;
        
        namen = new ArrayList< Name >();
        
        random = new Random();
    }
    
    /**
     * Sortiert die Liste nach Bezeichnung und Id.
     */
    public int compareTo( Namenliste o ) {
        
        int ergebnisId = getId().compareTo( o.getId() );
        
        if ( 0 == ergebnisId )
        {
            return 0;
        }
        
        int ergebnisBezeichnung = 
            Util.compare( getBezeichnung(),o.getBezeichnung() );
        
        if ( 0 != ergebnisBezeichnung )
        {
            return ergebnisBezeichnung;
        }
        
        return ergebnisId;
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj )
        {
            return false;
        }
        
        if ( this == obj )
        {
            return true;
        }
        
        if ( getClass() != obj.getClass() )
        {
            return false;
        }
        
        Namenliste andere = (Namenliste) obj;
        
        return 0 == compareTo( andere );
    }
    
    /**
     * @return Die Bezeichnung der Liste.
     */
    public String getBezeichnung() {

        if ( null != bezeichnung ){
            
            return bezeichnung;
        
        } else if ( I18n.hatLokalisierungstext( lokalisierungsschluesselBezeichnung ) ) {
            
            return I18n.i18n( lokalisierungsschluesselBezeichnung );
        
        } else {
            
            return id;
        }
    }
    
    /** 
     * @return Die ID der Liste. Die ID ist nur unter den Listen einer Factory
     *         eindeutig.
     */
    public String getId()
    {
        return id;
    }
    
    /**
     * @return Die Namen in der Liste.
     */
    public List< Name > getNamen() {
        
        return new ArrayList< Name >( namen );
    }
    
    /**
     * @return Liefert einen Namen aus der Liste. Liefert einen leeren String,
     *         wenn die Liste leer ist.
     */
    public String getZufaelligenNamen() {
        
        if ( namen.isEmpty() ) {
            
            return "";
        }
        
        int size = namen.size();
        
        int index = random.nextInt( size );
        
        return namen.get( index ).toString();
    }
    
    /**
     * @return Den Hash Code der ID.
     */
    @Override
    public int hashCode() {
        
        return id.hashCode();
    }
    
    /**
     * Legt die Bezeichnung der Liste fest.
     */
    public void setBezeichnung( String dieBezeichnung ) {

        dieBezeichnung = dieBezeichnung.trim();
        
        if ( ( null == dieBezeichnung ) || dieBezeichnung.equals( "" ) ) {
            
            bezeichnung = null;
        
        } else if ( I18n.hatLokalisierungstext( lokalisierungsschluesselBezeichnung ) ) {
            
            final String lokalisierung = I18n.i18n( lokalisierungsschluesselBezeichnung );
            
            if ( lokalisierung.equals( dieBezeichnung ) ) {
                
                bezeichnung = null;
            
            } else {
                
                bezeichnung = dieBezeichnung;
            }
        
        } else {
            
            bezeichnung = dieBezeichnung;
        }
    }
    
    /**
     * Legt die Namen der Liste fest.
     * 
     * @param neueNamen Die Namen der Liste.
     */
    public void setNamen( final List< Name > neueNamen ) {
        
        namen.clear();
        
        namen.addAll( neueNamen );
    }
    
    /**
     * @return Die Bezeichnung der Liste.
     */
    @Override
    public String toString() {
        
        return getBezeichnung();
    }
    
    /**
     * Wandelt die Liste in eine XML Repräsentation um, damit sie so gespeichert
     * werden kann.
     * 
     * @return Die XML Repräsentation der Liste.
     */
    public IXMLElement toXml() {
    
        IXMLElement element = new XMLElement( TAG_NAMENLISTE );
        
        element.setAttribute( ATTR_ID, id );
            
        if ( null != bezeichnung
                && ( ! id.equals( bezeichnung ) )
                && ( ! I18n.hatLokalisierungstext( lokalisierungsschluesselBezeichnung ) 
                    || ! I18n.i18n( lokalisierungsschluesselBezeichnung ).equals( bezeichnung ) ) ) {
            
            element.setAttribute( ATTR_BEZEICHNUNG, bezeichnung );
        }
        
        for ( final Name name : namen ) {
            
            element.addChild( name.toXml() );
        }
        
        return element;
    }    
}
