/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.Comparator;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Util;

public class Variante {

    /**
     * Sortiert die Varianten alphabetische nach Name.
     */
    public static final Comparator< Variante > NAME_COMPARATOR =
        new Comparator< Variante >() {

            public int compare(
                    final Variante v1,
                    final Variante v2 ) {
                
                return Util.compare( v1.getName(), v2.getName() );
            }
        
    };
    
    /** 
     * Hier kann ein Regelelement, z.B. ein Talent oder eine Ruestung 
     * gespeichert werden.
     */
    private final ElementMitName element;
    
    /** 
     * Wenn die Variante ein lokalisierbarer Text ist dies der 
     * Lokalisierungsschluessel ueber den der Name der Variante aus der
     * Lokalisierungsdatei geladen wird. Wenn die Variante aus nur aus einem 
     * nicht lokalisierten Text besteht wird hier der Text gespeichert.  
     */
    private final String schluesselOderName;
    
    public Variante() {
        
        element = null;
        
        schluesselOderName = "";
    }

    public Variante( final ElementMitName dasElement ) {
        
        element = dasElement;
        
        schluesselOderName = "";
    }
    
    /**
     * Hier kann entweder ein Lokalisierungsschluessel oder ein Name uebergeben
     * werden. Es geht auch die ID eines Elements mit Name.
     */
    public Variante( final String id ) {
        
        if ( ElementMitNameImpl.istId( id ) ) {
            
            element = ElementMitNameImpl.getElement( id );
            
            schluesselOderName = "";
        
        } else {
            
            element = null;
            
            if ( I18n.hatLokalisierungstext( id + ".Name" ) ) {
                
                schluesselOderName = id + ".Name";
            
            } else {
                
                schluesselOderName = id;
            }
        }
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj ) {
            
            return false;
        }
        
        if ( ! getClass().equals( obj.getClass() ) ) {
            
            return false;
        }
        
        final Variante variante = (Variante) obj;
        
        if ( hatElementMitName() && variante.hatElementMitName() ) {
            
            return element.equals( variante.element );
        
        } else if ( hatSchluesselOderName() && variante.hatSchluesselOderName() ) {
            
            return schluesselOderName.equals( variante.schluesselOderName );
        
        } else if ( isNull() && variante.isNull() ) {
            
            return true;
        
        } else {
            
            return false;
        }
    }
    
    /**
     * @return Das Element (Talent, Vorteil...) aus dem die Variante beseteht.
     *         Wenn die Variante nur aus einem Text besteht wird {@code null}
     *         zurueckt gegeben.  
     */
    public final ElementMitName getElement() {
        
        return element;
    }

    /**
     * @return Eine ID, um das Element zu identifizieren. Es ist entweder
     *          der SchluesselOderName oder die ID des ElementMitName.
     */
    public final String getId() {
        
        if ( hatSchluesselOderName() ) {
            
            return schluesselOderName;
        
        } else {
            
            return element.getId();
        }
    }
    
    /**
     * @return Die Bezeichnung der Talentspezialisierung.
     */
    public final String getName() {
        
        if ( hatElementMitName() ) {
            
            return element.getName();
            
        } else if ( I18n.hatLokalisierungstext( schluesselOderName ) ) {
            
            return I18n.i18n( schluesselOderName );
        
        } else {
            
            return schluesselOderName;
        }
    }
    
    /**
     * @return Der Schluessel der zur Lokalisierung des Namens der Variante
     *         verwendet wird. Es kann aber auch einfach direkt der Name der
     *         Variante sein, wenn es keinen Lokalisierungsschluessel gibt.
     */
    public final String getSchluesselOderName() {
        
        return schluesselOderName;
    }
    
    @Override
    public int hashCode() {
        
        if ( hatElementMitName() ) {
            
            return 17 + element.hashCode();
        
        } else {
            
            return 17 + schluesselOderName.hashCode();
        }
    }
    
    /**
     * @return {@code true}, wenn die Variante aus einem Regelelement wie einem
     *          Talent besteht.
     */
    public final boolean hatElementMitName() {
        
        return null != element;
    }
    
    /**
     * @return {@code true}, wenn ein Text als Lokalisierungsschluessel oder 
     *          Name gesetzt ist, {@code false} sonst.
     */
    public final boolean hatSchluesselOderName() {
        
        return ! schluesselOderName.equals( "" );
    }
    
    /**
     * @return {@code true}, wenn es sich bei der Variante um die leere Variante
     *         handelt, d.h. eine Variante ohne Element mit leerem Text.
     */
    public boolean isNull() {
        
        if ( hatElementMitName() ) {
            
            return false;
        
        } else {
            
            return schluesselOderName.equals( "" );
        }
    }
    
    /**
     * @return Liefert den Namen der Spezialisierung.
     */
    @Override
    public String toString() {
        
        return getName();
    }
}
