/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basiswerte;

import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.CH;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.FF;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.GE;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.IN;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KK;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KL;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KO;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.MU;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.basis.Steigerbar;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class Basiswert
extends ElementMitNameImpl
implements Comparable< Basiswert >, Steigerbar {

    /** Eine sortierte Liste aller Basiswerte. */
    private static final Basiswert[] BASISWERTE;

    /** Eine Map in der jeder Basiswert unter seiner Id gespeichert ist. */
    private static final Map< String, Basiswert > BASISWERTE_MAP =
        new HashMap< String, Basiswert >();

    /**
     * Beschreibt dem Benutzer, dass maximal x Punkte zum Basiswert
     * hinzugekauft werden duerfen.
     */
    private static final String MAX_ZUKAUF =
        I18n.i18n( "Basiswert.MaxZukauf" );

    /** Der Basiswert Astralenergie */
    public static final Basiswert AE = new Astralenergie(
            "Basiswert.Astralenergie", 3,
            new Formel( 2F, MU, IN, CH ),
            SKT.G,
            new MaxZukauf( Eigenschaft.CH, 1F ) );

    /** Der Basiswert Attacke-Basiswert */
    public static final Basiswert ATB = new Basiswert(
            "Basiswert.Attacke", 5,
            new Formel( 5F, MU, GE, KK ) );

    /** Der Basiswert Ausdauer */
    public static final Basiswert AU = new Ausdauer(
            "Basiswert.Ausdauer", 1,
            new Formel( 2F, MU, KO, GE ),
            SKT.E,
            new MaxZukauf( Eigenschaft.KO, 1F ) );

    /** Der Basiswert Fernkampf-Basiswert */
    public static final Basiswert FKB = new Basiswert(
            "Basiswert.Fernkampf", 7,
            new Formel( 5F, IN, FF, KK ) );

    /** Der Basiswert Initiative-Basiswert */
    public static final Basiswert INI = new Initiative(
            "Basiswert.Initiative", 8,
            new Formel( 5F, MU, MU, IN, GE ) );

    /** Der Basiswert Lebensenergie. */
    public static final Basiswert LE = new Basiswert(
            "Basiswert.Lebensenergie", 0,
            new Formel( 2F, KO, KO, KK ),
            SKT.H,
            new MaxZukauf( Eigenschaft.KO, 2F ) );

    /** Der Basiswert Magieresistenz. */
    public static final Basiswert MR = new Basiswert(
            "Basiswert.Magieresistenz", 2,
            new Formel( 5F, MU, KL, KO ),
            SKT.H,
            new MaxZukauf( Eigenschaft.MU, 2F ) );

    /** Der Basiswert Parade-Basiswert */
    public static final Basiswert PAB = new Basiswert(
            "Basiswert.Parade", 6,
            new Formel( 5F, IN, GE, KK ) );

    /** Der Basiswert Wundschwelle. */
    public static final Basiswert WUNDSCHWELLE = new Basiswert(
            "Basiswert.Wundschwelle", 4,
            new Formel( 2F, KO ) );

    static {

        AE.setNachteilMalus( VorNachteil.NIEDRIGE_ASTRALKRAFT );
        AE.setVorteilBonus( VorNachteil.ASTRALMACHT );

        LE.setNachteilMalus( VorNachteil.NIEDRIGE_LEBENSKRAFT );
        LE.setVorteilBonus( VorNachteil.HOHE_LEBENSKRAFT );

        MR.setNachteilMalus( VorNachteil.NIEDRIGE_MAGIERESISTENZ );
        MR.setVorteilBonus( VorNachteil.HOHE_MAGIERESISTENZ );

        BASISWERTE = new Basiswert[]{
                LE, AU, MR, AE, WUNDSCHWELLE, ATB, PAB, FKB, INI };

    }

    /**
     * Liefert einen Basiswert.
     *
     * @param id des gesuchten Basiswert.
     * @return den zur Id gehoerenden Basiswert.
     * @throws IllegalArgumentException wenn es keinen Basiswert mit der
     *      uebergebenen Id gib.t
     */
    public static final Basiswert getBasiswert( final String id ) {

        if ( BASISWERTE_MAP.containsKey( id ) ) {

            return BASISWERTE_MAP.get( id );

        } else {

            throw new IllegalArgumentException(
                    "Es existiert kein Basiswert mit der Id " + id );

        }
    }

    /**
     * @return eine sortierte Liste aller Basiswerte.
     */
    public static final Basiswert[] getBasiswerte() {
        return BASISWERTE.clone();
    }

    /** Initialisiert die Basiswerte vollstaendig */
    public static final void initialisieren() {

        WUNDSCHWELLE.modifikationen.put( VorNachteil.GLASKNOCHEN, -2 );
        WUNDSCHWELLE.modifikationen.put( VorNachteil.EISERN, 2 );

        MR.modifikationen.put( VorNachteil.VOLLZAUBERER, 2 );

        AE.modifikationen.put( VorNachteil.VOLLZAUBERER, 12 );
        AE.modifikationen.put( VorNachteil.VIERTELZAUBERER, -6 );
        AE.modifikationen.put( VorNachteil.VIERTELZAUBERER_UNBEWUSST, -6 );

    }

    /**
     * Die Formel zum Berechnen des Basiswerts.
     */
    private final Formel formel;

    /**
     * Der Index zum sortieren der Basiswerte.
     * Die Basiswerte werden auf dem Charakterbogen nich alphabetisch
     * sortiert.
     */
    private final int index;

    /** Moegliche Modifikationen fuer den Basiswert. */
    private final Map< VorNachteil, Integer > modifikationen;

    /**
     * Ein Nachteil dessen Stufe als Malus auf den Basiswert gerechnet wird.
     * Beispiel: Niedrige Lebenkraft.
     */
    private VorNachteil nachteilMalus;

    /** Der Steigerungsfaktor nach dem der Basiswert gesteigert werden kann. */
    private final int steigerungsfaktor;

    /**
     * Ein Vorteil dessen Stufe als Bonus auf den Basiswert gerechnet wird.
     * Beispiel: Hohe Lebenskraft.
     */
    private VorNachteil vorteilBonus;

    private final MaxZukauf zukauf;
    /**
     *
     * @param dieId des Basiswerts.
     * @param dasKuerzel des Basiswerts.
     * @param derIndex zum sortieren des Basiswerts.
     * @param dieFormel zur Berechnung des Basiswerts.
     */
    Basiswert(
            final String dieId,
            final int derIndex,
            final Formel dieFormel ) {

        this ( dieId, derIndex, dieFormel, -1, null );
    }

    /**
     *
     * @param dieId
     *            des Basiswerts.
     * @param dasKuerzel
     *            des Basiswerts.
     * @param derIndex
     *            zum sortieren des Basiswerts.
     * @param dieFormel
     *            zur Berechnung des Basiswerts.
     * @param derSteigerungsfaktor
     *            nach dem der Basiswert gesteigert wird.
     * @param derZukauf
     *            das Objekt, das bestimmt wie viele Punkte maximal zur
     *            Startstufe des Basiswerts hinzugekauft werden koennen.
     */
    Basiswert(
            final String dieId,
            final int derIndex,
            final Formel dieFormel,
            final int derSteigerungsfaktor,
            final MaxZukauf derZukauf ) {

        super( dieId );

        index = derIndex;

        modifikationen = new HashMap< VorNachteil, Integer >();

        formel = dieFormel;

        steigerungsfaktor = derSteigerungsfaktor;

        zukauf = derZukauf;

        BASISWERTE_MAP.put( dieId, this );
    }

    /**
     * Sortiert die Werte anhand ihres Indexes.
     *
     * @return eine Zahl kleiner 0 wenn {@code this} &lt; {@code anderer},
     *      0 wenn {@code this} == {@code anderer},
     *      eine Zahl groesser 0 wenn {@code this} &gt; {@code anderer}.
     */
    public final int compareTo( final Basiswert anderer ) {
        return getIndex() - anderer.getIndex();
    }

    /**
     * @return die Stufe des Basiswertes ohne die Modifikationen. Der Wert, der
     *         durch Berechnung der Formel entsteht.
     */
    public int getBasisstufe( final Held held ) {
        return formel.berechnen( held );
    }

    /**
     * @return die zum Berechnen des Basiswerts benoetigten Eigenschaften.
     */
    public final Collection< Eigenschaft > getFormelEigenschaften() {
        return formel.getEigenschaften();
    }

    /**
     * @return der sortierindex des Basiswerts.
     */
    public final int getIndex() {
        return index;
    }

    /** Die Puntke die maximal zum Startwert hinzugekauft werden koennen. */
    final int getMaxZukauf( final Held held ) {
        if ( null != zukauf ) {

            return zukauf.getMaxZukauf( held );

        } else {

            return 0;
        }
    }

    /**
     * @param held dessen Modifikation gefragt ist.
     * @return die Modifikation die der Held auf den Basiswert erhaelt.
     */
    public int getModifikation( final Held held ) {

        int modifikation = 0;

        if ( held.getRasse().hatModifikation( this ) ) {

            modifikation += held.getRasse().getModifikation( this );

        }

        if ( held.getKultur().hatModifikation( this ) ) {

            modifikation += held.getKultur().getModifikation( this );

        }

        if ( held.getProfession().hatModifikation( this ) ) {

            modifikation += held.getProfession().getModifikation( this );

        }

        if ( ( null != vorteilBonus ) && held.hat( vorteilBonus ) ) {

            final VorNachteilWert wert = held.getWert( vorteilBonus );

            modifikation += wert.getStufe();
        }

        if ( ( null != nachteilMalus ) && held.hat( nachteilMalus ) ) {

            final VorNachteilWert wert = held.getWert( nachteilMalus );

            modifikation -= wert.getStufe();
        }

        for ( final VorNachteil v : modifikationen.keySet() ) {

            if ( held.hat( v ) ) {

                modifikation += modifikationen.get( v );
            }
        }

        return modifikation;
    }

    /** {@inheritDoc} */
    public final int getSteigerungsspalte() {
        return steigerungsfaktor;
    }

    /**
     * @param wert eines Helden.
     * @return die Stufe des Wertes mit allen Modifikationen.
     */
    public int getStufe( final BasiswertWert wert ) {

        return wert.getBasisstufe()
             + wert.getModifikation()
             + wert.getZukauf()
             + wert.getBonus();
    }

    /**
     * @return {@code true}, wenn die Stufe des Basiswerts durch Zukaufen von
     *         Punkten erhoeht werden darf, sonst {@code false}.
     */
    public boolean istSteigerbar() {
        return ( -1 != steigerungsfaktor );
    }

    /**
     * Prueft ob ein Held die Voraussetzungen fuer einen Basiswert erfuellt.
     *
     * @param held
     *            der den Basiswert besitzt.
     * @param wert
     *            dessen Voraussetzungen ueberprueft werden sollen.
     * @return die Beschreibung der aufgetretenen Regelprobleme.
     */
    public KonfliktBeschreibung pruefen(
            final Held held,
            final BasiswertWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        final int maxZukauf = wert.getMaxZukauf();

        final int zukauf = wert.getZukauf();

        if ( maxZukauf < zukauf ) {

            beschreibung.einfuegen( String.format( MAX_ZUKAUF, maxZukauf ) );
        }

        return beschreibung;
    }

    /**
     * @param vorteil
     *            dessen Stufe als Bonus auf den Basiswert gerechnet wird.
     */
    private void setVorteilBonus( final VorNachteil vorteil ) {

        vorteilBonus = vorteil;
    }

    /**
     * @param nachteil
     *            dessen Stufe als Malus auf den Basiswert gerechent wird.
     */
    private void setNachteilMalus( final VorNachteil nachteil ) {

        nachteilMalus = nachteil;
    }
}
