/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten;

import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_KULTUR;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_PROFESSION;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_RASSE;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_FERNKAMPFWAFFEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_NAHKAMPFWAFFEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_NAMENLISTEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_NAME_FACTORYS;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_RUESTUNGEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEI_SCHILDE;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.VERZEICHNIS_KULTUREN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.VERZEICHNIS_PROFESSIONEN;
import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.VERZEICHNIS_RASSEN;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.StdXMLReader;
import net.n3.nanoxml.XMLParserFactory;
import de.bernhardjung.dsaprogramm.daten.factorys.Factory;
import de.bernhardjung.dsaprogramm.daten.factorys.FactoryFernwaffen;
import de.bernhardjung.dsaprogramm.daten.factorys.FactoryNahkampfwaffen;
import de.bernhardjung.dsaprogramm.daten.factorys.FactoryNameFactorys;
import de.bernhardjung.dsaprogramm.daten.factorys.FactoryNamenlisten;
import de.bernhardjung.dsaprogramm.daten.factorys.FactoryRuestungen;
import de.bernhardjung.dsaprogramm.daten.factorys.FactorySchilde;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class Datenlader {

    private static final String RESOURCE_FERNKAMPFWAFFEN =
        "diverses/Fernkampfwaffen.xml";

    private static final String RESOURCE_NAHKAMPFWAFFEN =
        "diverses/Nahkampfwaffen.xml";

    private static final String RESOURCE_NAME_FACTORYS =
        "diverses/NameFactorys.xml";

    private static final String RESOURCE_NAMENLISTEN =
        "diverses/Namenlisten.xml";

    private static final String RESOURCE_RUESTUNGEN =
        "diverses/Ruestungen.xml";

    private static final String RESOURCE_SCHILDE =
        "diverses/Schilde.xml";

    /**
     * @throws Exception
     *             wenn der XML-Parser zum Laden der Daten nicht initialisiert
     *             werden kann.
     */
    public static final void ladeDaten() throws Exception {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        String datenquelle = einstellungen.getDatenquelle();

        if ( ! Einstellungen.DATENQUELLE_INTERN.equals( datenquelle ) ) {

            final File basisverzeichnis = new File( datenquelle );

            if ( ! basisverzeichnis.exists() ) {

                datenquelle = Einstellungen.DATENQUELLE_INTERN;

                einstellungen.setDatenquelle( Einstellungen.DATENQUELLE_INTERN);
            }

        }

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        final RassenLader rassenLader = new RassenLader();

        final KulturenLader kulturenLader = new KulturenLader();

        final ProfessionenLader professionenLader = new ProfessionenLader();

        final FactoryFernwaffen fernkampfwaffen = new FactoryFernwaffen();

        final FactoryNahkampfwaffen nahkampfwaffen =
            new FactoryNahkampfwaffen();

        final FactoryNameFactorys nameFactorys = new FactoryNameFactorys();

        final FactoryNamenlisten namenlisten = new FactoryNamenlisten();
        
        final FactoryRuestungen ruestungen = new FactoryRuestungen();

        final FactorySchilde schilde = new FactorySchilde();

        if ( Einstellungen.DATENQUELLE_INTERN.equals( datenquelle ) ) {

            speicher.setFernwaffen( ladeAusJar(
                    RESOURCE_FERNKAMPFWAFFEN,
                    fernkampfwaffen ) );

            speicher.setNahkampfwaffen( ladeAusJar(
                    RESOURCE_NAHKAMPFWAFFEN,
                    nahkampfwaffen ) );

            speicher.setNameFactorys( ladeAusJar(
                    RESOURCE_NAME_FACTORYS,
                    nameFactorys ) );

            speicher.setNamenlisten( ladeAusJar(
                    RESOURCE_NAMENLISTEN,
                    namenlisten ) );

            speicher.setRuestungen( ladeAusJar(
                    RESOURCE_RUESTUNGEN,
                    ruestungen ) );

            speicher.setSchilde( ladeAusJar(
                    RESOURCE_SCHILDE,
                    schilde ) );

            // Rassen, Kulturen und Professionen nach allem anderen Laden,
            // sonst werden die Varianten von Werten nicht gefunden.
            speicher.setProfessionen( professionenLader.ladeAusJar() );

            speicher.setKulturen( kulturenLader.ladeAusJar() );

            speicher.setRassen( rassenLader.ladeAusJar() );

        } else {

            final File basisverzeichnis = new File( datenquelle );

            // ***************
            // Fernkampfwaffen
            // ***************
            final File dateiFernwaffen =
                new File( basisverzeichnis, DATEI_FERNKAMPFWAFFEN );

            if ( dateiFernwaffen.exists() ) {

                speicher.setFernwaffen(
                        lade( dateiFernwaffen, fernkampfwaffen ) );

            } else {

                speicher.setFernwaffen( ladeAusJar(
                        RESOURCE_FERNKAMPFWAFFEN,
                        fernkampfwaffen ) );

                Datenspeicherer.speichereFernkampfwaffen();
            }

            // **************
            // Nahkampfwaffen
            // **************
            final File dateiNahkampfwaffen =
                new File( basisverzeichnis, DATEI_NAHKAMPFWAFFEN );

            if ( dateiNahkampfwaffen.exists() ) {

                speicher.setNahkampfwaffen( lade(
                        dateiNahkampfwaffen,
                        nahkampfwaffen ) );

            } else {

                speicher.setNahkampfwaffen( ladeAusJar(
                        RESOURCE_NAHKAMPFWAFFEN,
                        nahkampfwaffen ) );

                Datenspeicherer.speichereNahkampfwaffen();
            }

            // **************
            // NameFactorys
            // **************
            final File dateiNameFactorys =
                new File( basisverzeichnis, DATEI_NAME_FACTORYS );

            if ( dateiNameFactorys.exists() ) {

                speicher.setNameFactorys( lade(
                        dateiNameFactorys,
                        nameFactorys ) );

            } else {

                speicher.setNameFactorys( ladeAusJar(
                        RESOURCE_NAME_FACTORYS,
                        nameFactorys ) );

                Datenspeicherer.speichereNameFactorys();
            }

            // **************
            // Namenlisten
            // **************
            final File dateiNamenlisten =
                new File( basisverzeichnis, DATEI_NAMENLISTEN );

            if ( dateiNamenlisten.exists() ) {

                speicher.setNamenlisten( lade(
                        dateiNamenlisten,
                        namenlisten ) );

            } else {

                speicher.setNamenlisten( ladeAusJar(
                        RESOURCE_NAMENLISTEN,
                        namenlisten ) );

                Datenspeicherer.speichereNamenlisten();
            }

            // **********
            // Ruestungen
            // **********
            final File dateiRuestungen =
                new File( basisverzeichnis, DATEI_RUESTUNGEN );

            if ( dateiRuestungen.exists() ) {

                speicher.setRuestungen( lade(
                        dateiRuestungen,
                        ruestungen ) );

            } else {

                speicher.setRuestungen( ladeAusJar(
                        RESOURCE_RUESTUNGEN,
                        ruestungen ) );

                Datenspeicherer.speichereRuestungen();
            }

            // *******
            // Schilde
            // *******
            final File dateiSchilde =
                new File( basisverzeichnis, DATEI_SCHILDE );

            if ( dateiSchilde.exists() ) {

                speicher.setSchilde( lade(
                        dateiSchilde,
                        schilde ) );

            } else {

                speicher.setSchilde( ladeAusJar(
                        RESOURCE_SCHILDE,
                        schilde ) );

                Datenspeicherer.speichereSchilde();
            }

            // ********
            // Kulturen
            // Nach allem anderen laden, damit bei Variante von Werten die
            // IDs von ElementenMitName im Datenspeicher gefunden werden.
            // ********
            final File verzeichnisKulturen =
                new File( basisverzeichnis, VERZEICHNIS_KULTUREN );

            if ( ! verzeichnisKulturen.exists() ) {

                speicher.setKulturen( kulturenLader.ladeAusJar() );

                Datenspeicherer.speichernHeldenbaustein(
                        speicher.getKulturen(),
                        verzeichnisKulturen,
                        DATEIERWEITERUNG_KULTUR );
            } else {

                speicher.setKulturen(
                        kulturenLader.laden( verzeichnisKulturen ) );
            }

            // ************
            // Professionen
            // Nach allem anderen laden, damit bei Variante von Werten die
            // IDs von ElementenMitName im Datenspeicher gefunden werden.
            // ************
            final File verzeichnisProfessionen =
                new File( basisverzeichnis, VERZEICHNIS_PROFESSIONEN );

            if ( ! verzeichnisProfessionen.exists() ) {

                speicher.setProfessionen( professionenLader.ladeAusJar() );

                Datenspeicherer.speichernHeldenbaustein(
                        speicher.getProfessionen(),
                        verzeichnisProfessionen,
                        DATEIERWEITERUNG_PROFESSION );

            } else {

                speicher.setProfessionen(
                        professionenLader.laden( verzeichnisProfessionen));

            }

            // ******
            // Rassen
            // Nach allem anderen laden, damit bei Variante von Werten die
            // IDs von ElementenMitName im Datenspeicher gefunden werden.
            // ******
            final File verzeichnisRassen =
                new File( basisverzeichnis, VERZEICHNIS_RASSEN );

            if ( ! verzeichnisRassen.exists() ) {

                speicher.setRassen( rassenLader.ladeAusJar() );

                Datenspeicherer.speichernHeldenbaustein(
                        speicher.getRassen(),
                        verzeichnisRassen,
                        DATEIERWEITERUNG_RASSE );

            } else {

                speicher.setRassen(
                        rassenLader.laden( verzeichnisRassen ) );
            }

        }
    }

    @SuppressWarnings( "unchecked" )
    private static final <T> Collection< T > lade(
            final File datei,
            final Factory< T > factory ) {

        try {
            final InputStream in = new FileInputStream( datei );

            return lade( in, factory );

        } catch( final Exception e ) {

            // Wir haben ja gerade getestet ob die Datei da ist.
            // Kann alsoe eigentlich nicht passieren.
            DialogFehlermeldung.zeigeDialog( String.format(
                    "Datei %1$s konnte nicht geladen werden.",
                    datei.getName() ), e );
        }

        return Collections.EMPTY_LIST;
    }

    @SuppressWarnings( "unchecked" )
    private static final <T> Collection< T > lade(
            final InputStream in,
            final Factory< T > factory ) throws Exception {

        final IXMLReader reader = new StdXMLReader(
                new BufferedInputStream( in ) );

        final IXMLParser parser = XMLParserFactory.createDefaultXMLParser();

        parser.setReader( reader );

        final IXMLElement wurzelElement = (IXMLElement) parser.parse();

        return factory.erzeugen( wurzelElement );
    }

    @SuppressWarnings( "unchecked" )
    private static final <T> Collection< T > ladeAusJar(
            final String name,
            final Factory< T> factory ) {

        final InputStream in;
        in = KulturenLader.class.getResourceAsStream( name );

        if ( null != in ) {

            try {
                return lade( in, factory );
            } catch (final Exception e) {
                DialogFehlermeldung.zeigeDialog(
                        name +
                        " konnte nicht aus der Jar-Datei geladen werden.",
                        e );
            }
        }

        return Collections.EMPTY_LIST;
    }
}
