/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten;

import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_KULTUR;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_NAME_FACTORY;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_ALLE_VON;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_LEITZAUBER;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_MUTTERSPRACHE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_PROFESSIONEN;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_ZWEITSPRACHEN;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.StdXMLReader;
import net.n3.nanoxml.XMLParserFactory;
import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitWertBonus;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionen;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionenAlleAusser;
import de.bernhardjung.dsaprogramm.bausteine.moeglicheprof.MoeglicheProfessionenAlleVon;
import de.bernhardjung.dsaprogramm.daten.factorys.bausteine.FactoryNameIn;
import de.bernhardjung.dsaprogramm.daten.factorys.bausteine.FactorySonderfertigkeitWertIn;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class KulturenLader extends HeldenbausteinLader< Kultur > {

    /** Die Kultur-Dateien die mit der Jar-Datei ausgeliefert werden. */
    private static final String[] KULTUR_DATEIEN = {
        "Ambosszwerge.kul",
        "AndergastNostria.kul",
        "Auelfen.kul",
        "AuelfenHalbelf.kul",
        "Bornland.kul",
        "Horasreich.kul",
        "MittellaendischeStaedte.kul",
        "Novadis.Frauen.kul",
        "Novadis.Maenner.kul",
        "Suedaventurien.kul",
        "Thorwal.kul",
        "TulamidischeStadtstaaten.kul"
    };

    /** Der Parser zum Einlesen der XML-Dateien. */
    private final IXMLParser parser;

    /** Erzeugt die Leitsonderfertigkeite aus dem XML-Dokument. */
    private final FactorySonderfertigkeitWertIn leitsonderfertigkeiten;

    /** Erzeugt den Namen der Kultur aus dem XML-Dokument. */
    private final FactoryNameIn name;

    /**
     * @throws Exception
     *             wenn es beim Initialisieren des XML-Parsers zu einem Fehler
     *             kommt.
     */
    public KulturenLader() throws Exception {

        parser = XMLParserFactory.createDefaultXMLParser();

        leitsonderfertigkeiten = new FactorySonderfertigkeitWertIn();

        name = new FactoryNameIn();
    }

    /**
     * @param wurzelElement
     *            der XML-Datei.
     * @return die aus der XML-Datei erzeugte Kultur.
     */
    private final Kultur erzeugeKultur(
            final IXMLElement wurzelElement ) {

        final Kultur kultur = erzeugeBaustein( wurzelElement );

        kultur.setName( name.erzeugen( wurzelElement, kultur.getId() ) );
        
        //**********************
        // NameFactory ermitteln
        //**********************
        if ( wurzelElement.hasAttribute( ATTR_NAME_FACTORY ) ) {
            
            final String id = wurzelElement.getAttribute( ATTR_NAME_FACTORY );
            
            Datenspeicher speicher = Datenspeicher.getInstanz();
            
            final NameFactory factory = speicher.getNameFactory( id );
            
            kultur.setNameFactory( factory );
        
        } else {
            
            kultur.setNameFactory( null );
        }

        //************************
        // Muttersprache ermitteln
        //************************
        final String mutterspracheId =
            wurzelElement.getFirstChildNamed( TAG_MUTTERSPRACHE ).getContent();
        final Sprache muttersprache =
            (Sprache) Talent.getTalent( mutterspracheId );
        kultur.setMuttersprache( muttersprache );


        //************************
        // Zweitsprachen ermitteln.
        //************************
        final IXMLElement zweitsprachenElement =
            wurzelElement.getFirstChildNamed( TAG_ZWEITSPRACHEN );

        final List< Sprache > zweitsprachen = new ArrayList< Sprache >(2);
        if ( null != zweitsprachenElement ) {

            for ( final IXMLElement e : zweitsprachenElement.getChildren() ) {

                zweitsprachen.add( (Sprache) Talent.getTalent( e.getContent() ) );
            }

        }
        kultur.setZweitsprachen( zweitsprachen );

        // *********************************
        // Moegliche Professionen ermitteln.
        //**********************************
        final MoeglicheProfessionen professionen =
            erzeugeMoeglicheProfessionen( wurzelElement );

        kultur.setProfessionen( professionen );

        // ******************************************
        // Anzahl zusaetzlicher Leitzauber ermitteln.
        //*******************************************
        final IXMLElement leitzauberElement =
            wurzelElement.getFirstChildNamed( TAG_LEITZAUBER );

        if ( null != leitzauberElement ) {

            final int anzahl = Integer.valueOf( leitzauberElement.getContent() );

            kultur.setAnzahlLeitzauber( anzahl );

        } else {

            kultur.setAnzahlLeitzauber( 0 );
        }

        // *************************************
        // Die Leitsonderfertigkeiten ermitteln.
        //**************************************
        final List< SonderfertigkeitWertBonus > leitsf =
            leitsonderfertigkeiten.erzeugen( wurzelElement );

        kultur.setLeitsonderfertigkeiten( leitsf );

        return kultur;
    }

    private MoeglicheProfessionen erzeugeMoeglicheProfessionen(
            final IXMLElement wurzelElement ) {

        final IXMLElement professionenElt =
            wurzelElement.getFirstChildNamed( TAG_PROFESSIONEN );

        final IXMLElement alleElt = professionenElt.getChildren().get( 0 );

        final Vector< IXMLElement > professionElemente = alleElt.getChildren();

        final List< Profession > professionen =
            new ArrayList< Profession >( professionElemente.size() );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final IXMLElement professionElt : professionElemente ) {

            final String professionId = professionElt.getContent();

            professionen.add( speicher.getProfession( professionId ) );
        }

        if ( alleElt.getName().equals( TAG_ALLE_VON ) ) {

            return new MoeglicheProfessionenAlleVon( professionen );

        } else {

            return new MoeglicheProfessionenAlleAusser( professionen );

        }
    }

    /* *
     * Speichert alles Kulturen aus der jar-Datei in ein Verzeichnis.
     *
     * @param verzeichnis
     *            in das die Kulturen gespeichert werden sollen.
     * /
    public final void exportieren( final File verzeichnis ) {

        verzeichnis.mkdir();

        for ( final String datei : KULTUR_DATEIEN ) {

            final String pfad = "kulturen/" + datei;

            InputStream in = null;

            OutputStream out = null;

            try {

                in = new BufferedInputStream(
                        RassenLader.class.getResourceAsStream( pfad ) );

                out = new BufferedOutputStream(
                            new FileOutputStream(
                                    new File( verzeichnis, datei ) ) );

                final byte[] buffer = new byte[1024];

                int length;

                while ( ( length = in.read( buffer ) ) > 0 ) {

                    out.write( buffer, 0, length );
                }

            } catch( final IOException e ) {

                DialogFehlermeldung.zeigeDialog(
                        "Die Datei " + datei
                        + " konnte nicht exportiert werden.", e );

            } finally {

                if ( null != in ) {
                    try {
                        in.close();
                    } catch (final IOException e) {
                        e.printStackTrace();
                    }
                }

                if ( null != out ) {
                    try {
                        out.close();
                    } catch (final IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
*/
    @Override
    protected Kultur getBaustein( final String id ) {
        return Datenspeicher.getInstanz().getKultur( id );
    }

    /**
     * Laedt Kulturen aus der Jar-Datei.
     *
     * @return die aus der Jar-Dateien geladenen Kulturen.
     */
    public final List< Kultur > ladeAusJar() {

        final List< Kultur > kulturen;
        kulturen = new ArrayList< Kultur >( KULTUR_DATEIEN.length );

        for ( final String datei : KULTUR_DATEIEN ) {

            final String pfad = "kulturen/" + datei;

            final InputStream in;
            in = KulturenLader.class.getResourceAsStream( pfad );

            if ( null == in ) {

                DialogFehlermeldung.zeigeDialog( String.format(
                        "Die Kultur %1$s konnte nicht gefunden werden.",
                        pfad ) );

                continue;
            }

            try {

                final Kultur kultur = ladeKultur( datei, in );

                kulturen.add( kultur );

            } catch (final Exception e) {

                DialogFehlermeldung.zeigeDialog( String.format(
                        "Die Datei %1$s konnte nicht aus der Jar-Datei " +
                        "geladen werden.", datei ),
                        e );

            }
        }

        return kulturen;
    }

    /**
     * @param dateiname
     *            der Datei aus der die Kultur geladen wird.
     * @param in
     *            der Inputstream der Datei aus der die Kultur geladen wird.
     * @return die geladene Kultur.
     * @throws Exception
     *             wenn es zu einem Fehler beim laden der Kultur kommt.
     */
    public final Kultur ladeKultur(
            final String dateiname,
            final InputStream in ) throws Exception {

        final IXMLReader reader = new StdXMLReader( in );

        parser.setReader( reader );

        final IXMLElement wurzelElement = (IXMLElement) parser.parse();

        return erzeugeKultur( wurzelElement );

    }

    /**
     * Laedt alle Kulturen aus einem Verzeichnis.
     *
     * @param verzeichnis das die Kulturen enthaelt.
     * @return die Liste der geladenen Kulturen.
     */
    public final List< Kultur > laden( final File verzeichnis ) {

        final List< Kultur > kulturen = new ArrayList< Kultur >();

        for ( final File datei : verzeichnis.listFiles() ) {

            final String name = datei.getName();

            if ( name.toLowerCase().endsWith( DATEIERWEITERUNG_KULTUR ) ) {

                try {

                    final InputStream in = new FileInputStream( datei );

                    final Kultur kultur = ladeKultur( name, in );

                    kulturen.add( kultur );

                } catch (final Exception e) {

                    DialogFehlermeldung.zeigeDialog( String.format(
                            "Die Datei %1$s konnte nicht " +
                            "geladen werden.", datei ),
                            e );
                }
            }
        }

        return kulturen;
    }

}
