/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten;

import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_PROFESSION;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_KAEMPFERPROFESSION;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_ZEITAUFWAENDIG;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.StdXMLReader;
import net.n3.nanoxml.XMLParserFactory;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.bonus.BesondererBesitz;
import de.bernhardjung.dsaprogramm.daten.factorys.bausteine.FactoryNameProfessionIn;
import de.bernhardjung.dsaprogramm.daten.factorys.bausteine.FactoryStringsIn;
import de.bernhardjung.dsaprogramm.daten.factorys.bausteine.FactoryNameProfessionIn.Namen;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;

public class ProfessionenLader extends HeldenbausteinLader< Profession > {

    /** Die Professions-Dateie die mit der Jar-Datei ausgeliefert werden. */
    private static final String[] PROFESSIONS_DATEIEN = {
        "Botenreiter.pro",
        "Einbrecher.pro",
        "Entdecker.pro",
        "Gaukler.pro",
        "Jaeger.pro",
        "Krieger.pro",
        "Kundschafter.pro",
        "Legendensaenger.pro",
        "Magier.pro",
        "Pirat.pro",
        "Ritter.pro",
        "Soeldner.pro",
        "Streuner.pro",
        "Wildnislaeufer.pro",
        "Wundarzt.pro"
    };

    /**
     * Erzeugt die Moeglichkeiten fuer den Vorteil Besonderer Besitz aus dem
     * XML-Dokument.
     */
    private final FactoryStringsIn besondererBesitz;

    /** Der Parser zum Einlesen der XML-Dateien. */
    private final IXMLParser parser;

    /** Erzeugt den Namen der Profession aus dem XML-Dokument. */
    private final FactoryNameProfessionIn name;

    /**
     * @throws Exception
     *             wenn es beim Initialisieren des XML-Parsers zu einem Fehler
     *             kommt.
     */
    public ProfessionenLader() throws Exception {

        besondererBesitz = new FactoryStringsIn();

        parser = XMLParserFactory.createDefaultXMLParser();

        name = new FactoryNameProfessionIn();
    }

    /**
     * @param wurzelElement der XML-Datei, die die Profession repraesentiert.
     * @return die, aus der XML-Datei erzeugte Profession.
     */
    private final Profession erzeugeProfession(
            final IXMLElement wurzelElement ) {

        final Profession profession = erzeugeBaustein( wurzelElement );

        // Der Name der Profession
        // ***********************
        final Namen namen = name.erzeugen( wurzelElement, profession.getId() );
        profession.setName( namen.maennlich );
        profession.setNameWeiblich( namen.weiblich );

        // zeitaufwaendig?
        // ***************
        if ( wurzelElement.hasAttribute( ATTR_ZEITAUFWAENDIG ) ) {

            profession.setZeitaufwaendig( true );

        } else {

            profession.setZeitaufwaendig( false );
        }

        // Kaempfer?
        // ***************
        if ( wurzelElement.hasAttribute( ATTR_KAEMPFERPROFESSION)){

            profession.setKaempferprofession( true );

        } else {

            profession.setKaempferprofession( false );
        }

        // Besonderer Besitz
        //******************
        final List< BesondererBesitz > besitz =
            besondererBesitz.erzeugen( wurzelElement);

        profession.setBesondererBesitz( besitz );

        return profession;
    }

    /* *
     * Exportiert alles Professionen aus der jar-Datei in ein Verzeichnis.
     *
     * @param verzeichnis
     *            in das die Professionen exportiert werden sollen.
     * /
    public final void exportieren( final File verzeichnis ) {

        verzeichnis.mkdir();

        for ( final String datei : PROFESSIONS_DATEIEN ) {

            final String pfad = "professionen/" + datei;

            InputStream in = null;

            OutputStream out = null;

            try {

                in = new BufferedInputStream(
                        RassenLader.class.getResourceAsStream( pfad ) );

                out = new BufferedOutputStream(
                            new FileOutputStream(
                                    new File( verzeichnis, datei ) ) );

                final byte[] buffer = new byte[1024];

                int length;

                while ( ( length = in.read( buffer ) ) > 0 ) {

                    out.write( buffer, 0, length );
                }

            } catch( final IOException e ) {

                DialogFehlermeldung.zeigeDialog(
                        "Die Datei " + datei
                        + " konnte nicht exportiert werden.", e );

            } finally {

                if ( null != in ) {
                    try {
                        in.close();
                    } catch (final IOException e) {
                        e.printStackTrace();
                    }
                }

                if ( null != out ) {
                    try {
                        out.close();
                    } catch (final IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
*/

    @Override
    protected final Profession getBaustein( final String id ) {
        return Datenspeicher.getInstanz().getProfession( id );
    }

    /**
     * Laedt Professionen aus der Jar-Datei.
     *
     * @return die aus der Jar-Datei geladenen Professionen.
     */
    public final List< Profession > ladeAusJar() {

        final List< Profession > professionen;
        professionen = new ArrayList<Profession>( PROFESSIONS_DATEIEN.length );

        for ( final String datei : PROFESSIONS_DATEIEN ) {

            final String pfad = "professionen/" + datei;

            final InputStream in;
            in = ProfessionenLader.class.getResourceAsStream( pfad );

            if ( null == in ) {

                DialogFehlermeldung.zeigeDialog( String.format(
                        "Die Profession %1$s konnte nicht gefunden werden.",
                        pfad ) );

                continue;
            }

            try {

                final Profession profession = ladeProfession( datei, in );

                professionen.add( profession );

            } catch (final Exception e) {

                DialogFehlermeldung.zeigeDialog( String.format(
                        "Die Datei %1$s konnte nicht aus der Jar-Datei " +
                        "geladen werden.", datei ),
                        e );

            }
        }

        return professionen;
    }

    /**
     * Laedt alle Professionen aus einem Verzeichnis.
     *
     * @param verzeichnis das die Professionen enthaelt.
     * @return die Liste der geladenen Rassen.
     */
    public final List< Profession > laden( final File verzeichnis ) {

        final List< Profession > professionen = new ArrayList< Profession >();

        for ( final File datei : verzeichnis.listFiles() ) {

            final String name = datei.getName();

            if ( name.toLowerCase()
                    .endsWith( DATEIERWEITERUNG_PROFESSION ) ) {

                try {

                    final InputStream in = new FileInputStream( datei );

                    final Profession profession = ladeProfession( name, in );

                    professionen.add( profession );

                } catch (final Exception e) {

                    DialogFehlermeldung.zeigeDialog( String.format(
                            "Die Datei %1$s konnte nicht " +
                            "geladen werden.", datei ),
                            e );
                }
            }
        }

        return professionen;
    }

    /**
     * Laedt eine Profession aus einem InputStream.
     *
     * @param dateiname der Datei, aus der die Profession geladen werden soll.
     * @param in Der InputStream der die XML-Datei repraesentiert.
     * @return die Profession aus dem InputStream.
     * @throws Exception wenn beim Laden der Profession etwas schief geht.
     */
    public final Profession ladeProfession(
            final String dateiname,
            final InputStream in )
    throws Exception {

        final IXMLReader reader = new StdXMLReader( in );

        parser.setReader( reader );

        final IXMLElement wurzelElement = (IXMLElement) parser.parse();

        return erzeugeProfession( wurzelElement );
    }
/*
    /**
     * Laedt eine Profession aus einer Datei.
     *
     * @param datei XML-Datei die die Profession enthaelt.
     * @return die Profession aus der Datei.
     * /
    private final Profession ladeProfession( final File datei ) {

        try {

            final Document dokument = builder.build( datei );

            return erzeugeProfession( dokument.getRootElement() );

        } catch ( final Exception e ) {

            DialogFehlermeldung.zeigeDialog( String.format(
                    "Die Profession in %1$s konnte nicht geladen werden.",
                    datei.getAbsolutePath() ),
                    e );
        }

        return null;
    }

    /**
     * Laedt alle Professionen aus einem Verzeichnis.
     *
     * @param verzeichnis das die Professionen enthaelt.
     * @return die Liste der geladenen Professionen.
     * /
    final List< Profession > laden( final File verzeichnis ) {

        final List< Profession > professionen = new ArrayList< Profession >();

        for ( final File datei : verzeichnis.listFiles() ) {

            final String name = datei.getName().toLowerCase();

            if ( name.endsWith( ".xml" ) ) {

                final Profession profession = ladeProfession( datei );

                if ( null != profession ) {

                    professionen.add( profession );
                }
            }
        }

        return professionen;
    }
*/
}
