/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BONUS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_LEITTALENT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_TYP;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_ALTE_SPRACHE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_BONUS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_FEST;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_FREI;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_FREMDSPRACHEN;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_MUTTERSPRACHE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_NICHT_MUTTERSPRACHE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_SCHRIFTEN_MUTTERSPRACHE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_TALENT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_TALENTLISTE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_TALENT_BONI;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFestAlteSprache;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFrei;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusTyp;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListe;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListeFest;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class FactoryTalentBoniIn {

    /**
     * Erzeugt aus dem XML-Dokument die Talent-Boni eines Heldenbausteins.
     *
     * @param wurzelElement der XML-Datei.
     * @return die aus dem IXMLElement ermittelten Talent-Boni.
     */
    public final List< Talentbonus > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< Talentbonus > boni = new ArrayList< Talentbonus >();

        final IXMLElement talenteElement =
            wurzelElement.getFirstChildNamed( TAG_TALENT_BONI );

        if ( null != talenteElement ) {

            final Vector< IXMLElement > festeTalentBoni;
            festeTalentBoni = talenteElement.getChildrenNamed( TAG_FEST );
            for ( final IXMLElement bonus : festeTalentBoni ) {

                boni.add( erzeugeBonusFest( bonus ) );
            }

            final Vector< IXMLElement > freieTalentBoni;
            freieTalentBoni = talenteElement.getChildrenNamed( TAG_FREI );
            for ( final IXMLElement bonus : freieTalentBoni ) {

                boni.add( erzeugeBonusFrei( bonus ) );
            }

            final Vector< IXMLElement > alteSprache;
            alteSprache = talenteElement.getChildrenNamed( TAG_ALTE_SPRACHE );
            for ( final IXMLElement bonus : alteSprache ) {

                boni.add( erzeugeBonusAlteSprache( bonus ) );
            }
        }

        return boni;
    }

    /**
     * Erzeugt aus dem entsprechenden XML-IXMLElement einen
     * {@link TalentbonusFestAlteSprache}.
     *
     * @param bonusElement
     *            das den Bonus repraesentiert.
     * @return den aus dem {@code bonusElement} erzeugten Bonus.
     */
    private final TalentbonusFestAlteSprache erzeugeBonusAlteSprache(
            final IXMLElement bonusElement ) {

        final List< Integer > boni = new ArrayList< Integer >( 2 );

        for ( final IXMLElement element : bonusElement.getChildren() ){

            boni.add( Integer.valueOf( element.getContent() ) );
        }

        return new TalentbonusFestAlteSprache( boni );
    }

    /**
     * Erzeugt einen Talent-Bonus vom Typ {@link TalentbonusFest}.
     *
     * @param bonusElement das XML-IXMLElement, welches den Bonus repraesentiert.
     * @return der aus dem IXMLElement erzeugte Talent-Bonus.
     */
    private final TalentbonusFest erzeugeBonusFest(
            final IXMLElement bonusElement ) {

        final IXMLElement talentListeElt;
        talentListeElt = bonusElement.getFirstChildNamed( TAG_TALENTLISTE );

        final TalentListe talentListe = erzeugeTalentListe( talentListeElt );

        final List< Integer > boni = new ArrayList< Integer >();

        for ( final IXMLElement bonus : bonusElement.getChildrenNamed(TAG_BONUS) ){

            boni.add( Integer.valueOf( bonus.getContent() ) );
        }

        final TalentbonusFest bonus = new TalentbonusFest( talentListe, boni );

        if ( null != bonusElement.getAttribute( ATTR_TYP ) ) {

            bonus.setTyp( TalentbonusTyp.ENTDECKER );

        }

        if ( null != bonusElement.getAttribute( ATTR_LEITTALENT ) ) {

            bonus.setLeittalent( true );

        }

        return bonus;
    }

    /**
     * Erzeugt aus einem IXMLElement des XML-Dokuments einen Talent-Bonus vom Typ
     * {@link TalentbonusFrei}.
     *
     * @param bonusElement das den Talent-Bonus repraesentiert.
     * @return der aus dem IXMLElement ermittelte Bonus.
     */
    private final TalentbonusFrei erzeugeBonusFrei(
            final IXMLElement bonusElement ) {

        final List< Talent > talente;
        talente = new ArrayList< Talent >( bonusElement.getChildrenCount() );

        for ( final IXMLElement talent : bonusElement.getChildren() ) {

            talente.add( Talent.getTalent( talent.getContent() ) );

        }

        final String bonusString = bonusElement.getAttribute( ATTR_BONUS );

        final int bonus = Integer.valueOf( bonusString );

        return new TalentbonusFrei( bonus, talente );
    }

    /**
     * Erzeugt die Talent-Liste fuer einen {@link TalentbonusFest}.
     *
     * @param talentListeElement das XML-IXMLElement welches die Liste enthaelt.
     * @return die ermittelte Talent-Liste.
     */
    private final TalentListe erzeugeTalentListe(
            final IXMLElement talentListeElement ) {

        final Vector< IXMLElement > liste = talentListeElement.getChildren();

        final TalentListe talentListe;

        final String kindName = liste.get( 0 ).getName();

        if ( kindName.equals( TAG_TALENT ) ) {

            final List< Talent > talente;
            talente = new ArrayList< Talent >( liste.size() );

            for ( final IXMLElement talentElement : liste ) {

                final String id = talentElement.getContent();

                talente.add( Talent.getTalent( id ) );
            }

            talentListe = new TalentListeFest( talente );

        }/* else if ( kindName.equals( TAG_ALTE_FORM ) ) {

            talentListe = TalentListe.ALTE_FORM;

        }*/ else if ( kindName.equals( TAG_FREMDSPRACHEN ) ) {

            talentListe = TalentListe.FREMDSPRACHEN;

        } else if ( kindName.equals( TAG_MUTTERSPRACHE ) ) {

            talentListe = TalentListe.MUTTERSPRACHE;

        } else if ( kindName.equals( TAG_NICHT_MUTTERSPRACHE ) ) {

            talentListe = TalentListe.NICHT_MUTTERSPRACHE;

        } else if ( kindName.equals( TAG_SCHRIFTEN_MUTTERSPRACHE ) ) {

            talentListe = TalentListe.SCHRIFTEN_MUTTERSPRACHE;

        } else {

            throw new IllegalArgumentException(
                    "Es existiert keine Talent-Liste vom Typ " + kindName );

        }

        return talentListe;
    }
}
