/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_HOECHSTSTUFE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_MINDESTSTUFE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_GESCHLECHT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_MINDESTSTUFE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_SO;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_VORAUSSETZUNGEN;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_ZWERGIN;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungGeschlecht;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungMindeststufe;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungSo;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungZwergin;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

public class FactoryVoraussetzungenIn {

    /** Fehlermeldung wenn ein unbekanntes Element in Dokument auftaucht. */
    private static final String VORAUSSETZUNG_UNBEKANNT =
        "Die Voraussetzung vom Typ %1$s ist unbekannt.";

    /**
     * Erzeugt aus einem Element des XML-Dokuments die zugehoerige
     * Voraussetzung.
     *
     * @param element das in eine Voraussetzung umgewandelt werden soll.
     * @return die aus dem element ermittelte Voraussetzung.
     */
    private final Voraussetzung erzeugeVoraussetzung( final IXMLElement element ) {

        final String name = element.getName();

        if ( name.equals( TAG_MINDESTSTUFE ) ) {

            // Eine Voraussetzung wie "KL oder IN 12"

            final String mindeststufeString;
            mindeststufeString = element.getAttribute( ATTR_MINDESTSTUFE);

            // Die Mindeststufe die erreicht werden muss
            final int mindeststufe = Integer.valueOf( mindeststufeString );

            final Vector< IXMLElement > elemente = element.getChildren();

            final int anzahlEigenschaften = elemente.size();

            // Die Eigenschaften in denen die Mindeststufe erreicht werden
            // muss.
            final Eigenschaft[] eigenschaften;
            eigenschaften = new Eigenschaft[ anzahlEigenschaften ];

            for ( int i = 0; i < anzahlEigenschaften ; ++i ) {

                final String eigenschaftId;
                eigenschaftId = elemente.get( i ).getContent();

                eigenschaften[ i ] =
                    Eigenschaft.getEigenschaft( eigenschaftId );
            }

            return new VoraussetzungMindeststufe(mindeststufe, eigenschaften);

        } else if ( name.equals( TAG_SO ) ) {

            // Der SO muss zwischen zwei Grenzwerten liegen.

            final String mindeststufeString;
            mindeststufeString = element.getAttribute( ATTR_MINDESTSTUFE);

            final int mindeststufe = Integer.valueOf( mindeststufeString );

            final String hoechststufeString;
            hoechststufeString = element.getAttribute( ATTR_HOECHSTSTUFE);

            final int hoechststufe = Integer.valueOf( hoechststufeString );

            return new VoraussetzungSo( mindeststufe, hoechststufe );

        } else if ( name.equals( TAG_GESCHLECHT ) ) {

            // Der Charakter muss ein bestimmtes Geschlecht haben.

            final Geschlecht geschlecht;
            geschlecht = Geschlecht.valueOf( element.getContent() );

            return new VoraussetzungGeschlecht( geschlecht );

        } else if ( name.equals( TAG_ZWERGIN ) ) {

            return new VoraussetzungZwergin();

        } else {

            throw new IllegalArgumentException( String.format(
                    VORAUSSETZUNG_UNBEKANNT, name ) );
        }
    }

    /**
     * @param wurzelElement der XML-Datei.
     * @return die Liste der aus der XML-Datei ermittelten Voraussetzungen.
     */
    public final List< Voraussetzung > erzeugen(
            final IXMLElement wurzelElement ) {

        final IXMLElement voraussetzungenElement =
            wurzelElement.getFirstChildNamed( TAG_VORAUSSETZUNGEN );

        final List< Voraussetzung > voraussetzungen;
        voraussetzungen = new ArrayList<Voraussetzung>();

        if ( null != voraussetzungenElement ) {

            for( final IXMLElement element :
                voraussetzungenElement.getChildren() ) {

                voraussetzungen.add( erzeugeVoraussetzung( element ) );

            }
        }

        return voraussetzungen;
    }

}
