/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basis.varianten.VarianteListener;
import de.bernhardjung.dsaprogramm.util.Beobachtbar;
import de.bernhardjung.dsaprogramm.util.FehlendesArgumentException;
import de.bernhardjung.dsaprogramm.util.Util;

public class SonderfertigkeitWert extends Beobachtbar implements Cloneable {

    /**
     * Sortiert die Werte alphabetisch nach Namen.
     */
    public static final Comparator< SonderfertigkeitWert > NAME_COMPARATOR =
        new Comparator< SonderfertigkeitWert >() {
            public int compare( SonderfertigkeitWert o1, SonderfertigkeitWert o2 ) {
                
                return Util.compare( o1.getName(), o2.getName() );
            }
    };
    
    /**
     * Erzeugt eine Deep Copy einer Liste von SonderfertigkeitWert Elementen.
     * 
     * @param original Die orignal Liste, die kopiert werden soll.
     * @return Eine Deep Copy.
     */
    public static ArrayList< SonderfertigkeitWert > deepCopy(
            final Collection< SonderfertigkeitWert > original ) {
        
        final ArrayList< SonderfertigkeitWert > kopien = 
            new ArrayList< SonderfertigkeitWert >( original.size() );
        
        for ( final SonderfertigkeitWert wert : original ) {
            
            kopien.add( wert.clone() );
        }
        
        return kopien;
    }
    
   /**
    * Erzeugt eine Liste von Werten zu den uebergebenen Varianten.
    * 
    * @param sonderfertigkeiten zu denen Werte geliefert werden sollen.
    * @return Ein Wert pro Sonderfertigkeit.
    */
    public static final List< SonderfertigkeitWert > erzeugeWerte( 
            final List< Sonderfertigkeit > sonderfertigkeiten ) {
        
        final List< SonderfertigkeitWert > werte = 
            new ArrayList< SonderfertigkeitWert >();
        
        for ( final Sonderfertigkeit sonderfertigkeit : sonderfertigkeiten ) {
            
            if ( sonderfertigkeit.hatVarianten() ) {
                
                final List< Variante > varianten = 
                    sonderfertigkeit.getVarianten( null );
                
                final Variante variante;
                
                if ( 0 < varianten.size() ) {
                    
                    variante = varianten.get( 0 );
                
                } else {
                    
                    variante = new Variante();
                }
                
                werte.add( new SonderfertigkeitWert( sonderfertigkeit, variante ) );
            
            } else {
                
                werte.add( new SonderfertigkeitWert( sonderfertigkeit ) );
            }
        }
        
        return werte;
    }
    
    /** Die AP die der Held bezahlt hat um die Sonderfertigkeit zu erlernen. */
    private int lernkosten;

    /** Die Sonderfertigkeit des Wertes. */
    private final Sonderfertigkeit sonderfertigkeit;

    /**
     * Die Variante der Sonderfertigkeit, falls die Sonderfertigkeit
     * Varianten besitzt.
     */
    private Variante variante;

    /**
     * Gibt an ob die Variante des Wertes geaendert werden darf. Bei
     * Sonderfertigkeiten die ein Held durch Kultur oder Profession erhalten
     * hat ist das normalerweise nicht der Fall.
     */
    private boolean varianteAenderbar;

    /**
     * Die Objekte die ueber Aenderungen an der Veriante des Werts informiert
     * werden wollen.
     */
    private List< VarianteListener > varianteListeners;
    
    /**
     * Erzeugt einen neuen Wert.
     *
     * @param dieSonderfertigkeit auf die sich der Wert bezieht.
     * @throws IllegalArgumentException wenn die Sonderfertigkeit Varianten
     *      besitzt. In diesem Fall muss der Konstruktor {@link
     *      #SonderfertigkeitWert(Sonderfertigkeit, Object, boolean)} verwendet
     *      werden.
     */
    public SonderfertigkeitWert( final Sonderfertigkeit dieSonderfertigkeit ) {

        if ( dieSonderfertigkeit.hatVarianten() ) {

            throw new FehlendesArgumentException( String.format(
                    "Die Sonderfertigkeit %1$s besitzt Varianten. Bei der " +
                    "Erzeugung eines Wertes muss ein Variante angegeben " +
                    "werden. Bitte benutzen Sie den Konstruktor " +
                    "SonderfertigkeitWert( Sonderfertigkeit, Object ).",
                    dieSonderfertigkeit.getName() ) );
        }

        lernkosten = 0;

        sonderfertigkeit = dieSonderfertigkeit;

        variante = new Variante();

        varianteAenderbar = false;
        
        varianteListeners = new ArrayList< VarianteListener >();
    }

    /**
     * Erzeugt einen neuen Wert dessen Variante nachtraeglich geaendert
     * werden kann.
     *
     * @param dieSonderfertigkeit auf die sich der Wert bezieht.
     * @param variante der Sonderfertigkeit. Das Argument darf nicht
     *      {@code null} sein, siehe {@link #setVariante(Object)}.
     */
    public SonderfertigkeitWert(
            final Sonderfertigkeit dieSonderfertigkeit,
            final Object variante ) {

        this( dieSonderfertigkeit, variante, true );
    }

    /**
     * Erzeugt einen neuen Wert.
     *
     * @param dieSonderfertigkeit auf die sich der Wert bezieht.
     * @param variante der Sonderfertigkeit. Das Argument darf nicht
     *      {@code null} sein, siehe {@link #setVariante(Object)}.
     * @param istVarianteAenderbar gibt an ob die Variante nachtraeglich
     *      geaendert werden darf.
     */
    public SonderfertigkeitWert(
            final Sonderfertigkeit dieSonderfertigkeit,
            final Object variante,
            final boolean istVarianteAenderbar ) {

        if ( ! dieSonderfertigkeit.hatVarianten() ) {

            throw new IllegalArgumentException( String.format(
                    "Die Sonderfertigkeit %1$s besitzt keine Varianten. Bei " +
                    "der Erzeugung des Wertes wurde aber eine Variante " +
                    "angegeben. Bitte benutzen Sie den Konstruktor " +
                    "SonderfertigkeitWert( Sonderfertigkeit ).",
                    dieSonderfertigkeit.getName() ) );
        }

        lernkosten = 0;

        // Name wird von setVariante(String) gesetzt, siehe unten.
        // name = dieSonderfertigkeit.getName();

        sonderfertigkeit = dieSonderfertigkeit;
        
        varianteListeners = new ArrayList< VarianteListener >();
        
        // erst auf true setzen, damit setVariante aufgerufen werden kann.
        varianteAenderbar = true;
        setVariante( variante );

        varianteAenderbar = istVarianteAenderbar;
    }

    /**
     * Meldet einen Listener vom Wert ab. Macht nichts, wenn der Listener nicht
     * angemeldet ist.
     * 
     * @param listener der abgemeldet werden soll.
     */
    public void abmelden( final VarianteListener listener ) {
        
        varianteListeners.remove( listener );
    }

    /**
     * Meldet einen Listener an dem Wert an. Es kann kein Listener mehrfach
     * angemeldet werden. Wird versucht den Listener ein zweites mal anzumelden
     * wird der Aufruf einfach ignoriert.
     * 
     * @param listener der angemeldet werden soll.
     */
    public void anmelden( final VarianteListener listener ) {
        
        if ( ! varianteListeners.contains( listener ) ) {
            
            varianteListeners.add( listener  );
        }
    }

    /**
     * @return eine shallow Kopie des Wertes.
     */
    @Override
    public SonderfertigkeitWert clone() {
        
        SonderfertigkeitWert wert = (SonderfertigkeitWert) super.clone();
        
        wert.varianteListeners = new ArrayList< VarianteListener >();
        
        return wert;
    }

    /**
     * Die Methode wurde ueberschrieben, so dass Werte mit gleichem Namen als
     * gleich gelten.
     *
     * @param obj das andere Objekt mit dem verglichen wird.
     */
    @Override
    public boolean equals( final Object obj ) {

        if ( obj == null 
                || ! obj.getClass().equals( getClass() ) ) {
            
            return false;
        }

        final SonderfertigkeitWert wert = (SonderfertigkeitWert) obj;

        return Util.equals( 
                lernkosten, wert.lernkosten,
                sonderfertigkeit, wert.sonderfertigkeit,
                variante, wert.variante,
                varianteAenderbar, wert.varianteAenderbar );
    }
        
    /**
     * @return die AP-Kosten fuer die Sonderfertigkeit.
     */
    public double getApKosten() {
        return sonderfertigkeit.getApKosten( this );
    }

    /**
     * @param held der die Sonderfertigkeit bezahlen muss.
     * @return die AP die der Held fuer die Sonderfertigkeit bezahlen muss.
     */
    public final int getApKosten( final Held held ) {
        return sonderfertigkeit.getApKosten( held, this );
    }

    /**
     * @param held
     *            der die Sonderfertigkeit erlernen will.
     * @param lernmethode
     *            die zum Erlernen der Sonderfertigkeit verwendet werden soll.
     * @return die AP-Kosten die der Held zum lernen der Sonderfertigkeit zahlen
     *         muss.
     */
    public int getApKosten( final Held held, final Lernmethode lernmethode ) {
        return sonderfertigkeit.getApKosten( held, this, lernmethode );
    }

    /**
     * @see #getGpKosten(Held)
     * @return die Standard GP-Kosten fuer den Wert. Kann der Wert ohne den Held
     *         nicht genau bestimmt werden wird irgendein Wert geliefert.
     */
    public final int getGpKosten() {
        return sonderfertigkeit.getGpKosten( this );
    }

    /**
     * @param held der die Sonderfertigkeit bezahlen muss.
     * @return die GP die der Held fuer die Sonderfertigkeit bezahlen muss.
     */
    public final int getGpKosten( final Held held ) {
        return sonderfertigkeit.getGpKosten( held, this );
    }

    /**
     * @return die AP die ein Held aufgebracht hat um die Sonderfertigkeit zu
     *         erlernen.
     */
    public final int getLernkosten() {
        return lernkosten;
    }

    /**
     * @return die Bezeichnung des Wertes, z.B.
     *      'Talentspezialisierung Klettern (Freiklettern)'
     */
    public final String getName() {
        
        final StringBuilder name = new StringBuilder();
        
        name.append( getSonderfertigkeit().getName() );
        
        if ( getSonderfertigkeit().hatVarianten() ) {
            
            name.append( " (" );
            
            if ( null != getVariante() ) {
                
                name.append( getVariante().toString() );
            
            } else {
                
                name.append( "???" );
            }
            
            name.append( ")" );
        }
        
        return name.toString();
    }

    /**
     * @return die Sonderfertigkeit des Wertes.
     */
    public final Sonderfertigkeit getSonderfertigkeit() {
        return sonderfertigkeit;
    }

    /**
     * @return die gewaehlte Variante der Sonderfertigkeit oder {@code null}
     *      falls die Sonderfertigkeit keine Varianten besitzt.
     */
    public final Variante getVariante() {
        return variante;
    }

    @Override
    public int hashCode() {
        return getName().hashCode();
    }

    /**
     * @param wert Der Wert mit dem dieser Wert verglichen wird.
     * @return {@code true}, wenn beide Werte von der selben Sonderfertigkeit
     *         stammen und die selbe Variante haben.
     */
    public boolean hatGleicheSfUndVariante( final SonderfertigkeitWert wert ) {
        
        return Util.equals( 
                sonderfertigkeit, wert.sonderfertigkeit,
                variante, wert.variante );
    }
    
    /**
     * @return {@code true} wenn die Variante des Wertes geaendert werden darf,
     *      sonst {@code false}.
     */
    public final boolean istVarianteAenderbar() {
        return varianteAenderbar;
    }

    /**
     * Informiert die Listener ueber Aenderungen an der Variante es Wertes.
     * 
     * @param alt Die alte Variante des Wertes.
     * @param neu Die neue Variante des Wertes.
     */
    private void meldeVarianteGeaendert(
            final Variante alt,
            final Variante neu ) {
        
        for ( final VarianteListener listener : varianteListeners ) {
            
            listener.varianteGeandert( this, alt, neu );
        }
    }

    /**
     * Prueft ob ein Held alle Voraussetzungen erfuellt um die Sonderfertigkeit
     * einsetzen zu koennen.
     *
     * @param held der die Sonderfertigkeit einsetzen will.
     * @return eine Beschreibung der bestehenden Probleme wenn der Held nicht
     *      alle Voraussetzungen erfuellt.
     */
    public final KonfliktBeschreibung pruefen( final Held held ) {
        return sonderfertigkeit.pruefen( held, this );
    }

    /**
     * @param ap
     *            die ein Held aufgebracht hat, um die Sonderfertigkeit zu
     *            erlernen.
     */
    public final void setLernkosten( final int ap ) {
        lernkosten = ap;
    }

    /**
     * Bestimmt die Variante der Sonderfertigkeit.
     *
     * @see #istVarianteAenderbar()
     * @param dieVariante der Sonderfertigkeit. Das Objekt muss mittels
     *      {@link Object#toString()} sinnvoll in eine String umgewandelt
     *      werden koennen.
     * @throws IllegalArgumentException wenn {@code null} uebergeben wird.
     * @throws UnsupportedOperationException wenn die Variante nicht aenderbar
     *      ist oder die Sonderfertigkeit keine Varianten besitzt.
     */
    public final void setVariante( final Object dieVariante ) {

        if ( null == dieVariante
                || ! sonderfertigkeit.hatVarianten()
                || ! varianteAenderbar ) {

            return;
        }

        final Variante neu = getSonderfertigkeit().getVariante( dieVariante ); 
        
        if ( ! neu.equals( variante ) ) {
         
            final Variante alt = variante;
            
            variante = neu;
            
            meldeVarianteGeaendert( alt, neu );
            
            meldeBeobachtern( sonderfertigkeit );
        }
    }

    /**
     * @param aenderbar
     *            {@code true}, wenn die Variante aenderbar sein soll, sonst
     *            {@code false}.
     */
    public void setVarianteAenderbar( final boolean aenderbar ) {
        varianteAenderbar = aenderbar;
    }

    /**
     * @return {@link #getName()}
     */
    @Override
    public String toString() {
        return getName();
    }
}
