/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public interface Kosten {

    /**
     * Fuer Sonderfertigkeiten mit den fixen Kosten von 0 AP,
     * zB Grosse Meditation.
     */
    public static final Kosten AP_0 = new KostenFest( 0 );

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 50 AP */
    public static final Kosten AP_50 = new KostenFest( 50 );

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 75 AP */
    public static final Kosten AP_75 = new KostenFest( 75 );

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 100 AP */
    public static final Kosten AP_100 = new KostenFest(100);

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 150 AP. */
    public static final Kosten AP_150 = new KostenFest(150);

    /**
     * Fuer Sonderfertigkeiten mit den fixen Kosten von 100 AP, die aber fuer
     * Helden mit dem Vorteil Akademische Ausbildung Krieger zu 3/4 der Kosten
     * zu erwerben sind.
     */
    public static final Kosten AP_150_KAMPF = new KostenKampf(150);

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 200 AP. */
    public static final Kosten AP_200 = new KostenFest(200);

    /** Fuer Sonderfertigkeiten die durch Schlangenmensch verbilligt werden. */
    public static final Kosten AP_200_SCHLANGEMENSCH =
        new KostenSchlangenmensch( 200 );

    /**
     * Fuer Sonderfertigkeiten mit den fixen Kosten von 200 AP, die aber fuer
     * Helden mit dem Vorteil Akademische Ausbildung Krieger zu 3/4 der Kosten
     * zu erwerben sind.
     */
    public static final Kosten AP_200_KAMPF = new KostenKampf(200);

    /**
     * Fuer die Sonderfertigkeit Standfest, die man automatisch mit
     * (Herausragende) Balance erhaelt.
     */
    public static final Kosten AP_200_MIT_BALANCE =
        new KostenAutomatischMitVorNachteil(
                200,
                VorNachteil.BALANCE,
                VorNachteil.HERAUSRAGENDE_BALANCE );

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 250 AP. */
    public static final Kosten AP_250 = new KostenFest( 250 );

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 300 AP. */
    public static final Kosten AP_300 = new KostenFest(300);

    /**
     * Fuer Sonderfertigkeiten mit den fixen Kosten von 300 AP, die aber fuer
     * Helden mit dem Vorteil Akademische Ausbildung Krieger zu 3/4 der Kosten
     * zu erwerben sind.
     */
    public static final Kosten AP_300_KAMPF = new KostenFest(300);

    /** Fuer Sonderfertigkeiten die durch Schlangenmensch verbilligt werden. */
    public static final Kosten AP_300_SCHLANGEMENSCH =
        new KostenSchlangenmensch( 300 );

    /** Fuer Sonderfertigkeiten mit den fixen Kosten von 2000 AP. */
    public static final Kosten AP_2000 = new KostenFest( 2000 );

    /** Berechnet die Kosten fuer Gelaendekunde. */
    public static final Kosten GELAENDEKUNDE = new KostenGelaendekunde();

    /** Berechnet die Kosten fuer Kulturkunde. */
    public static final Kosten KULTURKUNDE = new KostenKulturkunde();

    /** Berechnet die Kosten fuer Nandusgefaelliges Wissen. */
    public static final Kosten NANDUSGEFAELLIGES_WISSEN =
        new KostenNandusgefaelligesWissen();

    /** Berechnet die Kosten fuer die Sonderfertigkeit Ortskenntnis. */
    public static final Kosten ORTSKENNTNIS = new KostenOrtskenntnis();

    /**
     * Das Objekt das sich um die Bestimmung der Kosten einer
     * Talentspezialisierung kuemmert.
     */
    public static final Kosten TALENTSPEZIALISIERUNG =
        new KostenTalentspezialisierung();

    /**
     * Das Objekt das sich um die Bestimmung der Kosten einer
     * Waffenspezialisierung kuemmert.
     */
    public static final Kosten WAFFENSPEZIALISIERUNG =
        new KostenWaffenspezialisierung();

    /**
     * @return die Regelelemente von denen die Kostenberechnung abhaengig ist.
     */
    public Collection< Object > getAbhaengigkeiten();

    /**
     * @param sonderfertigkeit
     *            fuer die die AP-Kosten bestimmt werden sollen.
     * @return die AP-Kosten, die fuer die Sonderfertigkeit zu
     *         zahlen sind.
     */
    public int getApKosten(Sonderfertigkeit sonderfertigkeit);

    /**
     * @param wert dessen AP-Kosten gesucht sind.
     * @see #getApKosten(Held, SonderfertigkeitWert)
     * @return die Standard AP-Kosten fuer die Sonderfertigkeit. Lassen sich
     *         ohne Angabe von Held und Wert keine Kosten festlegen liefert die
     *         Methode irgendeinen Wert.
     */
    public int getApKosten( final SonderfertigkeitWert wert );

    /**
     * @param held
     *            der die AP-Kosten zahlen muss.
     * @param wert
     *            fuer den die AP-Kosten gesucht sind.
     * @return die AP-Kosten die der Held fuer den Wert zahlen muss.
     */
    public int getApKosten(final Held held, final SonderfertigkeitWert wert);

    /**
     * @param held
     *            der die AP-Kosten zahlen muss.
     * @param wert
     *            fuer den die AP-Kosten gesucht sind.
     * @param lernmethode
     *            die zum Erlernen der Sonderfertigkeit angewendet werden soll.
     * @return die AP-Kosten die der Held fuer den Wert zahlen muss.
     */
    public int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert,
            final Lernmethode lernmethode );

    /**
     * @param wert fuer den die Kosten bestimmt werden sollen.
     * @see #getGpKosten(Held, SonderfertigkeitWert)
     * @return die Standard GP-Kosten fuer eine Sonderfertigkeit.
     */
    public int getGpKosten(final SonderfertigkeitWert wert);

    /**
     * Liefert die GP-Kosten die ein Held fuer einen Wert zahlen muss. Ist ein
     * Wert, z.B. durch Rasse, Kultur oder Profession vorgegeben, so sind seine
     * Kosten automatisch 0.
     * <p>
     * Nicht alle Sonderfertigkeiten koennen mit GP bezahlt werden.
     *
     * @see #hatGpKosten()
     * @param held
     *            der die GP-Kosten zahlen muss.
     * @param wert
     *            fuer den die GP-kosten gesucht sind.
     * @return die GP-Kosten die der Held fuer den Wert zahlen muss.
     */
    public int getGpKosten(final Held held, final SonderfertigkeitWert wert);

    /**
     * @return {@code true}, wenn die Sonderfertigkeit mit GP bezahlt werden
     *         kann, sonst {@code false}.
     */
    public boolean hatGpKosten();

}
