/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

class KostenWaffenspezialisierung implements Kosten {

    /**
     * Dieser Faktor mal Steigerungsfaktor des Talents mal Index der
     * Spezialisierung ergibt die AP-Kosten die fuer die
     * Waffenspezialisierung zu zahlen sind.
     */
    private static final int AP_KOSTEN = 20;

    /**
     * Die GP-Kosten die fuer die Talente der unterschiedlichen
     * Steigerungsfaktoren.
     * Eigentlich gibt es fuer Waffenspezialisierungen keine GP-Kosten.
     * Diese Zahlen hier sind von den Talentspezialisierungen uebernommen.
     */
    //                                          A  B  C  D  E
    private static final int[] GP_KOSTEN = { 0, 1, 1, 1, 2, 2 };

    /** @return Akad. Ausb. Krieger, Elfische Weltsicht. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList(
                (Object) VorNachteil.ELFISCHE_WELTSICHT,
                VorNachteil.AKADEMISCHE_AUSBILDUNG_KRIEGER );
    }

    public int getApKosten( final Sonderfertigkeit sonderfertigkeit ) {

        final int faktor = sonderfertigkeit.getTalent().getSteigerungsfaktor();

        return faktor * AP_KOSTEN;
    }

    public final int getApKosten(final SonderfertigkeitWert wert) {

        final Talent talent = wert.getSonderfertigkeit().getTalent();

        final int steigerungsfaktor = talent.getSteigerungsfaktor();

        return steigerungsfaktor * AP_KOSTEN;
    }

    /** {@inheritDoc} */
    public final int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {
            // vorgegebene Werte kosten nichts.
            return 0;
        }

        final int index = held.getIndex( wert );

        float faktor = 1.0F;

        if ( held.hatVerbilligt( wert ) ) {

            faktor = 0.5F;

        } else if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
                && ( ! held.hatLeitsonderfertigkeit( wert ) ) ) {

            faktor = 1.5F;
        }

        if ( held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_KRIEGER ) ) {

            // Mit Akademischer Ausbildung Krieger zahlt man nur 3/4 der
            // Kosten.
            faktor *= 0.75F;

        }

        return Math.round( faktor * index * getApKosten( wert ) );
    }

    /** {@inheritDoc} */
    public int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert,
            final Lernmethode lernmethode ) {

        int apKosten = getApKosten( held, wert );

        if ( Lernmethode.SPEZIELLE_ERFAHRUNG.equals( lernmethode ) ) {

            apKosten = Math.round( apKosten / 2F );

        } else if ( ! Lernmethode.LEHRMEISTER.equals( lernmethode ) ) {

            apKosten = 2 * apKosten;

        }

        return apKosten;
    }

    /** {@inheritDoc} */
    public final int getGpKosten( final SonderfertigkeitWert wert ) {

        final Talent talent = wert.getSonderfertigkeit().getTalent();

        final int steigerungsfaktor = talent.getSteigerungsspalte();

        return GP_KOSTEN[ steigerungsfaktor ];
    }

    /** {@inheritDoc} */
    public final int getGpKosten(
            final Held held,
            final SonderfertigkeitWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {
            // vorgegebene Werte kosten nichts.
            return 0;
        }

        final int index = held.getIndex( wert );

        float faktor = 1.0F;

        if ( held.hatVerbilligt( wert ) ) {

            faktor = 0.5F;

        }  else if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
                && ( ! held.hatLeitsonderfertigkeit( wert ) ) ) {

            faktor = 1.5F;

        }

        if ( held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_KRIEGER ) ) {

            // Mit Akademischer Ausbildung Krieger zahlt man nur 3/4 der
            // Kosten.
            faktor *= 0.75F;

        }

        return Math.round( faktor * index * getGpKosten( wert ) );
    }

    /** @return {@code false} */
    public final boolean hatGpKosten() {
        return false;
    }
}
