/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.util;

import java.awt.Dimension;
import java.util.Locale;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.UIManager;

public class Einstellungen {

    /** Singleton. */
    private static Einstellungen EINSTELLUNGEN;

    public static final Einstellungen getInstanz() {

        if ( null == EINSTELLUNGEN ) {

            EINSTELLUNGEN = new Einstellungen();
        }

        return EINSTELLUNGEN;

    }

    private static final String DATENQUELLE = "Datenquelle";

    /**
     * Gibt bei der Datenquelle an, dass die Daten aus der JAR-Datei geladen
     * werden sollen.
     *
     * @see #getDatenquelle()
     * @see #setDatenquelle(String)
     */
    public static final String DATENQUELLE_INTERN = "Basisregeln (intern)";

    private static final String EINSTELLUNGEN_PFAD = "/de/dsachargen";

    private static final String HAUPTFENSTER_GROESSE = "Hauptfenster-Groesse";
    
    private static final String HAUPTFENSTER_GROESSE_DSA_DESKTOP = "Hauptfenster-Groesse-DSA-Desktop";
    
    private static final String HAUPTFENSTER_GROESSE_STANDARD = "640,480";
    
    private static final String HELDEN_RTF_VERZEICHNIS = "Helden-RTF-Verzeichnis";

    private static final String HELDEN_XML_VERZEICHNIS = "Helden-XML-Verzeichnis";
    
    private static final String LOOK_AND_FEEL = "Look & Feel";
    
    private static final String SPRACHE = "Sprache";
    
    private static final String SPRACHE_GESETZT = "Sprache-Gesetzt";
    
    private static final String USER_HOME = System.getProperty( "user.home" );

    private final Preferences einstellungen;

    private Einstellungen() {

        einstellungen = Preferences.userRoot().node( EINSTELLUNGEN_PFAD );
    }

    /** Speichert eine Einstellung ab. */
    private final void put ( final String schluessel, final String wert ) {
        
        einstellungen.put ( schluessel, wert );
        
        try {
            einstellungen.flush();
        } catch (BackingStoreException e) {
        }
    }
    
    /**
     * Liefert eine Dimension, die mit {@link #setDimension(String, Dimension)}
     * abgespeichert wurde.
     * 
     * @param schluessel unter dem die Dimension abgespeichert worden ist.
     * @param standard Der Default werden fuer die Dimension.
     * @return Eine Dimension.
     */
    private final Dimension getDimension( 
            final String schluessel, 
            final String standard ) {

        String string = einstellungen.get( schluessel, standard );
        
        String[] array = string.split( "," );
        
        return new Dimension(
                Integer.valueOf( array[ 0 ] ), 
                Integer.valueOf( array[ 1 ] ) );
    }
    
    /**
     * Speichert eine Dimension, z.B. die Groesse des Hauptfensters, ab.
     * 
     * @param schluessel unter dem die Dimension gespeichert werden soll.
     * @param d Die Dimension, die es zu speichern gilt.
     */
    private final void setDimension( 
            final String schluessel, 
            final Dimension d ) {
        
        String wert = String.format( "%1$d,%2$d", d.width, d.height );
        
        put ( schluessel, wert );
    }
    
    
    /**
     * Gibt an von wo die Daten fuer das Programm geladen werden sollen.
     *
     * @return der absolute Pfad zu einem Verzeichnis, in dem sich die Daten
     *         fuer das Programm befinden, oder {@link #DATENQUELLE_INTERN},
     *         falls der Benutzer kein solches Verzeichnis gewaehlt hat.
     */
    public final String getDatenquelle() {

        return einstellungen.get( DATENQUELLE, DATENQUELLE_INTERN );
    }

    /**
     * @return {@code true}, wenn ein Verzeichnis als Datenquelle gewaehlt
     *         worden ist, sonst {@code false}.
     */
    public final boolean istDatenquelleVerzeichnis() {
       return ! getDatenquelle().equals( DATENQUELLE_INTERN );
    }

    /**
     * Bestimmt von wo die Daten fuer das Programm geladen werden sollen.
     *
     * @param quelle
     *            der Pfad zu einem Verzeichnis, mit den Daten oder
     *            {@link #DATENQUELLE_INTERN}, falls die Daten aus der
     *            jar-Datei geladen werden sollen.
     */
    public final void setDatenquelle( final String quelle ) {

        if ( null == quelle ) {
            throw new IllegalArgumentException(
                    "null ist als Datenquelle nicht zugelassen.");
        }

        put( DATENQUELLE, quelle );
    }
    
    /**
     * Liefert die groesse fuer das Hauptfenster, wenn das Programm als normale
     * Anwendung, und nicht als Plugin fuer den DSA-Desktop, betrieben wird.
     * @return Die groesse fuer das Hauptfenster.
     */
    public final Dimension getHauptfensterGroesse() {
        
        return getDimension( 
                HAUPTFENSTER_GROESSE, 
                HAUPTFENSTER_GROESSE_STANDARD );
    }

    /** 
     * Speichert die groesse fuer das Hauptfenster. Es handelt sich hierbei um
     * die Groesse des JFrames, der verwendet wird, wenn das Programm als 
     * normale Anwendung laeuft und nicht als Plugin fuer den DSA-Desktop.
     * 
     *  @param groesse Die Groesse des Hauptfensters wie sei von getSize() 
     *          geliefert wird.
     */
    public final void setHauptfensterGroesse ( final Dimension groesse ) {
        
        setDimension( HAUPTFENSTER_GROESSE, groesse );
    }

    /**
     * Liefert die groesse fuer das Hauptfenster, wenn das Programm als Plugin
     * fuer den DSA-Desktop betrieben wird.
     * @return Die groesse fuer den Internal Frame.
     */
    public final Dimension getHauptfensterGroesseDsaDesktop() {
        
        return getDimension( 
                HAUPTFENSTER_GROESSE_DSA_DESKTOP, 
                HAUPTFENSTER_GROESSE_STANDARD );
    }

    /** 
     * Speichert die groesse fuer das Hauptfenster. Es handelt sich hierbei um
     * die Groesse des Internal Frames, der verwendet wird, wenn das Programm 
     * als Plugin fuer den DSA-Desktop betrieben wird.
     * 
     *  @param groesse Die Groesse des Frames wie sei von getSize() 
     *          geliefert wird.
     */
    public final void setHauptfensterGroesseDsaDesktop ( 
            final Dimension groesse ) {
        
        setDimension( HAUPTFENSTER_GROESSE_DSA_DESKTOP, groesse );
    }
    
    /**
     * @return der Pfad zum Verzeichnis in dem die RTF-Dateien der Helden
     *         gespeichert werden.
     */
    public final String getHeldenRtfVerzeichnis() {

        return einstellungen.get( HELDEN_RTF_VERZEICHNIS, USER_HOME );
    }

    /**
     * @param pfad
     *            der Pfad zum Verzeichnis in dem die RTF-Dateien der Helden
     *            gespeichert werden.
     */
    public final void setHeldenRtfVerzeichnis( final String pfad ) {

        if ( null == pfad ) {
            throw new IllegalArgumentException(
                    "null ist kein gueltiger Pfad." );
        }

        put( HELDEN_RTF_VERZEICHNIS, pfad );
    }

    /**
     * @return der Pfad zum Verzeichnis in dem die XML-Dateien der Helden
     *         gespeichert werden.
     */
    public final String getHeldenXmlVerzeichnis() {

        return einstellungen.get( HELDEN_XML_VERZEICHNIS, USER_HOME );
    }

    /**
     * @param pfad
     *            der Pfad zum Verzeichnis in dem die XML-Dateien der Helden
     *            gespeichert werden.
     */
    public final void setHeldenXmlVerzeichnis( final String pfad ) {

        if ( null == pfad ) {
            throw new IllegalArgumentException(
                    "null ist kein gueltiger Pfad." );
        }

        put( HELDEN_XML_VERZEICHNIS, pfad );
    }
    
    /**
     * @return Das Look & Feel fuer die Anwendung.
     */
    public final String getLookAndFeel() {
        
        String defaultLookAndFeel = UIManager.getSystemLookAndFeelClassName();
        
        if ( System.getProperty( "os.name" ).contains( "Linux" ) ) {

            // Unter Linux auf jeden Fall GTK benutzen.
            // Wenn kein Gnome Desktop laeuft wird als System L&F das
            // Standard Java L&F geliefert.
            defaultLookAndFeel = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
            //laF = "org.freeasinspeech.kdelaf.KdeLAF";
            // PlasticLookAndFeel.setPlasticTheme(new ExperienceBlue());
            // laF = "com.jgoodies.looks.plastic.PlasticXPLookAndFeel";
        }

        return einstellungen.get( LOOK_AND_FEEL, defaultLookAndFeel );
    }
    
    /**
     * @param laf Der vollstaendige Klassenname des eingestellten Look & Feels,
     *          also Package und Klassenbezeichnung.
     */
    public final void setLookAndFeel( final String laf ) {
        
        put( LOOK_AND_FEEL, laf );
    }
    
    /**
     * Bestimmt die Sprache in der das Programm erscheint.
     * Die Sprache wird normalerweise aus dem Default Locale bestimmt, das vom
     * Betriebssystem vorgegeben wird. Die Sprache kann aber auch optional
     * ueberschrieben werden, damit das System auf einem englischen System auf
     * Deutsch erscheint ({@link #isSpracheGesetzt()}). Diese Methode kann immer
     * verwendet werden, um die Sprache für das Programm zu bestimmen, nicht nur
     * wenn eine bestimmte Sprache vom Benutzer gewaehlt wurde.
     * 
     * @return Ein Locale bei dem nur die Sprache gesetzt ist.
     * 
     * @see isSpracheGesetzt()
     */
    public final Locale getSprache() {
        
        final String sprache;
        
        if ( isSpracheGesetzt() ) {
            
            sprache = einstellungen.get( 
                    SPRACHE, 
                    Locale.getDefault().getLanguage() );
        
        } else {
            
            sprache = Locale.getDefault().getLanguage();
        }
        
        return new Locale( sprache );
    }
    
    /**
     * Aendert die Sprache in der das Programm erscheint. Damit die hier
     * gesetzte Sprache vom Programm verwendet wird, muss SpracheGesetzt auf
     * {@code true} gesetzt werden ({@link setSpracheGesetzt(boolean)}). Eine
     * Aenderung der Spracheinstellungen benoetigt einen Neustart, um aktiv zu
     * werden.
     * 
     * @param sprache
     *            Aendert die Sprache in der das Programm erscheint.
     */
    public final void setSprache( final Locale sprache ) {
        
        einstellungen.put( SPRACHE, sprache.getLanguage() );
    }
    
    /**
     * Gibt an, ob das Programm in einer vom Benutzer festgelegten Sprache
     * erscheinen soll.
     * 
     * @return {@code true}, wenn es eine vom Benutzer gewaehlte Sprache gibt,
     *         sonst {@code false}.
     */
    public final boolean isSpracheGesetzt() {
        
        return einstellungen.getBoolean( SPRACHE_GESETZT, false );
    }
    
    /**
     * Legt fest ob eine vom Benutzer in den Einstellungen festgelegte Sprache
     * fuer das Programm verwendet werden soll und nicht die Standard
     * System-Sprache.
     * 
     * @param gesetzt
     *            {@code true}, wenn der Benutzer eine eigene Sprache gewaehlt
     *            hat, sonst {@code false}.
     */
    public final void setSpracheGesetzt( final boolean gesetzt ) {
        
        einstellungen.putBoolean( SPRACHE_GESETZT, gesetzt );
    }
}
