/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.util;

import java.util.List;

import de.bernhardjung.dsaprogramm.i18n.I18n;

public class Strings {

    /** Ein Komma. */
    private static final String KOMMA = ", ";
    
    /**
     * Erzeugt eine Komma separierte Aufzaehlung.
     * Die {@code worte} werden mittels {@link Object#toString()} in Strings 
     * umgewandelt.
     * Enthaelt {@code worte} nur ein Element wird nur dieses Element in einen
     * String umgewandelt und zurueckgeliefert. Ist die Liste leer wird ein 
     * leerer String zurueckgegeben. 
     * 
     * @param worte die aufgezaehlt werden sollen.
     * @return eine Aufzaehlung wie z.B. "Singen, Tanzen, Zechen"
     */
    public static final String erzeugeAufzaehlung( final List< ? > worte ) {
        return erzeugeAufzaehlung( worte, KOMMA );
    }
    
    /**
     * Erzeugt eine Aufzaehlung die mit oder abgeschlossen wird.
     * Die {@code worte} werden mittels {@link Object#toString()} in Strings 
     * umgewandelt.
     * Enthaelt {@code worte} nur ein Element wird nur dieses Element in einen
     * String umgewandelt und zurueckgeliefert. Ist die Liste leer wird ein 
     * leerer String zurueckgegeben.
     * 
     * @param worte die aufgezaehlt werden sollen.
     * @return ein String mit einer Aufzaehlung wie 
     *      "Singen, Tanzen oder Zechen".
     */
    public static final String erzeugeAufzaehlungMitOder( 
            final List< ? > worte ) {
        return erzeugeAufzaehlung( worte, getOder() );
    }

    /**
     * Erzeugt eine Aufzaehlung die mit und abgeschlossen wird.
     * Die {@code worte} werden mittels {@link Object#toString()} in Strings 
     * umgewandelt.
     * Enthaelt {@code worte} nur ein Element wird nur dieses Element in einen
     * String umgewandelt und zurueckgeliefert. Ist die Liste leer wird ein 
     * leerer String zurueckgegeben.
     * 
     * @param worte die aufgezaehlt werden sollen.
     * @return ein String mit einer Aufzaehlung wie 
     *      "Singen, Tanzen und Zechen".
     */
    public static final String erzeugeAufzaehlungMitUnd( 
            final List< ? > worte ) {
        return erzeugeAufzaehlung( worte, getUnd() );
    }

    /**
     * Erzeugt eine Aufzaehlung die mit 'und/oder' abgeschlossen wird.
     * Die {@code worte} werden mittels {@link Object#toString()} in Strings 
     * umgewandelt.
     * Enthaelt {@code worte} nur ein Element wird nur dieses Element in einen
     * String umgewandelt und zurueckgeliefert. Ist die Liste leer wird ein 
     * leerer String zurueckgegeben. 
     * 
     * @param worte die aufgezaehlt werden sollen.
     * @return ein String mit einer Aufzaehlung wie 
     *      "Singen, Tanzen oder Zechen".
     */
    public static final String erzeugeAufzaehlungMitUndOder( 
            final List< ? > worte ) {
        return erzeugeAufzaehlung( worte, getUndOder() );
    }

    /**
     * Erzeugt eine mit Komma separierte Aufzaehlung die mit einem vorgegebenen 
     * Wort abgeschlossen wird.
     *  
     * @param worte fuer die Aufzaehlung
     * @param wort mit dem die Aufzaehlung abgeschlossen wird, z.B. " und " 
     *      oder " oder ". Man beachte das links und rechts von dem Wort ein 
     *      Leerzeichen stehen muss.
     * @return ein String mit einer Aufzaehlung wie 
     *      "Singen, Tanzen oder Zechen".
     */
    private static final String erzeugeAufzaehlung( 
            final List< ? > worte, final String wort ) {
        
        final StringBuilder aufzaehlung = new StringBuilder();
        final int laenge = worte.size();
        
        if ( laenge > 0 ) {
            
            aufzaehlung.append( worte.get( 0 ) );
            
            for ( int i = 1 ; i < laenge-1 ; ++i ) {
                
                aufzaehlung.append( ", " );
                
                aufzaehlung.append( worte.get( i ) );
            }
            
            if ( laenge > 1 ) {
                
                aufzaehlung.append( wort );
                
                aufzaehlung.append( worte.get( laenge - 1 ) );
            }
        }
        
        return aufzaehlung.toString();
    }
    
    /** Das Woertchen 'oder'. */
    private static final String getOder() {
        
        final StringBuilder oder = new StringBuilder( 6 );
        
        oder.append(  " " ).append( I18n.i18n( "Strings.oder" ) ).append( " " );
        
        return oder.toString();
    }

    /** Das Wort 'und'. */
    private static final String getUnd() {
        
        final StringBuilder und = new StringBuilder( 5 );
        
        und.append(  " " ).append( I18n.i18n( "Strings.und" ) ).append( " " );
        
        return und.toString();
    }
    
    /** Der Ausdruck 'und/oder'. */
    private static final String getUndOder() {
        
        final StringBuilder undOder = new StringBuilder( 10 );
        
        undOder.append( " " );
        
        undOder.append( I18n.i18n( "Strings.undOder" ) );
        
        undOder.append( " " );
        
        return undOder.toString();
    }

}
