/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.util;

import java.awt.Component;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public class Uebersetzer implements Uebersetzbar {

    /**
     * Die zu uebersetzenden Elemente.
     */
    private final Collection< Uebersetzbar > uebersetzbarCollection;
    
    /**
     * Tabbedpanes deren Tab-Titel uebersetzt werden sollen. Zu jeder Tabbedpane
     * wird der Index der Tabs mit den zugehoerigen Lokalisierungschluesseln
     * gespeichert. 
     */
    private final Map< JTabbedPane, Map< Integer, String > > tabbedPanes;
    
    public Uebersetzer() {
        
        uebersetzbarCollection = new ArrayList< Uebersetzbar >();
        
        tabbedPanes = new HashMap< JTabbedPane, Map<Integer,String> >();
    }
    
    /**
     * Fuegt den Uebersetzer ein neues Element hinzu, dass er uebersetzen soll.
     * 
     * @param uebersetzbar Das zu uebersetzende Element.
     */
    public final void einfuegen( final Uebersetzbar uebersetzbar ) {
        
        uebersetzbarCollection.add( uebersetzbar );
    }
    
    /**
     * Erzeugt einen Knopf, der automatisch neu uebersetzt wird, wenn die
     * neuUebersetzen-Methode dieses Objekts aufgerufen wird.
     * 
     * @param l10nschluessel Der Lokalisierungschluessel der verwendet wird
     *          um den uebersetzten Text aus der Lokalisierungsdatei zu laden.
     * @return einen Knopf mit dem lokalisierten Text.
     */
    public final JButton erzeugeButton( final String l10nschluessel ) {
        
        return erzeugeButton( l10nschluessel, null, null );
    }

    /**
     * Erzeugt einen Knopf, der automatisch neu uebersetzt wird, wenn die
     * neuUebersetzen-Methode dieses Objekts aufgerufen wird.
     * 
     * @param l10nschluesselText Der Lokalisierungschluessel der verwendet wird
     *          um den uebersetzten Text aus der Lokalisierungsdatei zu laden.
     * @param l10nschluesselMnemonic Der Lokalisierungschluessel fuer das
     *          Mnemonic des Buttons.
     * @return einen Knopf mit dem lokalisierten Text.
     */
    public final JButton erzeugeButton( 
            final String l10nschluesselText,
            final String l10nschluesselMnemonic ) {
        
        return erzeugeButton( 
                l10nschluesselText, 
                l10nschluesselMnemonic, 
                null );
    }

    /**
     * Erzeugt einen Knopf, der automatisch neu uebersetzt wird, wenn die
     * neuUebersetzen-Methode dieses Objekts aufgerufen wird.
     * 
     * @param l10nschluesselText Der Lokalisierungschluessel der verwendet wird
     *          um den uebersetzten Text aus der Lokalisierungsdatei zu laden.
     * @param l10nschluesselMnemonic Der Lokalisierungschluessel fuer das
     *          Mnemonic des Buttons.
     * @return einen Knopf mit dem lokalisierten Text.
     */
    public final JButton erzeugeButton( 
            final String l10nschluesselText,
            final String l10nschluesselToolTip,
            final String l10nschluesselMnemonic ) {
        
        final JButton button = new JButton();
        
        einfuegen( new UebersetzerButton( 
                button, 
                l10nschluesselText,
                null,
                l10nschluesselMnemonic) );
        
        return button;
    }
    
    /**
     * Erzeugt eine Checkbox, deren Text automatisch von diesem Uebersetzer 
     * uebersetzt wird.
     * 
     * @param l10nschluesselText Der Lokalisierungsschluessel fuer den Text
     *          der Checkbox.
     * @return Die Checkbox mit dem uebersetzten Text.
     */
    public final JCheckBox erzeugeCheckBox(
            final String l10nschluesselText ) {
        
        return erzeugeCheckBox( l10nschluesselText, null, null );
    }

    /**
     * Erzeugt eine Checkbox, deren Text automatisch von diesem Uebersetzer 
     * uebersetzt wird.
     * 
     * @param l10nschluesselText Der Lokalisierungsschluessel fuer den Text
     *          der Checkbox.
     * @return Die Checkbox mit dem uebersetzten Text.
     */
    public final JCheckBox erzeugeCheckBox(
            final String l10nschluesselText,
            final String l10nschluesselTooltip,
            final String l10nschluesselMnemonic ) {
        
        final JCheckBox checkBox = new JCheckBox();
        
        einfuegen( new UebersetzerButton(
                checkBox,
                l10nschluesselText,
                l10nschluesselTooltip,
                l10nschluesselMnemonic ) );
        
        return checkBox;
    }

    /**
     * Erzeugt eine Combobox deren Elemente vom Uebersetzer uebersetzt werden.
     * Das Uebersetzen wird ueber das Model der Combobox gemacht. Es darf also
     * nicht ausgetauscht werden, wenn das Uebersetzen funktionieren soll.
     * 
     * @param <T>
     * @param elemente Die Elemente die in der Combobox angezeigt werden sollen.
     * @param comparator Um die Elemente zu sortieren.
     * @return Die Liste mit den Elementen.
     */
    public final < T > JComboBox erzeugeComboBox(
            final T ... elemente ) {
        
        final CharGenListModel< T, T > model = new CharGenListModel< T, T >(
                Arrays.asList( elemente ) );
        
        einfuegen( model );
        
        return new JComboBox( model );
    }

    /**
     * Erzeugt eine Combobox deren Elemente vom Uebersetzer uebersetzt werden.
     * Das Uebersetzen wird ueber das Model der Combobox gemacht. Es darf also
     * nicht ausgetauscht werden, wenn das Uebersetzen funktionieren soll.
     * 
     * @param <T>
     * @param <S>
     * @param model Fuer die Combobox.
     * @return Die Combobox mit dem Model.
     */
    public final <T, S extends T > JComboBox erzeugeComboBox(
            final CharGenListModel< T, S > model ) {
        
        einfuegen( model );
        
        return new JComboBox( model );
    }

    /**
     * Erzeugt eine Combobox deren Elemente vom Uebersetzer uebersetzt werden.
     * Das Uebersetzen wird ueber das Model der Combobox gemacht. Es darf also
     * nicht ausgetauscht werden, wenn das Uebersetzen funktionieren soll.
     * 
     * @param <T>
     * @param elemente Die Elemente die in der Combobox angezeigt werden sollen.
     * @return Die Comobobox mit den Elementen.
     */
    public final <T > JComboBox erzeugeComboBox(
            final List< T > elemente ) {
        
        final CharGenListModel< T, T > model = new CharGenListModel< T, T >( elemente );
        
        einfuegen( model );
        
        return new JComboBox( model );
    }
    
    /**
     * Erzeugt eine Combobox deren Elemente vom Uebersetzer uebersetzt werden.
     * Das Uebersetzen wird ueber das Model der Combobox gemacht. Es darf also
     * nicht ausgetauscht werden, wenn das Uebersetzen funktionieren soll.
     * 
     * @param <T>
     * @param <S>
     * @param elemente Die Elemente die in der Combobox angezeigt werden sollen.
     * @param comparator Um die Elemente zu sortieren.
     * @return Die Comobobox mit den Elementen.
     */
    public final <T, S extends T > JComboBox erzeugeComboBox(
            final List< S > elemente,
            final Comparator< T > comparator ) {
        
        final CharGenListModel< T, S > model = new CharGenListModel< T, S >(
                elemente,
                comparator );
        
        einfuegen( model );
        
        return new JComboBox( model );
    }
    
    /**
     * Erzeugt einen Frame, der automatisch vom Uebersetzer uebersetzt wird.
     * 
     * @param l10nschluessel Der Lokalisierungsschluessel fuer den Titel des
     *          Frames.
     * @param component Der Inhalt des Frames. Wenn er uebersetzbar ist, so
     *          wird er ebenfalls uebersetzt.
     * @return Der Frame mit dem uebersetzten Titel und Inhalt.
     */
    public final SimpleInternalFrame erzeugeFrame(
            final String l10nschluessel,
            final JComponent component ) {
        
        final String titel = "<html><b>" + I18n.i18n( l10nschluessel );
        
        final SimpleInternalFrame frame = new SimpleInternalFrame( titel );
        
        frame.setContent( component );
        
        einfuegen( new UebersetzerFrame( frame, l10nschluessel ) );
        
        return frame;
    }
    
    /**
     * Erzeugt ein Label, dass bei einem Aufruf von {@link #neuUebersetzen()}
     * auf dieses Objekt automatisch neu uebersetzt wird.
     * 
     * @param l10nschluessel Der Lokalisierungschluessel der verwendet wird
     *          um den uebersetzten Text fuer das Label aus der 
     *          Lokalisierungsdatei zu laden.
     * @return ein Label mit dem lokalisierten Text.
     */
    public final JLabel erzeugeLabel( final String l10nschluessel ) {
        
        final JLabel label = new JLabel();
        
        einfuegen( new UebersetzerLabel( label, l10nschluessel ) );
        
        return label;
    }

    /**
     * Erzeugt eine Liste deren Elemente vom Uebersetzer uebersetzt werden.
     * Das Uebersetzen wird ueber das Model der Liste gemacht. Es darf also
     * nicht ausgetauscht werden, wenn das Uebersetzen funktionieren soll.
     * 
     * @param <T>
     * @param <S>
     * @param elemente Die Elemente die in der Liste angezeigt werden sollen.
     * @param comparator Um die Elemente zu sortieren.
     * @return Die Liste mit den Elementen.
     */
    public final <T, S extends T > JList erzeugeListe(
            final List< S > elemente,
            final Comparator< T > comparator ) {
        
        final CharGenListModel< T, S > model = new CharGenListModel< T, S >(
                elemente,
                comparator );
        
        einfuegen( model );
        
        return new JList( model );
    }

    /**
     * Erzeugt eine Liste mit einem vorgegebenen Model.
     * 
     * @param <T>
     * @param <S>
     * @param model Das Model der Liste.
     * @return die Liste mit den Model.
     */
    public final < T, S extends T > JList erzeugeListe(
            final CharGenListModel< T, S > model ) {
        
        einfuegen( model );
        
        return new JList( model );
    }

    /**
     * Erzeugt einen Radiobutton, dessen Text bei einem Aufruf von {@link 
     * #neuUebersetzen()} automatisch neu Uebersetzt wird.
     * 
     * @param l10nschluesselText Der Lokalisierungsschluessel fuer den Text
     *          des Buttons.
     * @return Der uebersetzte Button.
     */
    public final JRadioButton erzeugeRadioButton(
            final String l10nschluesselText ) {
        
        final JRadioButton radioButton = new JRadioButton();
        
        einfuegen( new UebersetzerButton(
                radioButton,
                l10nschluesselText,
                null,
                null ) );
        
        return radioButton;
    }
    
    /**
     * Erzeugt einen Separator, dessen Text automatisch uebersetzt wird.
     *  
     * @param l10nschluesselText Der Lokalisierungsschluesse der benutzt wird,
     *          um den Text des Separators zu uebersetzen. 
     * @return Eine Separator.
     */
    public final Component erzeugeSeparator( final String l10nschluesselText ) {
        
        final JLabel label = erzeugeLabel( l10nschluesselText );
        
        label.setHorizontalAlignment( JLabel.LEFT );
        
        return DefaultComponentFactory.getInstance().createSeparator( label );
    }
    
    /**
     * Erzeugt ein neues Tab in einer Tabbedpane. Der Titel und ggf. der Inhalt
     * des Tabs werden automatisch vom Uebersetzer uebersetzt.
     * Damit das Uebersetzen funktioniert duerfen Tabs nur ueber diesen 
     * Uebersetzer hinzugefuegt und entfernt werden.
     * 
     * @param tabbedPane Die Tabbedpane, die ein neues Tab erhalten soll.
     * @param l10nschluessel Der Lokalisierungschluessel fuer den Titel des 
     *          Tabs.
     * @param inhalt Der Inhalt des Tabs. Wenn er {@link Uebersetzbar} ist 
     *          wird auch er automatisch uebersetzt.
     */
    public final void erzeugeTab(
            final JTabbedPane tabbedPane,
            final String l10nschluessel,
            Component inhalt ) {
        
        if ( ! tabbedPanes.containsKey( tabbedPane ) ) {
            
            tabbedPanes.put( tabbedPane, new HashMap< Integer, String >() );
        }
        
        final Map< Integer, String > indexMap = tabbedPanes.get( tabbedPane );
        
        final int index = tabbedPane.getTabCount();
        
        indexMap.put( index, l10nschluessel );
        
        tabbedPane.addTab( I18n.i18n( l10nschluessel ), inhalt );
        
        if ( inhalt instanceof JScrollPane ) {
            
            inhalt = ((JScrollPane) inhalt).getViewport().getView();
        }

        if ( inhalt instanceof Uebersetzbar ) {
            
            einfuegen((Uebersetzbar) inhalt);
        }
    }
    
    /**
     * Erzeugt ein Label, dass bei einem Aufruf von {@link #neuUebersetzen()}
     * auf dieses Objekt automatisch neu uebersetzt wird.
     * 
     * @param l10nschluessel Der Lokalisierungschluessel der verwendet wird
     *          um den uebersetzten Text fuer das Label aus der 
     *          Lokalisierungsdatei zu laden.
     * @return ein Label mit dem lokalisierten Text.
     */
    public final JLabel erzeugeUeberschrift( final String l10nschluessel ) {
        
        final JLabel label = 
            Util.erzeugeUeberschrift( I18n.i18n( l10nschluessel ) );
        
        einfuegen( new UebersetzerLabel( label, l10nschluessel ) );
        
        return label;
    }
    
    /** Uebersetzt die Elemente in diesem Uebersetzer neu. */
    public void neuUebersetzen() {
     
        for ( final Uebersetzbar uebersetzbar : uebersetzbarCollection ) {
            
            uebersetzbar.neuUebersetzen();
        }
        
        for ( final JTabbedPane tabbedPane : tabbedPanes.keySet() ) {
            
            final Map< Integer, String > indexMap = 
                tabbedPanes.get( tabbedPane );
            
            for ( final Integer index : indexMap.keySet() ) {
                
                final String l10nschluessel = indexMap.get( index );
                
                final String titel = I18n.i18n( l10nschluessel );
                
                tabbedPane.setTitleAt( index, titel );
            }
        }
    }
    
    /**
     * Uebersetzt die uebergebenen Element. Jede Referenz wird dabei vorher 
     * darauf geprueft dass sie != {@code null} ist. null-Referenzen werden
     * uebergangen.
     * 
     * @param uebersetzbarArray Die zu uebersetzenden Elemente.
     */
    public static void neuUebersetzen( 
            final Uebersetzbar ... uebersetzbarArray ) {
        
        for ( final Uebersetzbar uebersetzbar : uebersetzbarArray ) {
            
            if ( null != uebersetzbar ) {
                
                uebersetzbar.neuUebersetzen();
            }
        }
    }
}
