/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.util;

import de.bernhardjung.dsaprogramm.Start;
import de.bernhardjung.dsaprogramm.i18n.I18n;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.net.URL;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class Util {

    /** Ein Collator zum sprachspezifischen Vergleichen von Strings. */
    public static Collator COLLATOR;

    /** Die Farbe die zum Markieren von Regelproblemen verwendet wird. */
    public static final Color FEHLERFARBE = Color.RED;
    
    /** Erlaubt es einen JSpinner mit dem Mausrad zu bedienen. */
    private static final MyMouseWheelListener SPINNER_MOUSE_WHEEL_LISTENER =
        new MyMouseWheelListener();

    /** Eine Schriftart die als Ueberschrift verwendet werden kann. */
    private static final Font UEBERSCHRIFT;

    /**
     * Der Wert der fuer das Attribut Unit Increment eines
     * {@link JScrollBar} verwendet wird.
     */
    public static final int UNIT_INCREMENT = 30;

    /**
     * Die Option Ja fuer einen Ja-Nein-Bestaetigungsdialog.
     */
    private static final String JA = "Util.Ja";
    
    /**
     * Die Option Nein fuer einen Ja-Nein-Bestaetigungsdialog.
     */
    private static final String NEIN = "Util.Nein";
    
    static {
        final Einstellungen einstellungen = Einstellungen.getInstanz();
        
        COLLATOR = Collator.getInstance( einstellungen.getSprache() );
        
        UEBERSCHRIFT = new JLabel().getFont().deriveFont( Font.BOLD );
    }

    /**
     * Aktualisiert das Look & Feel bei den uebergebenen Objekten.
     * 
     * @param objekte
     */
    public static void aktualisiereLookAndFeel(
            final LookAndFeelAktualisierbar ... objekte ) {
        
        for ( final LookAndFeelAktualisierbar objekt : objekte ) {
            
            if ( null != objekt ) {
                
                objekt.aktualisiereLookAndFeel();
            }
        }
    }

    public static int compare( final String s1, final String s2 ) {
        
        return COLLATOR.compare( s1, s2 );
    }
    
    /**
     * Erzeugt eienn Filechooser, der Datei nach einer bestimmten Dateiendung
     * filtert. 
     * 
     * @param dateierweiterung der zu filternden Dateien.
     * @return Ein Filechooser.
     */
    public static final JFileChooser erzeugeFileChooser( 
            final String dateierweiterung ) {
        
        final JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        fileChooser.setMultiSelectionEnabled( false );
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.setFileFilter( new FileFilter( dateierweiterung ) );

        return fileChooser;
    }
    
    /**
     * @return ein Label mit rechtsbuendig ausgerichtetem Text.
     */
    public static final JLabel erzeugeLabelRechtsbuendig() {

        final JLabel label = new JLabel();

        label.setHorizontalAlignment( SwingConstants.RIGHT );

        return label;
    }
    
    /**
     * Erzeugt einen ScrollPane fuer eine Liste.
     *  
     * @param liste fuer welche die Scrollpane erzeugt werden soll.
     * @return Die Scrollpane mit der Liste.
     */
    public static final JScrollPane erzeugeScrollPane(
            final JList liste ) {

        final Dimension size = liste.getPreferredScrollableViewportSize();
        
        return erzeugeScrollPane( liste, size.width, size.height );
    }
    
    /**
     * Erzeugt eine Scrollpane mit einer JComponent.
     * 
     * @param inhalt der Scrollpane.
     * @return Die Scrollpane mit dem uebergebenene Inhalt.
     */
    public static final JScrollPane erzeugeScrollPane(
            final JComponent inhalt ) {

        final Dimension size = inhalt.getPreferredSize();
        
        int breite = Math.min( size.width, 320 );
        
        int hoehe = Math.min( size.height, 240 );
        
        return erzeugeScrollPane( inhalt, breite, hoehe );
    }

    /**
     * Erzeugt eine Component in einer Scrollpane von einer vorgegeben Groesse.
     *  
     * @param inhalt der zu erzeugenden Scrollpane.
     * @param breite Die Breite der Scrollpane.
     * @param hoehe Die Hoehe der Scrollpane.
     * @return Die Scrollpane mit gegebenem Inhalt und gegebener Groesse.
     */
    public static final JScrollPane erzeugeScrollPane(
            final JComponent inhalt,
            final int breite,
            final int hoehe ) {

        final JScrollPane scrollPane = new JScrollPane( inhalt );
        
        scrollPane.setPreferredSize( new Dimension( breite, hoehe ) );
        
        return scrollPane;
    }
    
    /**
     * Pakt einen Componente in einer Scrollpane.
     *
     * @param inhalt
     *            die in die Scrollpane soll.
     * @return die Scrollpane mit der Componente.
     */
    public static final JScrollPane erzeugeScrollPane(
            final JPanel inhalt ) {

        final Dimension size = inhalt.getPreferredSize();
        
        int breite = Math.min( size.width, 320 );
        
        int hoehe = Math.min( size.height, 240 );
        
        return erzeugeScrollPane( inhalt, breite, hoehe );
    }
    
    /**
     * Erzeugt einen Scrollpane fuer ein Panel, mit einer vorgegebenen groesse.
     * 
     * @param inhalt der zu erzeugenden Scrollpane.
     * @param breite der Scrollpane.
     * @param hoehe der Scrollpane.
     * @return Die Scrollpane mit dem Panel in der uebergebenen Groesse.
     */
    public static final JScrollPane erzeugeScrollPane(
            final JPanel inhalt, 
            final int breite,
            final int hoehe ) {
        
        final JScrollPane scrollPane = new JScrollPane( inhalt );

        // Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( inhalt.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        scrollPane.setBorder( null );

        final JScrollBar scrollBar = scrollPane.getVerticalScrollBar();

        scrollBar.setUnitIncrement( Util.UNIT_INCREMENT );

        scrollPane.setPreferredSize( new Dimension( breite, hoehe ) );
        
        return scrollPane;
    }
    
    /**
     * Erzeugt einen JSpinner, der mit dem Mausrad bedient werden kann.
     *
     * @param wert der Startwert des Spinners.
     * @param minimum der kleinste noch erlaubte Wert.
     * @param maximum der groesste noch erlaubte Wert.
     * @param schrittweite fuer den Spinner.
     * @return einen Spinner
     */
    public static final JSpinner erzeugeSpinner(
            final double wert,
            final double minimum,
            final double maximum,
            final double schrittweite ) {

        final SpinnerModel model = new SpinnerNumberModel(
                wert,
                minimum,
                maximum,
                schrittweite );

        final JSpinner spinner = new JSpinner( model );

        spinner.addMouseWheelListener( SPINNER_MOUSE_WHEEL_LISTENER );

        return spinner;
    }

    /**
     * Erzeugt einen JSpinner, der mit dem Mausrad bedient werden kann.
     *
     * @param wert der Startwert des Spinners.
     * @param minimum der kleinste noch erlaubte Wert.
     * @param maximum der groesste noch erlaubte Wert.
     * @param schrittweite fuer den Spinner.
     * @return einen Spinner
     */
    public static final JSpinner erzeugeSpinner(
            final int wert,
            final int minimum,
            final int maximum,
            final int schrittweite ) {

        final SpinnerModel model = new SpinnerNumberModel(
                wert,
                minimum,
                maximum,
                schrittweite );

        final JSpinner spinner = new JSpinner( model );

        spinner.addMouseWheelListener( SPINNER_MOUSE_WHEEL_LISTENER );

        return spinner;
    }

    /**
     * @param text fuer die Ueberschrift.
     * @return ein als Ueberschrift formatiertes JLabel.
     */
    public static final JLabel erzeugeUeberschrift( final String text ) {

        final JLabel ueberschrift = new JLabel( text );

        ueberschrift.setFont( UEBERSCHRIFT );

        return ueberschrift;
    }

    /**
     * Vergleicht die Objekte paarweise mit equals. Es wird dabei das erste
     * mit den zweite, das dritte mit dem vierten usw. verglichen.
     * 
     * @param objekte Die zu vergleichenden Objekte.
     * 
     * @return {@code true}, wenn sich die Objekte immer gleiche, sonst
     *         {@code false}.
     */
    public static final boolean equals( final Object ... objekte ) {
        
        for ( int i = 0 ; i < objekte.length ; i += 2 ) {
            
            final Object o1 = objekte[ i ];
            
            final Object o2 = objekte[ i + 1 ];
            
            if ( null == o1 ^ null == o2 ) {
                
                return false;
            
            } else if ( null != o1 && ! o1.equals( o2 ) ) {
                
                return false;
            }
        }
        
        return true;
    }
    
    /**
     * Liefert zu einem Buchstaben die Konstante aus der Klasse
     * {@link KeyEvent}. Zu &quot;A&quot; z.B. {@link KeyEvent#VK_A}.
     *
     * @param buchstabe zu dem die Konstante gesucht ist.
     * @return die Konstante aus {@link KeyEvent}.
     */
    public static final int getKeyCode( final String buchstabe ) {
        return KeyStroke.getKeyStroke( buchstabe ).getKeyCode();
    }

    @SuppressWarnings( "unchecked" )
    public static final < T, S extends T > List< S >  konvertieren (
            final Collection< T > collection ) {
        
        final List< S > liste = new ArrayList< S >();
        
        for ( final T element : collection ) {
            
            liste.add( (S) element );
        }
        
        return liste;
    }
    
    /**
     * Laedt ein png-Icon aus dem Ordner bilder.
     *
     * @param name der Name der Datei wird vorne mit 'bilder/' und hinten
     *      mit '.png' ergaenzt.
     *      <dl><dt>Beispiel:</dt><dd>
     *      Mit name='icon' wird die Datei 'bilder/icon.png' geladen.
     *      </dd></dl>
     *
     * @return ein Bild.
     */
    public static final ImageIcon ladeIcon( final String name ) {

        final URL url = Start.class.getResource( "bilder/" + name + ".png");

        if ( null != url ) {

            return new ImageIcon( url );

        } else {

            throw new IllegalArgumentException( String.format(
                    "Das Icon %1$s konnte nicht gefunden werden.", name ) );
        }
    }
    
    public static final int zeigeJaNeinDialog(
            final Component parent,
            final String frage ) {
        
        final String ja = I18n.i18n( JA );
        
        final String nein = I18n.i18n( NEIN );
        
        final JOptionPane pane = new JOptionPane( 
                frage, 
                JOptionPane.QUESTION_MESSAGE,
                JOptionPane.YES_NO_OPTION,
                null,
                new Object[] { ja, nein },
                nein );
        
        final JDialog dialog = pane.createDialog( parent, null );
        
        dialog.setModal( true );
        
        dialog.setVisible( true );
        
        if( ja == pane.getValue() ) {
            
            return JOptionPane.YES_OPTION;
        
        } else if ( nein == pane.getValue() ) {
            
            return JOptionPane.NO_OPTION;
        
        } else {
            
            return JOptionPane.CANCEL_OPTION;
        }
    };
    
    /**
     * Ein MouseWheelListener der es erlaubt einen JSpinner, der einen Integer
     * Wert darstellt, mit dem Mausrade zu bedienen.
     *
     * @author Bernhard Jung
     */
    private static class MyMouseWheelListener implements MouseWheelListener {
        public final void mouseWheelMoved(final MouseWheelEvent e) {

            final JSpinner spinner = (JSpinner) e.getSource();

            if ( ! spinner.isEnabled() ) {
                return;
            }

            JSpinner.DefaultEditor editor = null;
            editor = (JSpinner.DefaultEditor) spinner.getEditor();

            if ( ! editor.getTextField().hasFocus() ) {
                return;
            }

            final SpinnerNumberModel model;
            model = (SpinnerNumberModel) spinner.getModel();

            if ( e.getWheelRotation() > 0 ) {

                final Object previousValue = model.getPreviousValue();

                if ( null != previousValue ) {

                    model.setValue( previousValue );
                }

            } else {

                final Object nextValue = model.getNextValue();

                if ( null != nextValue ) {

                    model.setValue( nextValue );
                }
            }
        }
    }
}
