/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basis.varianten.Varianten;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenFrei;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.anpasser.Anpasser;
import de.bernhardjung.dsaprogramm.vornachteile.anpasser.AnpasserSonderfertigkeit;
import de.bernhardjung.dsaprogramm.vornachteile.kosten.Kosten;
import de.bernhardjung.dsaprogramm.vornachteile.stufen.Stufen;
import de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen.Voraussetzung;

public class VorNachteil
extends ElementMitNameImpl {

    /**
     * Sortiert die Vor-/Nachteile:
     * 1. Vorteile vor Nachteile
     * 2. alphabetisch nach Name.
     */
    public static final Comparator< VorNachteil > NAME_COMPARATOR =
        new Comparator< VorNachteil >() {

            public final int compare( 
                    final VorNachteil v1,
                    final VorNachteil v2 ) {
    
                if ( v1.istVorteil() && ! v2.istVorteil() ) {
                    return -1;
                }
    
                if ( ! v1.istVorteil() && v2.istVorteil() ) {
                    return 1;
                }
    
                int ergebnis = Util.compare( v1.getName(), v2.getName() );
                
                if ( 0 != ergebnis ) {
                    
                    return ergebnis;
                }
                
                return v1.getId().compareTo( v2.getId() );
            }
    };
    
    /**
     * Dient dazu zu bestimmen ob es sich bei einer Instanz der Klasse um einen
     * Vorteil, einen Nachteil oder eine Schlechte Eigenschaft handelt.
     */
    private enum Typ {
        VORTEIL,
        NACHTEIL,
        SCHLECHTE_EIGENSCHAFT
    }

    /** Eine Liste aller Nachteile. */
    private static final Map< String, VorNachteil > NACHTEILE =
        new HashMap< String, VorNachteil >();

    /**
     * Speichert jeden Vor-/Nachteil unter seiner id ab.
     */
    private static final Map< String, VorNachteil > VOR_NACHTEILE =
        new HashMap< String, VorNachteil >();

    /** Eine Liste aller Vorteile. */
    private static final Map< String, VorNachteil > VORTEILE =
        new HashMap< String, VorNachteil >();

    // ******** //
    //          //
    // Vorteile //
    //          //
    // ******** //

    public static final VorNachteil ADLIGE_ABSTAMMUNG =
        new VorNachteil( "VorNachteil.AdligeAbstammung" );

    public static final VorNachteil AKADEMISCHE_AUSBILDUNG_GELEHRTER =
        new VorNachteil( "VorNachteil.AkademischeAusbildungGelehrter" );

    public static final VorNachteil AKADEMISCHE_AUSBILDUNG_KRIEGER =
        new VorNachteil( "VorNachteil.AkademischeAusbildungKrieger" );

    public static final VorNachteil ASTRALE_REGENERATION_I =
        new VorNachteil( "VorNachteil.AstraleRegenerationI" );

    public static final VorNachteil ASTRALE_REGENERATION_II =
        new VorNachteil( "VorNachteil.AstraleRegenerationII" );

    public static final VorNachteil ASTRALE_REGENERATION_III =
        new VorNachteil( "VorNachteil.AstraleRegenerationIII" );

    public static final VorNachteil ASTRALMACHT =
        new VorNachteil( "VorNachteil.Astralmacht" );

    public static final VorNachteil AUSDAUERND =
        new VorNachteil( "VorNachteil.Ausdauernd" );

    public static final VorNachteil BALANCE =
        new VorNachteil( "VorNachteil.Balance" );

    public static final VorNachteil BEGABUNG_TALENT =
        new VorNachteil( "VorNachteil.BegabungFuerTalent" );

    public static final VorNachteil BEGABUNG_TALENTGRUPPE =
        new VorNachteil( "VorNachteil.BegabungFuerTalentgruppe" );

    public static final VorNachteil BESONDERER_BESITZ =
        new VorNachteil( "VorNachteil.BesondererBesitz" );

    public static final VorNachteil DAEMMERUNGSSICHT =
        new VorNachteil( "VorNachteil.Daemmerungssicht" );

    public static final VorNachteil EISERN =
        new VorNachteil( "VorNachteil.Eisern" );

    public static final VorNachteil FLINK =
        new VorNachteil( "VorNachteil.Flink" );

    public static final VorNachteil GEBILDET =
        new VorNachteil( "VorNachteil.Gebildet" );

    public static final VorNachteil GEFAHRENINSTINKT =
        new VorNachteil( "VorNachteil.Gefahreninstinkt" );

    public static final VorNachteil GUT_AUSSEHEND =
        new VorNachteil( "VorNachteil.GutAussehend" );

    public static final VorNachteil GUTES_GEDAECHTNIS =
        new VorNachteil( "VorNachteil.GutesGedaechtnis" );

    public static final VorNachteil HERAUSRAGENDE_BALANCE =
        new VorNachteil( "VorNachteil.HerausragendeBalance" );

    public static final VorNachteil HERAUSRAGENDE_FF =
        new VorNachteil( "VorNachteil.HerausragendeFingerfertigkeit" );

    public static final VorNachteil HERAUSRAGENDE_GE =
        new VorNachteil( "VorNachteil.HerausragendeGewandtheit" );

    public static final VorNachteil HERAUSRAGENDE_IN =
        new VorNachteil( "VorNachteil.HerausragendeIntuition" );

    public static final VorNachteil HERAUSRAGENDE_KL =
        new VorNachteil( "VorNachteil.HerausragendeKlugheit" );

    public static final VorNachteil HERAUSRAGENDE_KO =
        new VorNachteil( "VorNachteil.HerausragendeKonstitution" );

    public static final VorNachteil HERAUSRAGENDE_KK =
        new VorNachteil( "VorNachteil.HerausragendeKoerperkraft" );

    public static final VorNachteil HERAUSRAGENDE_SICHT =
        new VorNachteil( "VorNachteil.HerausragendeSicht" );

    public static final VorNachteil HERAUSRAGENDER_GERUCHSSINN =
        new VorNachteil( "VorNachteil.HerausragenderGeruchssinn" );

    public static final VorNachteil HERAUSRAGENDER_MU =
        new VorNachteil( "VorNachteil.HerausragenderMut" );

    public static final VorNachteil HERAUSRAGENDER_TASTSINN =
        new VorNachteil( "VorNachteil.HerausragenderTastsinn" );

    public static final VorNachteil HERAUSRAGENDES_AUSSEHEN =
        new VorNachteil( "VorNachteil.HerausragendesAussehen" );

    public static final VorNachteil HERAUSRAGENDES_CH =
        new VorNachteil( "VorNachteil.HerausragendesCharisma" );

    public static final VorNachteil HERAUSRAGENDES_GEHOER =
        new VorNachteil( "VorNachteil.HerausragendesGehoer" );

    public static final VorNachteil HOHE_LEBENSKRAFT =
        new VorNachteil( "VorNachteil.HoheLebenskraft" );

    public static final VorNachteil HOHE_MAGIERESISTENZ =
        new VorNachteil( "VorNachteil.HoheMagieresistenz" );

    public static final VorNachteil IMMUNITAET_GIFT =
        new VorNachteil( "VorNachteil.ImmunitaetGegenGift" );

    public static final VorNachteil IMMUNITAET_GIFTART =
        new VorNachteil( "VorNachteil.ImmunitaetGegenGiftart" );

    public static final VorNachteil IMMUNITAET_GIFTE =
        new VorNachteil( "VorNachteil.ImmunitaetGegenGifte" );

    public static final VorNachteil IMMUNITAET_KRANKHEIT =
        new VorNachteil( "VorNachteil.ImmunitaetGegenKrankheit" );

    public static final VorNachteil IMMUNITAET_KRANKHEITEN =
        new VorNachteil( "VorNachteil.ImmunitaetGegenKrankheiten" );

    public static final VorNachteil INNERER_KOMPASS =
        new VorNachteil( "VorNachteil.InnererKompass" );

    public static final VorNachteil KAMPFRAUSCH =
        new VorNachteil( "VorNachteil.Kampfrausch" );

    public static final VorNachteil MAGIEGESPUER =
        new VorNachteil( "VorNachteil.Magiegespuer" );

    public static final VorNachteil NACHTSICHT =
        new VorNachteil( "VorNachteil.Nachtsicht" );

    public static final VorNachteil PROPHEZEIEN =
        new VorNachteil( "VorNachteil.Prophezeien" );

    public static final VorNachteil RESISTENZ_GIFT =
        new VorNachteil( "VorNachteil.ResistenzGegenGift" );

    public static final VorNachteil RESISTENZ_GIFTART =
        new VorNachteil( "VorNachteil.ResistenzGegenGiftart" );

    public static final VorNachteil RESISTENZ_GIFTE =
        new VorNachteil( "VorNachteil.ResistenzGegenGifte" );

    public static final VorNachteil RESISTENZ_KRANKHEITEN =
        new VorNachteil( "VorNachteil.ResistenzGegenKrankheiten" );

    public static final VorNachteil RICHTUNGSSINN =
        new VorNachteil( "VorNachteil.Richtungssinn" );

    public static final VorNachteil SCHLANGENMENSCH =
        new VorNachteil( "VorNachteil.Schlangenmensch" );

    public static final VorNachteil SCHNELLE_HEILUNG =
        new VorNachteil( "VorNachteil.SchnelleHeilung" );

    public static final VorNachteil SOZIALE_ANPASSUNGSFAEHIGKEIT =
        new VorNachteil( "VorNachteil.SozialeAnpassungsfaehigkeit" );

    public static final VorNachteil VERBINDUNGEN =
        new VorNachteil( "VorNachteil.Verbindungen" );

    public static final VorNachteil VIERTELZAUBERER =
        new VorNachteil( "VorNachteil.Viertelzauberer" );

    public static final VorNachteil VIERTELZAUBERER_UNBEWUSST =
        new VorNachteil( "VorNachteil.ViertelzaubererUnbewusst" );

    public static final VorNachteil VOLLZAUBERER =
        new VorNachteil( "VorNachteil.Vollzauberer" );

    public static final VorNachteil WOHLKLANG =
        new VorNachteil( "VorNachteil.Wohlklang" );

    public static final VorNachteil ZWEISTIMMIGER_GESANG =
        new VorNachteil( "VorNachteil.ZweistimmigerGesang" );

    public static final VorNachteil ZWERGENNASE =
        new VorNachteil( "VorNachteil.Zwergennase" );

    // ********* //
    //           //
    // Nachteile //
    //           //
    // ********* //

    public static final VorNachteil ASTRALER_BLOCK =
        new VorNachteil( "VorNachteil.AstralerBlock" );

    public static final VorNachteil BLUTRAUSCH =
        new VorNachteil( "VorNachteil.Blutrausch" );

    public static final VorNachteil EINAEUGIG =
        new VorNachteil( "VorNachteil.Einaeugig" );
    
    public static final VorNachteil EINARMIG =
        new VorNachteil( "VorNachteil.Einarmig" );

    public static final VorNachteil EINBEINIG =
        new VorNachteil( "VorNachteil.Einbeinig" );

    public static final VorNachteil EINGESCHRAENKTER_GERUCHSSINN =
        new VorNachteil( "VorNachteil.EingeschraenkterGeruchssinn" );

    public static final VorNachteil EINGESCHRAENKTER_TASTSINN =
        new VorNachteil( "VorNachteil.EingeschraenkterTastsinn" );

    public static final VorNachteil EINHAENDIG =
        new VorNachteil( "VorNachteil.Einhaendig" );

    public static final VorNachteil ELFISCHE_WELTSICHT =
        new VorNachteil( "VorNachteil.ElfischeWeltsicht" );

    public static final VorNachteil GELAEHMTER_ARM =
        new VorNachteil( "VorNachteil.GelaehmterArm" );

    public static final VorNachteil GLASKNOCHEN =
        new VorNachteil( "VorNachteil.Glasknochen" );

    public static final VorNachteil FETTLEIBIG =
        new VorNachteil( "VorNachteil.Fettleibig" );

    public static final VorNachteil KRANKHEITSANFAELLIG =
        new VorNachteil( "VorNachteil.Krankheitsanfaellig" );

    public static final VorNachteil KURZATMIG =
        new VorNachteil( "VorNachteil.Kurzatmig" );

    public static final VorNachteil KURZSICHTIG =
        new VorNachteil( "VorNachteil.Kurzsichtig" );

    public static final VorNachteil LAHM =
        new VorNachteil( "VorNachteil.Lahm" );

    public static final VorNachteil MISERABLE_FF =
        new VorNachteil( "VorNachteil.MiserableFingerfertigkeit" );

    public static final VorNachteil MISERABLE_GE =
        new VorNachteil( "VorNachteil.MiserableGewandtheit" );

    public static final VorNachteil MISERABLE_IN =
        new VorNachteil( "VorNachteil.MiserableIntuition" );

    public static final VorNachteil MISERABLE_KL =
        new VorNachteil( "VorNachteil.MiserableKlugheit" );

    public static final VorNachteil MISERABLE_KO =
        new VorNachteil( "VorNachteil.MiserableKonstitution" );

    public static final VorNachteil MISERABLE_KK =
        new VorNachteil( "VorNachteil.MiserableKoerperkraft" );

    public static final VorNachteil MISERABLER_MU =
        new VorNachteil( "VorNachteil.MiserablerMut" );

    public static final VorNachteil MISERABLES_CH =
        new VorNachteil( "VorNachteil.MiserablesCharisma" );

    public static final VorNachteil NACHTBLIND =
        new VorNachteil( "VorNachteil.Nachtblind" );

    public static final VorNachteil NIEDRIGE_ASTRALKRAFT =
        new VorNachteil( "VorNachteil.NiedrigeAstralkraft" );

    public static final VorNachteil NIEDRIGE_LEBENSKRAFT =
        new VorNachteil( "VorNachteil.NiedrigeLebenskraft" );

    public static final VorNachteil NIEDRIGE_MAGIERESISTENZ =
        new VorNachteil( "VorNachteil.NiedrigeMagieresistenz" );

    public static final VorNachteil SCHLECHTE_REGENERATION =
        new VorNachteil( "VorNachteil.SchlechteRegeneration" );

    public static final VorNachteil SCHWERHOERIG =
        new VorNachteil( "VorNachteil.Schwerhoerig" );

    public static final VorNachteil TOTENANGST =
        new VorNachteil( "VorNachteil.Totenangst" );

    public static final VorNachteil UNANGENEHME_STIMME =
        new VorNachteil( "VorNachteil.UnangenehmeStimme" );

    public static final VorNachteil UNANSEHNLICH =
        new VorNachteil( "VorNachteil.Unansehnlich" );

    public static final VorNachteil UNFAEHIGKEIT_TALENT =
        new VorNachteil( "VorNachteil.UnfaehigkeitFuerTalent" );

    public static final VorNachteil UNFAEHIGKEIT_TALENTGRUPPE =
        new VorNachteil( "VorNachteil.UnfaehigkeitFuerTalentgruppe" );

    public static final VorNachteil VORURTEILE_1 =
        new VorNachteil( "VorNachteil.Vorurteile1" );

    public static final VorNachteil VORURTEILE_2 =
        new VorNachteil( "VorNachteil.Vorurteile2" );

    public static final VorNachteil WIDERWAERTIGES_AUSSEHEN =
        new VorNachteil( "VorNachteil.WiderwaertigesAussehen" );

    public static final VorNachteil ZWERGENWUCHS =
        new VorNachteil( "VorNachteil.Zwergenwuchs" );

    /**
     * Fuegt einen Vor-/Nachteil in die Map {@link #VOR_NACHTEILE} ein.
     *
     * @param vorNachteil
     *            der in die Map eingefuegt werden soll.
     */
    private static final void einfuegen( final VorNachteil vorNachteil ) {

        assert ( ! VOR_NACHTEILE.containsKey( vorNachteil.getId() ) ) :
            String.format( "Der Vor-/Nachteil %1$s sollte ein zweites mal " +
                    "in die Map der Vor-/Nachteile eingefuegt werden.",
                    vorNachteil.getName() );

        VOR_NACHTEILE.put( vorNachteil.getId(), vorNachteil );

        final Map< String, VorNachteil > map;
        
        if ( vorNachteil.istVorteil() ) {

            map = VORTEILE;

        } else {

            map = NACHTEILE;
        }

        map.put( vorNachteil.getId(), vorNachteil );
    }

    /**
     * @return eine Liste aller Nachteile.
     */
    public static final List< VorNachteil > getNachteile() {
        
        final List< VorNachteil > liste = 
            new ArrayList< VorNachteil >( NACHTEILE.values() );
        
        Collections.sort( liste, NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * @param id des gesuchten Vor-/Nachteils
     * @return ein Vor-/Nachteil mit der uebergebenen ID.
     * @throws IllegalArgumentException wenn kein Vor-/Nachteil mit dieser ID
     *      existiert.
     */
    public static final VorNachteil getVorNachteil( final String id ) {

        if ( VOR_NACHTEILE.containsKey( id ) ) {

            return VOR_NACHTEILE.get( id );

        } else {

            throw new IllegalArgumentException( "Es existiert kein " +
                    "Vor-/Nachteil mit der Id " + id );
        }
    }

    /**
     * @return eine Liste aller Vor- und Nachteile.
     */
    public static final List< VorNachteil > getVorNachteile() {

        final List< VorNachteil > liste =
            new ArrayList< VorNachteil >( VOR_NACHTEILE.size() );

        liste.addAll( VORTEILE.values() );

        liste.addAll( NACHTEILE.values() );

        Collections.sort( liste, NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * @return eine Liste aller Vorteile.
     */
    public static final List< VorNachteil > getVorteile() {
        
        final List< VorNachteil > liste = 
            new ArrayList< VorNachteil >( VORTEILE.values() );
        
        Collections.sort( liste, NAME_COMPARATOR );
        
        return liste;
    }

    /**
     * Initialisiert alle Vor-/Nachteile.
     */
    public static final void initialisieren( ) {

        // ******** //
        //          //
        // Vorteile //
        //          //
        // ******** //

        initialisieren(
                ADLIGE_ABSTAMMUNG,
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null,
                Anpasser.SO_8,
                Voraussetzung.SO_8 );

        initialisieren(
                AKADEMISCHE_AUSBILDUNG_GELEHRTER,
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null,
                new AnpasserSonderfertigkeit(
                        Sonderfertigkeit.NANDUSGEFAELLIGES_WISSEN ) );
        AKADEMISCHE_AUSBILDUNG_GELEHRTER.einfuegen( Anpasser.ALTER );

        initialisieren(
                AKADEMISCHE_AUSBILDUNG_KRIEGER,
                Typ.VORTEIL,
                Kosten.GP_20,
                null, null, null,
                Voraussetzung.NICHT_FREI_WAEHLBAR );

        initialisieren(
                "VorNachteil.Altersresistenz",
                Typ.VORTEIL,
                Kosten.GP_3,
                null, null, null );

        initialisieren(
                ASTRALE_REGENERATION_I,
                Typ.VORTEIL,
                Kosten.GP_4,
                null, null, null,
                Voraussetzung.nichtKombinierbar( ASTRALER_BLOCK ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_II ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_III ),
                Voraussetzung.ZV )
                .setStaerkereAuspraegung( ASTRALE_REGENERATION_II );

        initialisieren(
                ASTRALE_REGENERATION_II,
                Typ.VORTEIL,
                Kosten.GP_8,
                null, null, null,
                Voraussetzung.nichtKombinierbar( ASTRALER_BLOCK ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_I ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_III ),
                Voraussetzung.ZV )
                .setStaerkereAuspraegung( ASTRALE_REGENERATION_III );
        ASTRALE_REGENERATION_II.setSchwaechereAuspraegung(
                ASTRALE_REGENERATION_I );

        initialisieren(
                ASTRALE_REGENERATION_III,
                Typ.VORTEIL,
                Kosten.GP_12,
                null, null, null,
                Voraussetzung.nichtKombinierbar( ASTRALER_BLOCK ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_I ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_II ),
                Voraussetzung.ZV )
                .setSchwaechereAuspraegung( ASTRALE_REGENERATION_II );

        initialisieren(
                ASTRALMACHT,
                Typ.VORTEIL,
                Kosten.GP_1_pro_1,
                Stufen.STUFE_1_BIS_6,
                null, null,
                Voraussetzung.STUFEN,
                Voraussetzung.nichtKombinierbar( NIEDRIGE_ASTRALKRAFT ),
                Voraussetzung.ZV );

        initialisieren(
                AUSDAUERND,
                Typ.VORTEIL,
                Kosten.GP_1_pro_1,
                Stufen.STUFE_1_BIS_3,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Ausruestungsvorteil",
                Typ.VORTEIL,
                Kosten.GP_1_pro_1,
                Stufen.AUSRUESTUNGSVORTEIL,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                BALANCE,
                Typ.VORTEIL,
                Kosten.BALANCE,
                null, null, null,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_BALANCE ) )
                .setStaerkereAuspraegung( HERAUSRAGENDE_BALANCE );

        initialisieren(
                BEGABUNG_TALENT,
                Typ.VORTEIL,
                Kosten.BEGABUNG_TALENT,
                null,
                Varianten.BEGABUNG_TALENT,
                null,
                Voraussetzung.BEGABUNG_TALENT,
                Voraussetzung.TALENT );

        initialisieren(
                BEGABUNG_TALENTGRUPPE,
                Typ.VORTEIL,
                Kosten.BEGABUNG_TALENTGRUPPE,
                null,
                Varianten.TALENTGRUPPE,
                null,
                Voraussetzung.BEGABUNG_TALENTGRUPPE,
                Voraussetzung.BEGABUNG_TALENTGRUPPE_2 );

        initialisieren(
                BESONDERER_BESITZ,
                Typ.VORTEIL,
                Kosten.BESONDERER_BESITZ,
                null,
                Varianten.BESONDERER_BESITZ,
                null,
                Voraussetzung.BESONDERER_BESITZ );

        initialisieren(
                DAEMMERUNGSSICHT,
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( NACHTSICHT ),
                Voraussetzung.nichtKombinierbar( NACHTBLIND ) )
                .setStaerkereAuspraegung( NACHTSICHT );

        initialisieren(
                EISERN,
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null, null );

        initialisieren(
                "VorNachteil.Feenfreund",
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null, null );

        initialisieren(
                FLINK,
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null );

        initialisieren(
                GEBILDET,
                Typ.VORTEIL,
                Kosten.GP_1_pro_1,
                Stufen.STUFE_1_BIS_5,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                GEFAHRENINSTINKT,
                Typ.VORTEIL,
                Kosten.GP_12,
                null, null,
                Anpasser.GEFAHRENINSTINKT );

        initialisieren(
                "VorNachteil.Glueck",
                Typ.VORTEIL,
                Kosten.GP_12,
                null, null, null );

        initialisieren(
                "VorNachteil.GlueckImSpiel",
                Typ.VORTEIL,
                Kosten.GP_3,
                null, null, null );

        initialisieren(
                GUT_AUSSEHEND,
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( WIDERWAERTIGES_AUSSEHEN ),
                Voraussetzung.nichtKombinierbar( UNANSEHNLICH ),
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDES_AUSSEHEN ) )
                .setStaerkereAuspraegung( HERAUSRAGENDES_AUSSEHEN );

        initialisieren(
                GUTES_GEDAECHTNIS,
                Typ.VORTEIL,
                Kosten.GUTES_GEDAECHTNIS,
                null, null, null );

        initialisieren(
                HERAUSRAGENDE_BALANCE,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_BALANCE,
                null, null, null,
                Voraussetzung.nichtKombinierbar( BALANCE ) )
                .setSchwaechereAuspraegung( BALANCE );

        initialisieren(
                HERAUSRAGENDE_FF,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDE_FF,
                Voraussetzung.nichtKombinierbar( MISERABLE_FF ),
                Voraussetzung.STUFEN );

        initialisieren(
                HERAUSRAGENDE_GE,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDE_GE,
                Voraussetzung.nichtKombinierbar( MISERABLE_GE ),
                Voraussetzung.STUFEN );

        initialisieren(
                HERAUSRAGENDE_IN,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDE_IN,
                Voraussetzung.nichtKombinierbar( MISERABLE_IN ),
                Voraussetzung.STUFEN );

        initialisieren(
                HERAUSRAGENDE_KL,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDE_KL,
                Voraussetzung.nichtKombinierbar( MISERABLE_KL ),
                Voraussetzung.STUFEN );

        initialisieren(
                HERAUSRAGENDE_KK,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDE_KK,
                Voraussetzung.nichtKombinierbar( MISERABLE_KK ),
                Voraussetzung.nichtKombinierbar( SCHLANGENMENSCH ),
                Voraussetzung.STUFEN );

        initialisieren(
                HERAUSRAGENDE_KO,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDE_KO,
                Voraussetzung.nichtKombinierbar( MISERABLE_KO ),
                Voraussetzung.STUFEN );

        initialisieren(
                HERAUSRAGENDE_SICHT,
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( KURZSICHTIG ) );

        initialisieren(
                HERAUSRAGENDER_GERUCHSSINN,
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( EINGESCHRAENKTER_GERUCHSSINN));

        initialisieren(
                HERAUSRAGENDER_MU,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDER_MU,
                Voraussetzung.nichtKombinierbar( MISERABLER_MU ),
                Voraussetzung.STUFEN );

        initialisieren(
                HERAUSRAGENDER_TASTSINN,
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( EINGESCHRAENKTER_TASTSINN ) );

        initialisieren(
                HERAUSRAGENDES_AUSSEHEN,
                Typ.VORTEIL,
                Kosten.GP_12,
                null, null, null,
                Voraussetzung.nichtKombinierbar( WIDERWAERTIGES_AUSSEHEN ),
                Voraussetzung.nichtKombinierbar( UNANSEHNLICH ),
                Voraussetzung.nichtKombinierbar( GUT_AUSSEHEND ) )
                .setSchwaechereAuspraegung( GUT_AUSSEHEND );

        initialisieren(
                HERAUSRAGENDES_CH,
                Typ.VORTEIL,
                Kosten.HERAUSRAGENDE_EIGENSCHAFT,
                Stufen.STUFE_1_BIS_MAX,
                null,
                Anpasser.HERAUSRAGENDES_CH,
                Voraussetzung.nichtKombinierbar( MISERABLES_CH ),
                Voraussetzung.STUFEN  );

        initialisieren(
                HERAUSRAGENDES_GEHOER,
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( SCHWERHOERIG ) );

        initialisieren(
                "VorNachteil.Hitzeresistenz",
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null );

        initialisieren(
                HOHE_LEBENSKRAFT,
                Typ.VORTEIL,
                Kosten.GP_2_pro_1,
                Stufen.STUFE_1_BIS_6,
                null, null,
                Voraussetzung.STUFEN,
                Voraussetzung.nichtKombinierbar( NIEDRIGE_LEBENSKRAFT ) );

        initialisieren(
                HOHE_MAGIERESISTENZ,
                Typ.VORTEIL,
                Kosten.GP_2_pro_1,
                Stufen.STUFE_1_BIS_3,
                null, null,
                Voraussetzung.STUFEN,
                Voraussetzung.nichtKombinierbar( NIEDRIGE_MAGIERESISTENZ ) );

        initialisieren(
                IMMUNITAET_GIFT,
                Typ.VORTEIL,
                Kosten.IMMUNITAET_GIFT,
                null,
                Varianten.GIFTE,
                null,
                Voraussetzung.IMMUNITAET_GIFT,
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFTE ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_GIFTE ) );

        initialisieren(
                IMMUNITAET_GIFTART,
                Typ.VORTEIL,
                Kosten.IMMUNITAET_GIFTART,
                null,
                Varianten.GIFTARTEN,
                null,
                Voraussetzung.IMMUNITAET_GIFTART,
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFTE ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_GIFTE ) );

        initialisieren(
                IMMUNITAET_GIFTE,
                Typ.VORTEIL,
                Kosten.GP_35,
                null, null, null,
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFTE ),
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFTART ),
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFT ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_GIFTART ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_GIFT ) )
                .setSchwaechereAuspraegung( RESISTENZ_GIFTE );

        initialisieren(
                IMMUNITAET_KRANKHEIT,
                Typ.VORTEIL,
                Kosten.GP_3,
                null,
                Varianten.KRANKHEITEN,
                null,
                Voraussetzung.nichtKombinierbar( IMMUNITAET_KRANKHEITEN ) );

        initialisieren(
                IMMUNITAET_KRANKHEITEN,
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( RESISTENZ_KRANKHEITEN ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_KRANKHEIT ) )
                .setSchwaechereAuspraegung( RESISTENZ_KRANKHEITEN );

        initialisieren(
                INNERER_KOMPASS,
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null, null,
                Voraussetzung.nichtKombinierbar( RICHTUNGSSINN ) )
                .setSchwaechereAuspraegung( RICHTUNGSSINN );

        initialisieren(
                "VorNachteil.Kaelteresistenz",
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null );

        initialisieren(
                KAMPFRAUSCH,
                Typ.VORTEIL,
                Kosten.GP_15,
                null, null, null,
                Voraussetzung.nichtKombinierbar( BLUTRAUSCH ) );

        initialisieren(
                "VorNachteil.Koboldfreund",
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null );

        initialisieren(
                MAGIEGESPUER,
                Typ.VORTEIL,
                Kosten.GP_12,
                null, null,
                Anpasser.MAGIEGESPUER );

        initialisieren(
                "VorNachteil.Meisterhandwerk",
                Typ.VORTEIL,
                Kosten.MEISTERHANDWERK,
                null,
                Varianten.MEISTERHANDWERK,
                null,
                Voraussetzung.ZV,
                Voraussetzung.TALENT,
                Voraussetzung.MEISTERHANDWERK );

        initialisieren(
                NACHTSICHT,
                Typ.VORTEIL,
                Kosten.GP_20,
                null, null, null,
                Voraussetzung.nichtKombinierbar( DAEMMERUNGSSICHT ),
                Voraussetzung.nichtKombinierbar( NACHTBLIND ),
                Voraussetzung.NACHTSICHT )
                .setSchwaechereAuspraegung( DAEMMERUNGSSICHT );

        initialisieren(
                PROPHEZEIEN,
                Typ.VORTEIL,
                Kosten.GP_12,
                null, null,
                Anpasser.PROPHEZEIEN );

        initialisieren(
                RESISTENZ_GIFT,
                Typ.VORTEIL,
                Kosten.GP_3,
                null,
                Varianten.GIFTE,
                null,
                Voraussetzung.RESISTENZ_GIFT,
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFTE ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_GIFTE ) );

        initialisieren(
                RESISTENZ_GIFTART,
                Typ.VORTEIL,
                Kosten.GP_7,
                null,
                Varianten.GIFTARTEN,
                null,
                Voraussetzung.RESISTENZ_GIFTART,
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFTE ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_GIFTE ) );

        initialisieren(
                RESISTENZ_GIFTE,
                Typ.VORTEIL,
                Kosten.GP_25,
                null, null, null,
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFTART ),
                Voraussetzung.nichtKombinierbar( RESISTENZ_GIFT ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_GIFTE ) )
                .setStaerkereAuspraegung( IMMUNITAET_GIFTE );

        initialisieren(
                RESISTENZ_KRANKHEITEN,
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null, null,
                Voraussetzung.nichtKombinierbar( IMMUNITAET_KRANKHEIT ) )
                .setStaerkereAuspraegung( IMMUNITAET_KRANKHEITEN );

        initialisieren(
                RICHTUNGSSINN,
                Typ.VORTEIL,
                Kosten.GP_3,
                null, null, null,
                Voraussetzung.nichtKombinierbar( INNERER_KOMPASS ) )
                .setStaerkereAuspraegung( INNERER_KOMPASS );

        initialisieren(
                SCHLANGENMENSCH,
                Typ.VORTEIL,
                Kosten.GP_25,
                null, null,
                Anpasser.GE_14,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_KK ),
                Voraussetzung.GE_14 );

        initialisieren(
                SCHNELLE_HEILUNG,
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( SCHLECHTE_REGENERATION ) );

        initialisieren(
                "VorNachteil.Schutzgeist",
                Typ.VORTEIL,
                Kosten.GP_3,
                null, null, null,
                Voraussetzung.V );

        initialisieren(
                "VorNachteil.SchwerZuVerzaubern",
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null, null,
                Voraussetzung.nichtKombinierbar( VOLLZAUBERER ),
                Voraussetzung.nichtKombinierbar( VIERTELZAUBERER ) );

        initialisieren(
                SOZIALE_ANPASSUNGSFAEHIGKEIT,
                Typ.VORTEIL,
                Kosten.GP_7,
                null, null, null );

        initialisieren(
                "VorNachteil.UebernatuerlicheBegabung",
                Typ.VORTEIL,
                Kosten.GP_1,
                null,
                Varianten.UEBERNATUERLICHE_BEGABUNG,
                Anpasser.UEBERNATUERLICHE_BEGABUNG,
                Voraussetzung.UEBERNATUERLICHE_BEGABUNG,
                Voraussetzung.V );

        initialisieren(
                VERBINDUNGEN,
                Typ.VORTEIL,
                Kosten.VERBINDUNGEN,
                Stufen.VERBINDUNGEN,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                WOHLKLANG,
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( UNANGENEHME_STIMME ) );

        initialisieren(
                "VorNachteil.ZaeherHund",
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null );

        initialisieren(
                "VorNachteil.Zeitgefuehl",
                Typ.VORTEIL,
                Kosten.GP_3,
                null, null, null );

        initialisieren(
                ZWEISTIMMIGER_GESANG,
                Typ.VORTEIL,
                Kosten.GP_10,
                null, null, null,
                Voraussetzung.NICHT_FREI_WAEHLBAR );

        initialisieren(
                ZWERGENNASE,
                Typ.VORTEIL,
                Kosten.GP_12,
                null, null,
                Anpasser.ZWERGENNASE );

        initialisieren(
                VIERTELZAUBERER,
                Typ.VORTEIL,
                Kosten.GP_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar(AKADEMISCHE_AUSBILDUNG_KRIEGER),
                Voraussetzung.nichtKombinierbar( VOLLZAUBERER ),
                Voraussetzung.nichtKombinierbar( VIERTELZAUBERER_UNBEWUSST ),
                Voraussetzung.NICHT_KAEMPFER );

        initialisieren(
                VIERTELZAUBERER_UNBEWUSST,
                Typ.VORTEIL,
                Kosten.GP_3,
                null, null, null,
                Voraussetzung.nichtKombinierbar(AKADEMISCHE_AUSBILDUNG_KRIEGER),
                Voraussetzung.nichtKombinierbar( VOLLZAUBERER ),
                Voraussetzung.nichtKombinierbar( VIERTELZAUBERER ),
                Voraussetzung.NICHT_KAEMPFER );

        initialisieren(
                VOLLZAUBERER,
                Typ.VORTEIL,
                Kosten.GP_20,
                null, null, null,
                Voraussetzung.NICHT_FREI_WAEHLBAR );

        // ********* //
        //           //
        // Nachteile //
        //           //
        // ********* //
        initialisieren(
                "VorNachteil.Aberglaube",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Albino",
                Typ.NACHTEIL,
                Kosten.ALBINO,
                null, null,
                Anpasser.ALBINO,
                Voraussetzung.ALBINO );

        initialisieren(
                "VorNachteil.AngstVor1",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                new VariantenFrei(),
                null );

        initialisieren(
                "VorNachteil.AngstVor2",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                new VariantenFrei(),
                null );

        initialisieren(
                "VorNachteil.Arkanophobie",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.ARKANOPHOBIE,
                Voraussetzung.NICHT_ELFEN,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Arroganz",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN  );

        initialisieren(
                ASTRALER_BLOCK,
                Typ.NACHTEIL,
                Kosten.ASTRALER_BLOCK,
                null, null, null,
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_I ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_II ),
                Voraussetzung.nichtKombinierbar( ASTRALE_REGENERATION_III ),
                Voraussetzung.ZV );

        initialisieren(
                "VorNachteil.Autoritaetsglaeubig",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN  );

        initialisieren(
                BLUTRAUSCH,
                Typ.NACHTEIL,
                Kosten.MINUS_15,
                null, null, null,
                Voraussetzung.nichtKombinierbar( KAMPFRAUSCH ) );

        initialisieren(
                "VorNachteil.Dunkelangst",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_2_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN  );

        initialisieren(
                EINAEUGIG,
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null );

        initialisieren(
                EINARMIG,
                Typ.NACHTEIL,
                Kosten.MINUS_15,
                null, null, null,
                Voraussetzung.nichtKombinierbar( GELAEHMTER_ARM ),
                Voraussetzung.nichtKombinierbar( EINHAENDIG ),
                Voraussetzung.NICHT_LINKHAND,
                Voraussetzung.NICHT_SCHILDKAMPF_I,
                Voraussetzung.NICHT_SCHILDKAMPF_II,
                Voraussetzung.NICHT_ZWEIHAND_HIEBWAFFEN,
                Voraussetzung.NICHT_ZWEIHANDSCHWERTER );

        initialisieren(
                EINBEINIG,
                Typ.NACHTEIL,
                Kosten.MINUS_25,
                null, null, null );

        initialisieren(
                EINGESCHRAENKTER_GERUCHSSINN,
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDER_GERUCHSSINN ) );

        initialisieren(
                EINGESCHRAENKTER_TASTSINN,
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDER_TASTSINN ) );

        initialisieren(
                EINHAENDIG,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( EINARMIG ),
                Voraussetzung.nichtKombinierbar( GELAEHMTER_ARM ) );

        initialisieren(
                "VorNachteil.Eitelkeit",
                Typ.NACHTEIL,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                ELFISCHE_WELTSICHT,
                Typ.NACHTEIL,
                Kosten.MINUS_30,
                null, null, null,
                Voraussetzung.NICHT_FREI_WAEHLBAR );

        initialisieren(
                "VorNachteil.Farbenblind",
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null );

        initialisieren(
                FETTLEIBIG,
                Typ.NACHTEIL,
                Kosten.MINUS_15,
                null, null,
                Anpasser.FETTLEIBIG );

        initialisieren(
                "VorNachteil.Geiz",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Gerechtigkeitswahn",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.GesuchtI",
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null );

        initialisieren(
                "VorNachteil.GesuchtII",
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null );

        initialisieren(
                "VorNachteil.GesuchtIII",
                Typ.NACHTEIL,
                Kosten.MINUS_15,
                null, null, null );

        initialisieren(
                GELAEHMTER_ARM,
                Typ.NACHTEIL,
                Kosten.MINUS_7,
                null, null, null,
                Voraussetzung.nichtKombinierbar( EINARMIG ),
                Voraussetzung.nichtKombinierbar( EINHAENDIG ),
                Voraussetzung.NICHT_LINKHAND,
                Voraussetzung.NICHT_SCHILDKAMPF_I,
                Voraussetzung.NICHT_SCHILDKAMPF_II,
                Voraussetzung.NICHT_ZWEIHAND_HIEBWAFFEN,
                Voraussetzung.NICHT_ZWEIHANDSCHWERTER );

        initialisieren(
                GLASKNOCHEN,
                Typ.NACHTEIL,
                Kosten.MINUS_7,
                null, null, null );

        initialisieren(
                "VorNachteil.Goldgier",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Hoehenangst",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Jaehzorn",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.KrankhafteReinlichkeit",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                KRANKHEITSANFAELLIG,
                Typ.NACHTEIL,
                Kosten.MINUS_7,
                null, null, null,
                Voraussetzung.nichtKombinierbar( RESISTENZ_KRANKHEITEN ),
                Voraussetzung.nichtKombinierbar( IMMUNITAET_KRANKHEITEN ) );

        initialisieren(
                KURZATMIG,
                Typ.NACHTEIL,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_1_BIS_3,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                KURZSICHTIG,
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_SICHT ) );

        initialisieren(
                LAHM,
                Typ.NACHTEIL,
                Kosten.MINUS_15,
                null, null, null );

        initialisieren(
                "VorNachteil.Lichtempfindlich",
                Typ.NACHTEIL,
                Kosten.MINUS_15,
                null, null, null );

        initialisieren(
                "VorNachteil.Lichtscheu",
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null );

        initialisieren(
                "VorNachteil.Meeresangst",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                MISERABLE_FF,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLE_FF,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_FF ) );

        initialisieren(
                MISERABLE_GE,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLE_GE,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_GE ) );

        initialisieren(
                MISERABLE_IN,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLE_IN,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_IN ) );

        initialisieren(
                MISERABLE_KL,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLE_KL,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_KL ) );

        initialisieren(
                MISERABLE_KO,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLE_KO,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_KO ) );

        initialisieren(
                MISERABLE_KK,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLE_KK,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDE_KK ) );

        initialisieren(
                MISERABLER_MU,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLER_MU,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDER_MU ) );

        initialisieren(
                MISERABLES_CH,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null,
                Anpasser.MISERABLES_CH,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDES_CH ) );

        initialisieren(
                NACHTBLIND,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( DAEMMERUNGSSICHT ),
                Voraussetzung.nichtKombinierbar( NACHTSICHT ) );

        initialisieren(
                "VorNachteil.Nahrungsrestriktion",
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null );

        initialisieren(
                "VorNachteil.Neid",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Neugier",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                NIEDRIGE_ASTRALKRAFT,
                Typ.NACHTEIL,
                Kosten.MINUS_1_PRO_1,
                Stufen.STUFE_1_BIS_6,
                null, null,
                Voraussetzung.STUFEN,
                Voraussetzung.nichtKombinierbar( ASTRALMACHT ),
                Voraussetzung.ZV );

        initialisieren(
                NIEDRIGE_LEBENSKRAFT,
                Typ.NACHTEIL,
                Kosten.MINUS_2_PRO_1,
                Stufen.STUFE_1_BIS_6,
                null, null,
                Voraussetzung.STUFEN,
                Voraussetzung.nichtKombinierbar( HOHE_LEBENSKRAFT ) );

        initialisieren(
                NIEDRIGE_MAGIERESISTENZ,
                Typ.NACHTEIL,
                Kosten.MINUS_2_PRO_1,
                Stufen.STUFE_1_BIS_3,
                null, null,
                Voraussetzung.nichtKombinierbar( HOHE_MAGIERESISTENZ ),
                Voraussetzung.NIEDRIGE_MAGIERESISTENZ,
                Voraussetzung.STUFEN  );

        initialisieren(
                "VorNachteil.Platzangst",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Prinzipientreue",
                Typ.NACHTEIL,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_3_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Rachsucht",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Raumangst",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.SchlechteEigenschaft1",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.SchlechteEigenschaft2",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.SchlechteEigenschaft3",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.SchlechteEigenschaft4",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_2_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                SCHLECHTE_REGENERATION,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( SCHNELLE_HEILUNG ) );

        initialisieren(
                SCHWERHOERIG,
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDES_GEHOER ) );

        initialisieren(
                "VorNachteil.SensiblerGeruchssinn",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_3_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                null, null,
                Voraussetzung.STUFEN,
                Voraussetzung.NUR_HALBELFEN );

        initialisieren(
                "VorNachteil.Speisegebote",
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null );

        initialisieren(
                "VorNachteil.Spruchhemmung",
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null,
                Voraussetzung.Z );

        initialisieren(
                "VorNachteil.Tollpatsch",
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null );

        initialisieren(
                TOTENANGST,
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                null, null,
                Voraussetzung.STUFEN,
                Voraussetzung.NICHT_ANATOMIE );

        initialisieren(
                UNANGENEHME_STIMME,
                Typ.NACHTEIL,
                Kosten.MINUS_5,
                null, null, null,
                Voraussetzung.nichtKombinierbar( WOHLKLANG ) );

        initialisieren(
                UNANSEHNLICH,
                Typ.NACHTEIL,
                Kosten.MINUS_10,
                null, null, null,
                Voraussetzung.nichtKombinierbar( WIDERWAERTIGES_AUSSEHEN ),
                Voraussetzung.nichtKombinierbar( GUT_AUSSEHEND ),
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDES_AUSSEHEN ) )
                .setStaerkereAuspraegung( WIDERWAERTIGES_AUSSEHEN );

        initialisieren(
                UNFAEHIGKEIT_TALENT,
                Typ.NACHTEIL,
                Kosten.UNFAEHIGKEIT_TALENT,
                null,
                Varianten.BASIS_TALENTE,
                null,
                Voraussetzung.NICHT_FREI_WAEHLBAR );

        initialisieren(
                UNFAEHIGKEIT_TALENTGRUPPE,
                Typ.NACHTEIL,
                Kosten.UNFAEHIGKEIT_TALENTGRUPPE,
                null,
                Varianten.TALENTGRUPPE,
                null,
                Voraussetzung.UNFAEHIGKEIT_TALENTGRUPPE );

        initialisieren(
                "VorNachteil.Verpflichtungen",
                Typ.NACHTEIL,
                Kosten.MINUS_12,
                null,
                new VariantenFrei(),
                null );

        initialisieren(
                VORURTEILE_1,
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_2_SE,
                Stufen.STUFE_6_BIS_12,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                VORURTEILE_2,
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                "VorNachteil.Weltfremd",
                Typ.SCHLECHTE_EIGENSCHAFT,
                Kosten.MINUS_1_PRO_1_SE,
                Stufen.STUFE_5_BIS_12,
                new VariantenFrei(),
                null,
                Voraussetzung.STUFEN );

        initialisieren(
                WIDERWAERTIGES_AUSSEHEN,
                Typ.NACHTEIL,
                Kosten.MINUS_15,
                null, null, null,
                Voraussetzung.nichtKombinierbar( UNANSEHNLICH ),
                Voraussetzung.nichtKombinierbar( GUT_AUSSEHEND ),
                Voraussetzung.nichtKombinierbar( HERAUSRAGENDES_AUSSEHEN ) )
                .setSchwaechereAuspraegung( UNANSEHNLICH );

        initialisieren(
                ZWERGENWUCHS,
                Typ.NACHTEIL,
                Kosten.MINUS_7,
                null, null, null,
                Voraussetzung.NICHT_FREI_WAEHLBAR );

    }

    /**
     * Erzeugt und initialisiert einen Vor- oder Nachteil. Ruft die Methode
     * {@link #initialisieren(VorNachteil, Typ, Kosten, Stufen, Varianten,
     * Anpasser, Voraussetzung[])} auf.
     *
     * @param id
     *            des neu zu erschaffenden Vor-/Nachteils
     * @param typ
     *            {@link Typ#VORTEIL} oder {@link Typ#NACHTEIL}.
     * @param kosten
     *            des Vor-/Nachteils.
     * @param stufen
     *            kann {@code null} sein, wenn der Vor-/Nachteil keine Stufen
     *            hat.
     * @param varianten
     *            kann {@code null} sein, wenn der Vor-/Nachteil keine Varianten
     *            hat.
     * @param anpasser
     *            kann {@code null} sein, wenn der Vor-/Nachteil keinen
     *            Anpasser benoetigt.
     * @param voraussetzungen
     *            fuer den Vor-/Nachteil. Hier muss nichts uebergeben werden.
     * @return den erzeugten Vor-/Nachteil.
     */
    private static final VorNachteil initialisieren(
            final String id,
            final Typ typ,
            final Kosten kosten,
            final Stufen stufen,
            final Varianten varianten,
            final Anpasser anpasser,
            final Voraussetzung ... voraussetzungen ) {

        final VorNachteil vorNachteil = new VorNachteil( id );

        return initialisieren(
                vorNachteil,
                typ,
                kosten,
                stufen,
                varianten,
                anpasser,
                voraussetzungen );
    }

    /**
     * Initialisiert einen Vor- oder Nachteil.
     * Jeder Vor-/Nachteil muss ueber diese Methode initialisiert werden.
     *
     * @param vorNachteil
     *            der initialisiert werden soll.
     * @param typ
     *            {@link Typ#VORTEIL} oder {@link Typ#NACHTEIL}.
     * @param kosten
     *            des Vor-/Nachteils.
     * @param stufen
     *            kann {@code null} sein, wenn der Vor-/Nachteil keine Stufen
     *            hat.
     * @param varianten
     *            kann {@code null} sein, wenn der Vor-/Nachteil keine Varianten
     *            hat.
     * @param anpasser
     *            kann {@code null} sein, wenn der Vor-/Nachteil keinen
     *            Anpasser benoetigt.
     * @param voraussetzungen
     *            fuer den Vor-/Nachteil. Hier muss nichts uebergeben werden.
     * @return den initialisierten Vor-/Nachteil.
     */
    private static final VorNachteil initialisieren(
            final VorNachteil vorNachteil,
            final Typ typ,
            final Kosten kosten,
            final Stufen stufen,
            final Varianten varianten,
            final Anpasser anpasser,
            final Voraussetzung ... voraussetzungen ) {

        vorNachteil.setTyp( typ );

        vorNachteil.setKosten( kosten );

        if ( null == stufen ) {

            vorNachteil.setStufen( Stufen.KEINE );

        } else {

            vorNachteil.setStufen( stufen );

        }

        if ( null == varianten ) {

            vorNachteil.setVarianten( Varianten.KEINE );

        } else {

            vorNachteil.setVarianten( varianten );

        }

        if ( null != anpasser ) {

            vorNachteil.einfuegen( anpasser );

        }

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            vorNachteil.einfuegen( voraussetzung );
        }
        vorNachteil.einfuegen( Voraussetzung.STANDARD );

        einfuegen( vorNachteil );

        return vorNachteil;
    }

    /**
     * Abhaengigkeiten dieses Regelelements von anderen Regelelementen.
     * Zu diesen Abhaengigkeiten kommen noch weitere hinzu, die alle in der
     * Methode {@link #getAbhaengigkeiten()} aufgeloest werden.
     */
    private final Collection< Object > abhaengigkeiten;

    /**
     * Passt die Werte eines Helden an, nachdem er den Vor-/Nachteil
     * (ab)gewaehlt hat zB muss der Held das einsprechende Talent erhalten,
     * wenn er eine Gabe waehlt.
     */
    private final List< Anpasser > anpasser;

    /** Berechnet die Kosten des Vor-/Nachteils. */
    private Kosten kosten;

    /**
     * Wenn der Vor-/Nachteil Varianten besitzt, so setzt sich der Name eines
     * {@link VorNachteilWert}s aus diesem Praefix und der Variante zusammen.
     * <dl>
     * <dt>Beispiel:</dt>
     * <dd>
     *      Der Name eines Wertes des Vorteils 'Begabung fuer Talent' setzt
     *      sich zusammen aus dem Praefix 'Begabung fuer' und dem Name des
     *      Talent, also z.B. 'Begabung fuer Kochen'.
     * </dd>
     * </dl>
     * Das Praefix wird in der Benutzeroberlaeche gebraucht, wo das Praefix
     * angezeigt wird und dahinter die Combobox um die Variante zu waehlen.
     */
    private final String lokalisierungsschluesselPraefix;

    /**
     * Einige Vor-/Nachteile kommen in Abstufungen vor, z.B. Balance und
     * Herausragenden Balance. Balance ist hierbei die schwaechere Auspraegung
     * von Herausragende Balance.
     */
    private VorNachteil schwaechereAuspraegung;

    /**
     * Einige Vor-/Nachteile kommen in Abstufungen vor, z.B. Gut aussehend und
     * Herausragendes Aussehen. Herausragendes Aussehen ist die staerkere
     * Auspraegung von Gut aussehend.
     */
    private VorNachteil staerkereAuspraegung;

    /**
     * Legt fest ob der Vor-/Nachteil Stufen besitzt und was Mindeststufe,
     * Hoechststufe und Schrittweite der Stufe sind.
     */
    private Stufen stufen;

    /**
     * Legt fest ob es sich um einen Vorteil, einen Nachteil oder eine
     * Schlechte Eigenschaft handelt.
     */
    private Typ typ;

    /** Eine Liste der moegliche Varianten des Vor-/Nachteils. */
    private Varianten varianten;

    /** Die Voraussetzungen des Vor-/Nachteils. */
    private final List< Voraussetzung > voraussetzungen;

    private VorNachteil( final String dieId ) {

        super ( dieId );
        
        // Jetzt wird erst mal das Praefix ermittelt
        lokalisierungsschluesselPraefix = dieId + ".Praefix";

        abhaengigkeiten = new HashSet< Object >();

        anpasser = new ArrayList< Anpasser >( 1 );

        voraussetzungen = new ArrayList< Voraussetzung >();
    }

    /**
     * Fuegt dem Vor-/Nachteil einen Anpasser hinzu.
     * @param derAnpasser ein weitere Anpasser fuer den Vor-/Nachteil.
     */
    private final void einfuegen( final Anpasser derAnpasser ) {
        anpasser.add( derAnpasser );
    }

    /**
     * Fuegt dem Vor-/Nachteil eine weitere Voraussetzung hinzu.
     *
     * @param voraussetzung fuer den Vor-/Nachteil.
     */
    private final void einfuegen( final Voraussetzung voraussetzung ) {
        voraussetzungen.add( voraussetzung );
    }

    /**
     * Diese Methode muss aufgerufen werden, nachdem ein Held den Vor-/Nachteil
     * erhalten hat, damit ggf. Anpassungen an den Werten des Helden gemacht
     * werden koennen.
     *
     * @see #entfernt(Held)
     * @param held der den Vor-/Nachteil erhalten hat.
     */
    public final void eingefuegt(
            final Held held,
            final VorNachteilWert wert ) {

        for ( final Anpasser a : anpasser ) {

            a.eingefuegt( held, wert );
        }
    }

    /**
     * Diese Methode muss aufgerufen werden nachem ein Held den Vor-/Nachteil
     * verloren hat, damit ggf. Anpassungen an den Werten des Helden gemacht
     * werden koennen.
     *
     * @see #eingefuegt(Held,VorNachteilWert)
     * @param held der den Vor-/Nachteil verloren hat.
     * @param wert der vom Held entfernt wurde.
     */
    public final void entfernt( final Held held, final VorNachteilWert wert ) {

        for( final Anpasser a :anpasser ) {

            a.entfernt( held, wert );
        }
    }

    /**
     * Macht den Vor-/Nachteil von einem weiteren Regelelement (Eigenschaft,
     * Vor-/Nachteil, ...) abhaengig.
     */
    public final void ergaenzeAbhaengigkeit( final Object regelelement ) {
        abhaengigkeiten.add( regelelement );
    }


    /**
     * Wird aufgerufen wenn die Variante einen Wertes geaendert wurde.
     *
     * @param held dem der geaenderte Wert gehoert.
     * @param alteVariante Die bisherige Variante des Wertes.
     * @param neueVariante Die neue Variante des Wertes.
     */
    void geaendertVariante(
            final Held held,
            final Object alteVariante,
            final Object neueVariante ) {

        for ( final Anpasser a : anpasser ) {

            a.geaendertVariante( held, alteVariante, neueVariante );
        }
    }

    /**
     * Liefert, bei einem Nachteil, die AP, die zu bezahlen sind um ihn
     * abzubauen.
     *
     * @param held
     *            der den Nachteil abbauen will.
     * @param wert
     *            der abgebaut werden soll.
     * @return die AP die zu bezahlen sind, um den Nachteil abzubauen.
     */
    public final int getAbbaukosten(
            final Held held,
            final VorNachteilWert wert ) {
        return kosten.getAbbaukosten( held, wert );
    }

    /**
     * @return alle Regelelemente (Eigenschaften, Vor-/Nachteile usw.) die
     *         irgendwie im Zusammenhang mit diesem Vor-/Nachteil stehen und
     *         fuer die Berechnung seiner Werte wichtig sind.
     */
    public final Collection< Object > getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = new HashSet< Object >();

        abhaengigkeiten.add( this );

        abhaengigkeiten.addAll( this.abhaengigkeiten );

        VorNachteil vorNachteil = this;

        while( vorNachteil.hatSchwaechereAuspraegung() ) {

            vorNachteil = vorNachteil.getSchwaechereAuspraegung();

            abhaengigkeiten.add( vorNachteil );
        }

        vorNachteil = this;

        while( vorNachteil.hatStaerkereAuspraegung() ) {

            vorNachteil = vorNachteil.getStaerkereAuspraegung();

            abhaengigkeiten.add( vorNachteil );
        }

        abhaengigkeiten.addAll( kosten.getAbhaengigkeiten() );

        abhaengigkeiten.addAll( stufen.getAbhaengigkeiten() );

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            abhaengigkeiten.addAll( voraussetzung.getAbhaengigkeiten() );
        }

        return abhaengigkeiten;
    }

    /**
     * Liefert die Kosten des Vor-/Nachteils. Bei einigen Vor-/Nachteilen hängen
     * die Kosten von der gewaehlten Variante des Vor-/Nachteils ab, dafuer
     * existiert die Methode {@link #getGpKosten(VorNachteilWert)}. Die Methode
     * {@link #getGpKosten(Held, VorNachteilWert)} liefert die Kosten die ein
     * bestimmter Held fuer einen Vor-/Nachteil zahlen muss.
     * <p>
     * Sind die Kosten ueber diese Methode nicht genau zu ermitteln wird ein
     * beliebiger Wert geliefert.
     *
     * @see #getGpKosten(VorNachteilWert)
     * @see #getGpKosten(Held, VorNachteilWert)
     * @return die GP-Kosten, die bei der Erschaffung fuer den Vor-/Nachteil zu
     *         zahlen sind.
     */
    public final int getGpKosten() {
        return kosten.getGpKosten();
    }

    /**
     * Liefert die GP-Kosten die ein Held fuer einen Wert zahlen muss.
     *
     * @param held der die GP-Kosten zahlen soll.
     * @param wert fuer den die Kosten gesucht sind.
     * @return die Kosten, die {@code held} fuer {@code wert} zahlen muss.
     */
    public final int getGpKosten(
            final Held held,
            final VorNachteilWert wert ) {

        return kosten.getGpKosten( held, wert );
    }

    /**
     * Liefert die GP-Kosten fuer diesen Vor-/Nachteil. Bei einigen
     * Vor-/Nachteilen sind die Kosten abhaengig vom Helden. Fuer diese Faelle
     * gibt es die Methode {@link #getGpKosten(Held, VorNachteilWert)}. Lassen
     * sich die Kosten ueber diese Methode hier nicht genau bestimmten, so wird
     * ein beliebiger Wert zurueck gegeben.
     *
     * @see #getGpKosten(Held, VorNachteilWert)
     * @param wert
     *            dessen Kosten gesucht werden.
     * @return die GP-Kosten des Vor-/Nachteils.
     */
    public final int getGpKosten( final VorNachteilWert wert ) {
        return kosten.getGpKosten( wert );
    }

    /**
     * Liefert den groessten noch erlaubten Wert fuer die Stufe des
     * Vor-/Nachteils.
     *
     * @see #hatStufen()
     * @param held fuer den die Hoechststufe sein soll.
     * @return den groessten noch erlaubten Wert. Wenn der Vor-/Nachteil keine
     *      Stufen hat, so wird ein beliebiger Wert zurueck geliefert.
     */
    public final int getHoechststufe( final Held held ) {
        return stufen.getHoechststufe( held );
    }

    /**
     * @return Den Lokalisierungsschluessel unter dem man das Praefix des 
     *         Vor-/Nachteils in der Lokalisierungsdatei findet.
     */
    public final String getL10nschluesselPraefix() {

        if ( I18n.hatLokalisierungstext( lokalisierungsschluesselPraefix ) ) {

            return lokalisierungsschluesselPraefix;

        } else {

            return getLokalisierungsschluesselName();
        }   
    }
    
    /**
     * Liefert den kleinsten noch erlaubten Wert fuer die Stufe des
     * Vor-/Nachteils. Dieser Wert kann vom Helden abhaengen, daher gibt es
     * noch die Methode {@link #getMindeststufe(Held)}.
     *
     * @see #hatStufen()
     * @see #getMindeststufe(Held)
     * @return den kleinsten noch erlaubten Wert. Wenn der Vor-/Nachteil keine
     *      Stufen hat, so wird ein beliebiger Wert zurueck geliefert.
     */
    public final int getMindeststufe() {
        return stufen.getMindeststufe();
    }

    /**
     * Liefert den kleinsten noch erlaubten Wert fuer die Stufe des
     * Vor-/Nachteils.
     *
     * @see #hatStufen()
     * @param held fuer den die Mindeststufe sein soll.
     * @return den kleinsten noch erlaubten Wert. Wenn der Vor-/Nachteil keine
     *      Stufen hat, so wird ein beliebiger Wert zurueck geliefert.
     */
    public final int getMindeststufe( final Held held ) {
        return stufen.getMindeststufe( held );
    }

    /**
     * Liefert einen String der dazu dient den Namen eines
     * {@link VorNachteilWert}s zu bestimmen der eine Variante besitzt.
     * Der String kann mit {@link String#format(String, Object[])} verarbeitet
     * werden um die gewaehlte Variante in den Namen einzubauen.
     * <dl>
     * <dt>Beispiele:</dt>
     * <dd>
     * <ul>
     * <li>Beim Vorteil Begabung fuer Talent sieht der String so aus:
     * 'Begabung fuer %1$s'.
     * <li>Bei Prinzipientreue so: 'Prinzipientreue (%1$s)'.
     * </ul>
     * </dd>
     * </dl>
     *
     * @return einen String um den Namen eines Wertes zu bestimmen.
     */
    public final String getNameVariante() {

        if ( I18n.hatLokalisierungstext( lokalisierungsschluesselPraefix ) ) {

            // Bei Vor-/Nachteilen mit speziellem Praefix ist es in der Regel
            // so dass der Name eines Wertes gebildet wird, indem man die
            // Variante, ohne Klammern, anhaengt.
            return getPraefix() + " %1$s";

        } else {

            // Falls dieser Vor-/Nachteil Varianten hat, wird die der Name
            // eines Wertes gebildet, indem man die Variante in Klammern
            // anhaengt.
            return getPraefix() + " (%1$s)";
        }
    }

    /**
     * @return Bei einem Vor-/Nachteil mit Variante liefert die Methode das
     *         Paefix des Namens, das im Namen vor der Variante steht. Beispiel:
     *         Bei 'Begabung fuer Koche' ist das Praefix 'Begabung fuer', bei
     *         'Vorurteile gegen Maenner' ist das Praefix 'Vorurteile gegen'.
     */
    public final String getPraefix() {

        if ( I18n.hatLokalisierungstext( lokalisierungsschluesselPraefix ) ) {

            // Es gibt ein spezielles Praefix
            return I18n.i18n( lokalisierungsschluesselPraefix );

        } else {

            // Es gibt kein spezielles Praefix.
            // In diesen Faellen wird einfach der Name uebernommen.
            return getName();
        }
    }

    /**
     * @return die Schrittweite fuer die Stufen des Vor-/Nachteils. Wenn der
     *      Vor-/Nachteil keine Stufen besitzt, so wird ein beliebiger Wert
     *      geliefert.
     */
    public final int getSchrittweite() {
        return stufen.getSchrittweite();
    }

    /**
     * @see #hatSchwaechereAuspraegung()
     * @return die schwaechere Auspraegung des Vor-/Nachteils, so er eine
     *         besitzt, sonst {@code null}.
     */
    public final VorNachteil getSchwaechereAuspraegung() {
        return schwaechereAuspraegung;
    }

    /**
     * @return die AP, die bei einer Schlechten Eigenschaft zu zahlen sind, um
     *         den Wert um einen Punkt zu senken.
     */
    public final int getSenkungskosten( final Lernmethode lernmethode ) {
        return kosten.getSenkungskosten( lernmethode );
    }

    /**
     * @see #hatStaerkereAuspraegung()
     * @return die staerkere Auspraegung des Vor-/Nachteils, so er eine besitzt,
     *         sonst {@code null}.
     */
    public final VorNachteil getStaerkereAuspraegung() {
        return staerkereAuspraegung;
    }

    /**
     * @param objekt Ein String oder ein Objekt vom Typ Variante.
     * @return Liefert die passende Variante zum objekt. Wenn das Objekt vom Typ
     *      Variante ist, wird es einfach zur Variante gecastet und zurueck
     *      geliefert. Wenn es ein String ist, der vom Benutzer eingegeben
     *      wurde, wird geprueft ob es eine passende Variante mit dem String
     *      als Namen gibt. Sonst wird eine passende Variante erzeugt.
     */
    public final Variante getVariante( final Object objekt ) {
       
        return varianten.getVariante( objekt );
    }
    
    /**
     * @param held fuer den die Varianten sind.
     * @see #hatVarianten()
     * @return eine Liste der moeglichen Varianten des Vor-/Nachteils. Die
     *      Liste ist leer wenn alle Varianten vom Benutzer eingegeben werden
     *      muessen, kann sich aber durch die Eingaben fuellen.
     *      Die Liste ist auch leer wenn der Vor-/Nachteil nicht in Varianten
     *      vorkommt.
     */
    public final List< Variante > getVarianten(final Held held) {
        return varianten.getVarianten(held);
    }

    /**
     * @return {@code true} wenn der Benutzer weiter Varianten fuer den
     *      Vor-/Nachteil eingeben darf, sonst {@code false}.
     */
    public final boolean hatFreieVarianten() {
        return varianten.istFreieVariante();
    }

    /**
     * @return {@code true}, wenn der Vor-/Nachteil eine schwaechere
     *         Auspraegung besitzt, sonst {@code false}.
     */
    public final boolean hatSchwaechereAuspraegung() {
        return ( null != schwaechereAuspraegung );
    }

    /**
     * @return {@code true}, wenn der Vor-/Nachteil eine staerkere Auspraegung
     *         besitzt, sonst {@code false}.
     */
    public final boolean hatStaerkereAuspraegung() {
        return ( null != staerkereAuspraegung );
    }

    /**
     * @return {@code true} wenn der Vor-/Nachteil Stufen hat, sonst
     *      {@code false}.
     */
    public final boolean hatStufen() {
        return stufen.hatStufen();
    }

    /**
     * @return {@code true} wenn der Vor-/Nachteil in Varianten vorkommt, sonst
     *      {@code false}.
     */
    public final boolean hatVarianten() {
        return varianten.hatVarianten();
    }

    /**
     * @return {@code true}, wenn es sich um eine Schlechte Eigenschaft handelt,
     *         sonst {@code false}.
     */
    public final boolean istSchlechteEigenschaft() {
        return Typ.SCHLECHTE_EIGENSCHAFT == typ;
    }

    /**
     * @return {@code true}, wenn die Stufe des Nachteils durch Investition von
     *         AP gesenkt werden kann, sonst {@code false}.
     */
    public boolean istSenkbar() {
        return kosten.istSenkbar();
    }

    /**
     * @return {@code true}, wenn es sich um einen Vorteil handelt, sonst
     *         {@code false}.
     */
    public final boolean istVorteil() {
        return Typ.VORTEIL == typ;
    }

    /**
     * Prueft ob ein Held alle Voraussetzungen fuer einen Wert erfuellt.
     *
     * @param held der die Voraussetzungen erfuellen soll.
     * @param wert dessen Voraussetzungen zu erfuellen sind.
     * @return eine Beschreibung der nicht erfuellten Voraussetzungen.
     */
    public final KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            beschreibung.einfuegen( voraussetzung.pruefen( held, wert ) );

        }

        return beschreibung;
    }

    /**
     * Legt die Kosten eines Vor-/Nachteils fest.
     *
     * @param dieKosten Das Objekt, das sich um die Kostenberechnung fuer den
     *      Vor-/Nachteil kuemmert.
     */
    private final void setKosten( final Kosten dieKosten ) {

        assert( null == kosten ) : String.format(
                "Die Kosten von %1$s sollten ein zweites mal festgelegt " +
                "werden.",
                getName() );

        assert( null != dieKosten ) : String.format(
                "Die Kosten von %1$s sollten auf null gesetzt werden.",
                getName() );

        kosten = dieKosten;
    }

    /**
     * Bestimmt die Stufen des Vor-/Nachteils.
     *
     * @param dieStufen Das Objekt das sich um die Grenzwerte fuer die Stufe
     *      eines Vor-/Nachteils kuemmert.
     */
    private final void setStufen( final Stufen dieStufen ) {

        assert( null == stufen ) : String.format(
                "Die Stufen von %1$s sollten ein zweites mal festgelegt " +
                "werden." +
                "vorhanden: %2$s\n" +
                "      neu: %3$s",
                getName(),
                stufen.getClass(),
                dieStufen.getClass() );

        assert( null != dieStufen ) : String.format(
                "Die Stufen von %1$s sollten auf null gesetzt werden.",
                getName() );

        stufen = dieStufen;
    }

    /**
     * @param vorNachteil
     *            die schwaechere Auspraegung des Vor-/Nachteils. Bei
     *            Herausragendes Aussehn ist dies z.B. Gut aussehend.
     */
    private final void setSchwaechereAuspraegung(
            final VorNachteil vorNachteil ) {

        assert ( null == schwaechereAuspraegung ) : String.format(
                "Die schwaechere Auspraegung von %1$s sollte ein zweites mal " +
                "festgelegt werden.\n" +
                "vorhandene schw. Auspraegung: %2$s\n" +
                "      neue schw. Auspraegung: %3$s",
                getName(),
                schwaechereAuspraegung.getName(),
                vorNachteil.getName() );

        schwaechereAuspraegung = vorNachteil;
    }

    /**
     * @param vorNachteil
     *            die staerkere Auspraegung des Vor-/Nachteils. Bei Balance ist
     *            dies z.B. Herausragende Balance.
     */
    private final void setStaerkereAuspraegung( final VorNachteil vorNachteil ){

        assert ( null == staerkereAuspraegung ) : String.format(
                "Die staerkere Auspraegung von %1$s sollte ein zweites mal " +
                "festgelegt werden.\n" +
                "vorhandene st. Auspraegung: %2$s\n" +
                "      neue st. Auspraegung: %3$s",
                getName(),
                staerkereAuspraegung.getName(),
                vorNachteil.getName() );

        staerkereAuspraegung = vorNachteil;
    }

    /**
     * @param derTyp legt fest ob es sich um eine Vorteil, Nachteil oder eine
     *      Schlechte Eigenschaft handelt.
     */
    private final void setTyp( final Typ derTyp ) {

        assert( null == typ ) : String.format(
                "Der Typ von %1$s wurde schon festgelegt.\n" +
                "bestehender Typ: %2$s\n" +
                "      neuer Typ: %3$s",
                getName(),
                typ,
                derTyp );

        assert( null != derTyp ) : String.format(
                "Der Typ von %1$s sollte auf null gesetzt werden.",
                getName() );

        typ = derTyp;
    }

    /**
     * Bestimmt die Varianten des Vor-/Nachteils.
     *
     * @param dieVarianten des Vor-/Nachteils.
     */
    private final void setVarianten( final Varianten dieVarianten ) {

        assert ( null == varianten ) : String.format(
                "Die Varianten von %1$s sollten ein zweites mal festgelegt " +
                "werden.\n" +
                "bestehende Varianten: %2$s\n" +
                "      neue Varianten: %3$s",
                getName(),
                varianten,
                dieVarianten );

        assert ( null != dieVarianten ) : String.format(
                "Die Varianten von %1$s sollten auf null gesetzt werden.",
                getName() );

        varianten = dieVarianten;
    }

    /**
     * Passt die Werte eines Helden an, nachdem die Stufe eines Wertes geaendert
     * wurde.
     *
     * @param held
     *            der den Wert besitzt und der angepasst werden soll.
     * @param wert
     *            dessen Stufe veraendert wurde.
     */
    public final void stufeGeaendert(
            final Held held,
            final VorNachteilWert wert ) {

        for ( final Anpasser a : anpasser ) {

            a.stufeGeaendert( held, wert );
        }
    }

    /** @return {@link #getName()} */
    @Override
    public final String toString() {
        return getName();
    }
}
