/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Varianten;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public abstract class Voraussetzung {

    /**
     * Speichert unter jedem Vor-/Nachteil v die Voraussetzung:
     * "Nicht kombinierbar mit v".
     */
    private static final Map< VorNachteil, Voraussetzung > NICHT_KOMBINIERBAR =
        new HashMap<VorNachteil, Voraussetzung>();

    /** Die Voraussetzung fuer den Nachteil Albino. */
    public static final Voraussetzung ALBINO = new VoraussetzungAlbino();

    /** Eine Voraussetzung fuer den Nachteil Arkanophobie. */
    public static final Voraussetzung ARKANOPHOBIE =
        new VoraussetzungArkanophobie();

    /**
     * Die Voraussetzungen fuer Begabung fuer [Talent].
     * Prueft auf inkompatible Begabungen/Unfaehigkeiten.
     */
    public static final Voraussetzung BEGABUNG_TALENT =
        new VoraussetzungBegabungUnfaehigkeitTalent(
                VorNachteil.UNFAEHIGKEIT_TALENT );

    /**
     * Die Voraussetzungen fuer Begabung fuer [Talentgruppe].
     * Prueft auf inkompatible Begabungen/Unfaehigkeiten.
     */
    public static final Voraussetzung BEGABUNG_TALENTGRUPPE =
        new VoraussetzungBegabungUnfaehigkeitTalentgruppe(
                VorNachteil.UNFAEHIGKEIT_TALENTGRUPPE );

    /**
     * Die Voraussetzungen fuer Begabung fuer [Talentgruppe].
     * Prueft ob Begabung fuer Schrift gewaehlt wurde, das ist laut Regeln nicht
     * moeglich.
     */
    public static final Voraussetzung BEGABUNG_TALENTGRUPPE_2 =
        new VoraussetzungNichtBegabungSchrift();

    /** Die Voraussetzung fuer den Vorteil Besonderer Besitz. */
    public static final Voraussetzung BESONDERER_BESITZ =
        new VoraussetzungBesondererBesitz();

    /** Verlangt, dass die GE des Helden mindestens 14 betraegt. */
    public static final Voraussetzung GE_14 =
        new VoraussetzungEigenschaft( Eigenschaft.GE, 14 );

    /**
     * Immunitaet gegen Gift ist nicht kombinierbar mit Resistenz gegen
     * das selbe Gift.
     */
    public static final Voraussetzung IMMUNITAET_GIFT =
        new VoraussetzungNichtGleicheVariante( VorNachteil.RESISTENZ_GIFT );

    /**
     * Immunitaet gegen Giftart ist nicht kombinierbar mit Resistenz gegen
     * die selbe Giftart.
     */
    public static final Voraussetzung IMMUNITAET_GIFTART =
        new VoraussetzungNichtGleicheVariante( VorNachteil.RESISTENZ_GIFTART );

    /**
     * Die Voraussetzung ueberprueft, dass Meisterhandwerk nur 5 mal gewaehlt
     * wird.
     */
    public static final Voraussetzung MEISTERHANDWERK =
        new VoraussetzungMaximal5Aktivierbar( Varianten.MEISTERHANDWERK );

    /** Eine Voraussetzung fuer den Vorteil Nachtsicht. */
    public static final Voraussetzung NACHTSICHT =
        new VoraussetzungNachtsicht();

    /** Fuer Totenangst, die nicht mit Anatomie kombiniert werden kann. */
    public static final Voraussetzung NICHT_ANATOMIE =
        new VoraussetzungNichtTalent( Talent.ANATOMIE );

    /** Fuer Arkanophobie, die fuer Elfen nicht waehlbar ist. */
    public static final Voraussetzung NICHT_ELFEN =
        new VoraussetzungNichtRasse( "Rasse.Elfen" );

    /**
     * Fuer Vor-/Nachteile die nicht frei vom Benutzer gewaehlt werden duerfen.
     */
    public static final Voraussetzung NICHT_FREI_WAEHLBAR =
        new VoraussetzungNichtFreiWaehlbar();

    /**
     * Fuer den Vorteil Viertelzauberer, der nicht von Kaempferprofessionen
     * gewaehlt werden kann.
     */
    public static final Voraussetzung NICHT_KAEMPFER =
        new VoraussetzungNichtKaempferprofession();

    /**
     * Fuer den Nachteil Einarmig, der nicht mit der Sonderfertigkeit Linkhand
     * kombiniert werden kann.
     */
    public static final Voraussetzung NICHT_LINKHAND =
        new VoraussetzungNichtSf( Sonderfertigkeit.LINKHAND );

    /**
     * Fuer den Nachteil Einarmig, der nicht mit der Sonderfertigkeit
     * Schildkampf kombiniert werden kann.
     */
    public static final Voraussetzung NICHT_SCHILDKAMPF_I =
        new VoraussetzungNichtSf( Sonderfertigkeit.SCHILDKAMPF_I );

    /**
     * Fuer den Nachteil Einarmig, der nicht mit der Sonderfertigkeit
     * Schildkampf kombiniert werden kann.
     */
    public static final Voraussetzung NICHT_SCHILDKAMPF_II =
        new VoraussetzungNichtSf( Sonderfertigkeit.SCHILDKAMPF_II );

    /**
     * Fuer den Nachteil Einarmig, der nicht mit dem Talent Zweihand-Hiebwaffen
     * kombiniert werden kann.
     */
    public static final Voraussetzung NICHT_ZWEIHAND_HIEBWAFFEN =
        new VoraussetzungNichtTalent( Talent.ZWEIHAND_HIEBWAFFEN );

    /**
     * Fuer den Nachteil Einarmig, der nicht mit dem Talent Zweihandschwerter
     * kombiniert werden kann.
     */
    public static final Voraussetzung NICHT_ZWEIHANDSCHWERTER =
        new VoraussetzungNichtTalent( Talent.ZWEIHANDSCHWERTER );

    /**
     * Die Voraussetzung fuer Niedrige Magieresistenz.
     */
    public static final Voraussetzung NIEDRIGE_MAGIERESISTENZ =
        new VoraussetzungNiedrigeMagieresistenz();

    /**
     * Fuer eine Vor-/Nachteil der nur von der Rasse Halbelfen frei gewaehlt
     * werden darf.
     */
    public static final Voraussetzung NUR_HALBELFEN =
        new VoraussetzungRasse( "Rasse.Halbelfen" );

    /**
     * Resistenz gegen Gift ist nicht kombinierbar mit Immunitaet gegen das
     * selbe Gift.
     */
    public static final Voraussetzung RESISTENZ_GIFT =
        new VoraussetzungNichtGleicheVariante( VorNachteil.IMMUNITAET_GIFT );

    /**
     * Resistenz gegen Giftart ist nicht kombinierbar mit Immunitaet gegen
     * die selbe Giftart.
     */
    public static final Voraussetzung RESISTENZ_GIFTART =
        new VoraussetzungNichtGleicheVariante( VorNachteil.IMMUNITAET_GIFTART );

    /** Verlangt, dass der SO des Helden mindestens 8 betraegt. */
    public static final Voraussetzung SO_8 =
        new VoraussetzungEigenschaft( Eigenschaft.SO, 8 );

    /**
     * Die Standardvoraussetzung die von allen Vor-/Nachteilen zu erfuellen ist.
     */
    public static final Voraussetzung STANDARD = new VoraussetzungStandard();

    /**
     * Prueft fuer einen Vor-/Nachteil mit Stufen ob die Grenzwerte eingehalten
     * werden.
     */
    public static final Voraussetzung STUFEN = new VoraussetzungStufen();

    /**
     * Prueft ob der Held das als Variante fuer den Vor-/Nachteil gewaehlte
     * Talent auch besitzt.
     */
    public static final Voraussetzung TALENT = new VoraussetzungTalent();

    /**
     * Die Voraussetzung ueberprueft, dass der Vorteil nur 5 mal gewaehlt
     * wird.
     */
    public static final Voraussetzung UEBERNATUERLICHE_BEGABUNG =
        new VoraussetzungMaximal5Aktivierbar(
                Varianten.UEBERNATUERLICHE_BEGABUNG );

    /**
     * Die Voraussetzungen fuer Unfaehigkeit fuer [Talentgruppe].
     * Prueft auf inkompatible Begabungen/Unfaehigkeiten.
     */
    public static final Voraussetzung UNFAEHIGKEIT_TALENTGRUPPE =
        new VoraussetzungBegabungUnfaehigkeitTalentgruppe(
                VorNachteil.BEGABUNG_TALENTGRUPPE );

    /**
     * Fuer Vor-/Nachteile die nur von Viertelzauberern gewaehlt
     * werden duerfen.
     */
    public static final Voraussetzung V = new VoraussetzungZV(
            VorNachteil.VIERTELZAUBERER,
            VorNachteil.VIERTELZAUBERER_UNBEWUSST );

    /**
     * Fuer Vor-/Nachteile die nur von Voll- und Viertelzauberern gewaehlt
     * werden duerfen.
     */
    public static final Voraussetzung ZV = new VoraussetzungZV(
            VorNachteil.VOLLZAUBERER,
            VorNachteil.VIERTELZAUBERER,
            VorNachteil.VIERTELZAUBERER_UNBEWUSST );

    /**
     * Fuer Vor-/Nachteile die nur von Vollzauberern gewaehlt
     * werden duerfen.
     */
    public static final Voraussetzung Z = new VoraussetzungZV(
            VorNachteil.VOLLZAUBERER );

    /**
     * Liefert zu einem Vor-/Nachteil v die Voraussetzung: "Nicht kombinierbar
     * mit v".
     *
     * @param vorNachteil
     *            zu dem die Voraussetzung gesucht ist.
     * @return die Voraussetzung "Nicht kombinierbar mit Vor-/Nachteil".
     */
    public static final Voraussetzung nichtKombinierbar(
            final VorNachteil vorNachteil ) {

        if ( ! NICHT_KOMBINIERBAR.containsKey( vorNachteil ) ) {

            NICHT_KOMBINIERBAR.put( vorNachteil,
                    new VoraussetzungNichtKombinierbar( vorNachteil ) );
        }

        return NICHT_KOMBINIERBAR.get( vorNachteil );
    }

    /**
     * @return die Regelelemente (Eigenschaften, Vor-/Nachteile usw.) von denen
     *         die Voraussetzung abhaengt.
     */
    public abstract Collection< Object > getAbhaengigkeiten();

    /**
     * Prueft ob ein Held die Voraussetzung fuer einen Wert erfuellt.
     *
     * @param held der die Voraussetzung erfuellen soll.
     * @param wert dessen Voraussetzung zu erfuellen ist.
     * @return eine Beschreibung der bestehenden Probleme wenn der Held die
     *      Voraussetzung nicht erfuellt.
     */
    public abstract KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert );
}
