/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class VoraussetzungNichtGleicheVariante extends Voraussetzung {

    /** 
     * Der Vor-/Nachteil, den der Held nicht in der gleichen Variante 
     * besitzen darf. 
     */
    private final VorNachteil vorNachteil;
    
    /** Ein aus {@link #vorNachteil} erzeugter Wert. */
    private VorNachteilWert inkompatiblerWert;
    
    /**
     * @param vorNachteil
     *            Der Vor-/Nachteil, den der Held nicht in der gleichen Variante
     *            besitzen darf.
     */
    VoraussetzungNichtGleicheVariante( final VorNachteil vorNachteil ) {
        
        this.vorNachteil = vorNachteil;
    }
    
    /** 
     * @return den Vor-/Nachteil den der Held nicht in der gleiche Variante
     *         besitzen darf. 
     */
    @Override
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) vorNachteil );
    }

    /**
     * Aendert die Variante von {@link #inkompatiblerWert} auf die uebergebenen Variante.
     * 
     * @param variante
     *            Die Variante fuer {@link #inkompatiblerWert}.
     */
    private final void setWert( final Variante variante ) {
        
        if ( null == inkompatiblerWert ) {
            
            inkompatiblerWert = 
                new VorNachteilWert( vorNachteil, variante );
        
        } else {
            
            inkompatiblerWert.setVariante( variante );
            
        }
    }
    
    /**
     * Prueft ob der Held den Vor-/Nachteil in der gleichen Variante besitzt und
     * gibt ggf. eine Fehlermeldung aus.
     */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final VorNachteilWert wert ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        setWert( wert.getVariante() );
        
        if ( held.hat( inkompatiblerWert ) ) {
            
            beschreibung.einfuegen( String.format(
                    VoraussetzungNichtKombinierbar.getNichtKombinierbar(), 
                    inkompatiblerWert ) );
        }
        
        return beschreibung;
    }

}
