/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber;

import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.CH;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.FF;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.GE;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.IN;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KK;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KL;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KO;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.MU;
import static de.bernhardjung.dsaprogramm.zauber.Repraesentation.ELFEN;
import static de.bernhardjung.dsaprogramm.zauber.Repraesentation.MAGIER;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.bernhardjung.dsaprogramm.basis.Eigenschaftsprobe;
import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.basis.Steigerbar;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basis.varianten.Varianten;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenFrei;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.zauber.kosten.Kosten;
import de.bernhardjung.dsaprogramm.zauber.voraussetzungen.Voraussetzung;

public class Zauber
extends ElementMitNameImpl
implements Steigerbar {

    /** Eine Map in der jeder zauber unter seiner Id gespeichert ist. */
    private static final Map< String, Zauber > ZAUBER =
        new HashMap< String, Zauber >();

    /**
     * Liefert einen Zauber mit einer bestimtmen Id.
     *
     * @param id
     *            des gesuchten Zaubers.
     * @return den Zauber mit der Id.
     * @throws IllegalArgumentException
     *             wenn es keinen Zauber mit der id gibt.
     */
    public static Zauber getZauber( final String id ) {

        if ( ZAUBER.containsKey( id ) ) {

            return ZAUBER.get( id );

        } else {

            throw new IllegalArgumentException(
                    "Es gibt keinen Zauber mit der Id " + id );
        }
    }

    /** Initialisiert alle Zauber. */
    @SuppressWarnings( "unchecked" )
    public static final void initialisieren() {

        new Zauber(
                "Zauber.AbvenenumReineSpeise",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, KL, FF );

        new Zauber(
                "Zauber.AdleraugeLuchsenohr",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, FF );

        new Zauber(
                "Zauber.AdlerschwingeWolfsgestalt",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.VOLLZAUBERER ),
                MU, IN, GE )
                .setVarianten( new VariantenFrei() )
                .setKosten( Kosten.ADLERSCHWINGE_WOLFSGESTALT );

        new Zauber(
                "Zauber.AnalysArcanstruktur",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, KL, IN );

        new Zauber(
                "Zauber.Armatrutz",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, GE, KO );

        new Zauber(
                "Zauber.Attributo",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, CH, MU, KL, IN, CH, FF, GE, KO, KK );

        new Zauber(
                "Zauber.AurisNasusOculus",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, CH, FF );

        new Zauber(
                "Zauber.AxxeleratusBlitzgeschwind",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, GE, KO );

        new Zauber(
                "Zauber.BalsamSalabunde",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, CH );

        new Zauber(
                "Zauber.Bannbaladin",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, CH, CH );

        new Zauber(
                "Zauber.BlickInDieGedanken",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, KL, CH );

        new Zauber(
                "Zauber.BlitzDichFind",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, GE );

        new Zauber(
                "Zauber.ClaudibusClavistibor",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_MAGIER,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, FF, KK );

        new Zauber(
                "Zauber.DuplicatusDoppelbild",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, CH, GE );

        new Zauber(
                "Zauber.EinflussBannen",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, CH, CH );

        new Zauber(
                "Zauber.ElfenstimmeFloetenton",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( ELFEN ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, CH, KO );

        new Zauber(
                "Zauber.ExposamiLebenskraft",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, IN );

        new Zauber(
                "Zauber.FalkenaugeMeisterschuss",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, FF, GE );

        new Zauber(
                "Zauber.FlimFlamFunkel",
                Zaubergruppe.ZAUBER,
                SKT.A,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, KL, FF );

        new Zauber(
                "Zauber.ForamenForaminor",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, KL, FF );

        new Zauber(
                "Zauber.FulminictusDonnerkeil",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, GE, KO );

        new Zauber(
                "Zauber.GardianumZauberschild",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, KO );

        new Zauber(
                "Zauber.GedankenbilderElfenruf",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, CH );

        new Zauber(
                "Zauber.HorriphobusSchreckgestalt",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                MU, IN, CH );

        new Zauber(
                "Zauber.IgnifaxiusFlammenstrahl",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, FF, KO );

        new Zauber(
                "Zauber.KlarumPurum",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, KL, CH );

        new Zauber(
                "Zauber.MotoricusGeisteshand",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, FF, KK );

        new Zauber(
                "Zauber.MovimentoDauerlauf",
                Zaubergruppe.ZAUBER,
                SKT.A,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, GE, KO );

        new Zauber(
                "Zauber.NebelwandUndMorgendunst",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, FF, KO );

        new Zauber(
                "Zauber.OdemArcanum",
                Zaubergruppe.ZAUBER,
                SKT.A,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, IN );

        new Zauber(
                "Zauber.ParalysisStarrWieStein",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, CH, KK );

        new Zauber(
                "Zauber.PenetrizzelTiefenblick",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_MAGIER,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, KL, KO );

        new Zauber(
                "Zauber.Psychostabilis",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_MAGIER,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                MU, KL, KO );

        new Zauber(
                "Zauber.SalanderMutander",
                Zaubergruppe.ZAUBER,
                SKT.E,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, CH, KO );

        new Zauber(
                "Zauber.Sanftmut",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                MU, CH, CH );

        new Zauber(
                "Zauber.SeidenzungeElfenwort",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, CH );

        new Zauber(
                "Zauber.SensibarEmpathicus",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, CH );

        new Zauber(
                "Zauber.SilentiumSchweigekreis",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, CH );

        new Zauber(
                "Zauber.SomnigravisTieferSchlaf",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, CH, CH );

        new Zauber(
                "Zauber.SpurlosTrittlos",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                IN, GE, GE );

        new Zauber(
                "Zauber.Tiergedanken",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                MU, IN, CH );

        new Zauber(
                "Zauber.VisibiliVanitar",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, CH );

        new Zauber(
                "Zauber.VocolimboHohlerKlang",
                Zaubergruppe.ZAUBER,
                SKT.B,
                Arrays.asList( MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, CH, FF );

        new Zauber(
                "Zauber.Wasseratem",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN, MAGIER ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.BEGINN_ELFEN,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                MU, KL, KO );

        new Zauber(
                "Zauber.WeisseMaehnUndGoldnerHuf",
                Zaubergruppe.ZAUBER,
                SKT.D,
                Arrays.asList( ELFEN ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                KL, IN, CH );

        new Zauber(
                "Zauber.ZaubernahrungHungerbann",
                Zaubergruppe.ZAUBER,
                SKT.C,
                Arrays.asList( ELFEN ),
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.REPRAESENTATION,
                               Voraussetzung.VOLLZAUBERER ),
                MU, MU, KO );

        // ***************************
        // UEBERNATUERLICHE BEGABUNGEN
        // ***************************

        new Zauber(
                "Zauber.AdleraugeLuchsenohr.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, FF );

        new Zauber(
                "Zauber.AdlerschwingeWolfsgestalt.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG,
                               Voraussetzung.HALBELFEN ),
                MU, IN, GE )
                .setVarianten( new VariantenFrei() );

        new Zauber(
                "Zauber.Armatrutz.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                IN, GE, KO );

        new Zauber(
                "Zauber.Attributo.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, CH, MU, KL, IN, CH, FF, GE, KO, KK );

        new Zauber(
                "Zauber.AxxeleratusBlitzgeschwind.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, GE, KO );

        new Zauber(
                "Zauber.BalsamSalabunde.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, CH );

        new Zauber(
                "Zauber.Bannbaladin.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                IN, CH, CH );

        new Zauber(
                "Zauber.BlickInDieGedanken.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, KL, CH );

        new Zauber(
                "Zauber.BlitzDichFind.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, GE );

        new Zauber(
                "Zauber.ExposamiLebenskraft.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, IN );

        new Zauber(
                "Zauber.FalkenaugeMeisterschuss.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                IN, FF, GE );

        new Zauber(
                "Zauber.FlimFlamFunkel.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, KL, FF );

        new Zauber(
                "Zauber.ForamenForaminor.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, KL, FF );

        new Zauber(
                "Zauber.FulminictusDonnerkeil.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG,
                               Voraussetzung.HALBELFEN ),
                IN, GE, KO );

        new Zauber(
                "Zauber.GedankenbilderElfenruf.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, CH );

        new Zauber(
                "Zauber.HorriphobusSchreckgestalt.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                MU, IN, CH );

        new Zauber(
                "Zauber.KlarumPurum.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, KL, CH );

        new Zauber(
                "Zauber.MovimentoDauerlauf.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                IN, GE, KO );

        new Zauber(
                "Zauber.OdemArcanum.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, IN );

        new Zauber(
                "Zauber.PenetrizzelTiefenblick.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, KL, KO );

        new Zauber(
                "Zauber.Psychostabilis.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                MU, KL, KO );

        new Zauber(
                "Zauber.Sanftmut.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                MU, CH, CH );

        new Zauber(
                "Zauber.SeidenzungeElfenwort.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, CH );

        new Zauber(
                "Zauber.SensibarEmpathicus.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                KL, IN, CH );

        new Zauber(
                "Zauber.SpurlosTrittlos.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                IN, GE, GE );

        new Zauber(
                "Zauber.Tiergedanken.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                MU, IN, CH );

        new Zauber(
                "Zauber.VisibiliVanitar.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG,
                               Voraussetzung.HALBELFEN ),
                KL, IN, CH );

        new Zauber(
                "Zauber.Wasseratem.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                MU, KL, KO );

        new Zauber(
                "Zauber.ZaubernahrungHungerbann.Begabung",
                Zaubergruppe.BEGABUNG,
                SKT.F,
                Collections.EMPTY_LIST,
                Arrays.asList( Voraussetzung.STANDARD,
                               Voraussetzung.VIERTELZAUBERER,
                               Voraussetzung.BEGABUNG ),
                MU, MU, KO );

    }
    
    /**
     * Prueft ob der uebergebenen String die ID eines Zaubers ist.
     * 
     * @param string der ueberprueft werden soll.
     * @return {@code true}, wenn es sich um eine ID handelt.
     */
    public static final boolean istZauberId( final String string ) {
        
        return ZAUBER.containsKey( string );
    }

    /** Die Zaubergruppe zu der der Zauber gehoert. */
    private final Zaubergruppe gruppe;

	/**
	 * Das Objekt, dass den Steigerungsfaktor des Zaubers und die Kosten
	 * bestimmt.
	 */
	private Kosten kosten;

	/** Die Probe auf die bei dem Zauber gewuerfelt wird. */
	private final Eigenschaftsprobe probe;

	/** Die Repraesentationen in denen der Zauber vorkommt.*/
	private final List< Repraesentation > repraesentationen;

	/** Die Komplexitaet des Zaubers, nach der er gesteigert wird. */
	private final int steigerungsspalte;

	/** Die moeglichen Varianten des Zaubers. */
	private Varianten varianten;

	/**
	 * Die Voraussetzungen des Zaubers.
	 */
	private final List< Voraussetzung > voraussetzungen;

	/**
	 * Erzeugt einen neuen Zauber und speichert ihn in {@link #ZAUBER}.
	 *
     * @param dieId
     *            des Zaubers.
     * @param dieZaubergruppe
     *            zu der der Zauber gehoert.
     * @param derSteigerungsfaktor
     *            bzw. die Komplexitaet, nach der der Zauber gesteigert wird.
     * @param dieRepraesentationen
     *            in denen der Zauber bekannt ist.
     * @param dieVoraussetzungen
     *            fuer den Zauber.
     * @param eigenschaft1
     *            die erste Probeneigenschaft.
     * @param eigenschaft2
     *            die zweite Probeneigenschaft.
     * @param eigenschaften3
     *            die moeglichen Eigenschaften fuer den dritten Probenwurf.
     */
	private Zauber(
	        final String dieId,
	        final Zaubergruppe dieZaubergruppe,
	        final int derSteigerungsfaktor,
	        final List< Repraesentation > dieRepraesentationen,
	        final List< Voraussetzung > dieVoraussetzungen,
	        final Eigenschaft eigenschaft1,
	        final Eigenschaft eigenschaft2,
	        final Eigenschaft ... eigenschaften3 ) {

	    super( dieId );
	    
        gruppe = dieZaubergruppe;

	    kosten = Kosten.STANDARD;

        probe = new Eigenschaftsprobe( eigenschaft1, eigenschaft2, eigenschaften3 );

        repraesentationen =
            new ArrayList< Repraesentation >( dieRepraesentationen );

        steigerungsspalte = derSteigerungsfaktor;

        varianten = Varianten.KEINE;

        voraussetzungen = new ArrayList< Voraussetzung >( dieVoraussetzungen );

        gruppe.einfuegen( this );

        ZAUBER.put( dieId, this );
	}

	/**
	 * Sortiert die Zauber alphabetisch nach Namen.
	 */
    public final int compareTo( final Zauber o ) {

        int ergebnis = Util.compare( getName(), o.getName() );

        if ( 0 == ergebnis ) {

            ergebnis = gruppe.compareTo( o.gruppe );
        }

        return ergebnis;
    }

    /**
     * Sucht die hoechsten Eigenschaftsstufe eines Helden.
     *
     * @param eigenschaften unter denen gesucht werden soll.
     * @param held dessen hoechster Wert gesucht ist.
     * @return die Stufe der hoechsten Eigenschaft.
     */
    private final int findeHoechsteEigenschaft(
            final Collection< Eigenschaft > eigenschaften,
            final Held held ) {

        int maximum = 0;

        for ( final Eigenschaft e : eigenschaften ) {

            final int stufe = held.getStufe( e );

            if ( stufe > maximum ) {

                maximum = stufe;
            }
        }

        return maximum;
    }

    /**
     * Sucht die niedrigste Eigenschaftsstufe eines Helden.
     *
     * @param eigenschaften unter denen gesucht werden soll.
     * @param held dessen niedrigster Wert gesucht ist.
     * @return die Stufe der niedrigsten Eigenschaft.
     */
    private final int findeNiedrigsteEigenschaft(
            final Collection< Eigenschaft > eigenschaften,
            final Held held ) {

        int minimum = Integer.MAX_VALUE;

        for ( final Eigenschaft e : eigenschaften ) {

            final int stufe = held.getStufe( e );

            if ( stufe < minimum ) {

                minimum = stufe;
            }
        }

        return minimum;
    }

    /**
     * @return Die Regelelemente von denen der Zauber abhaengig ist. Jeder
     *         Zauber ist auch von sich selbst abhaengig.
     */
    public final Collection< Object > getAbhaengigkeiten() {

        final Set< Object > abhaengigkeiten = new HashSet< Object >();

        abhaengigkeiten.add( this );

        abhaengigkeiten.addAll( probe.getEigenschaften() );

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            abhaengigkeiten.addAll( voraussetzung.getAbhaengigkeiten() );
        }

        abhaengigkeiten.addAll(kosten.getAbhaengigkeiten());

        return abhaengigkeiten;
    }

    /**
     * @param held
     *            der den Zauber aktivieren will.
     * @param wert
     *            dessen Aktivierungskosten ermittelt werden sollen.
     * @return die AP die der Held aufbringen muss um den Zauber zu aktivieren.
     */
    public int getAktivierungskosten( final Held held, final ZauberWert wert ) {
        return kosten.getAktivierungskosten( held, wert, Lernmethode.KEINE );
    }

    /**
     * @param held
     *            der den Zauber aktivieren will.
     * @param wert
     *            dessen Aktivierungskosten ermittelt werden sollen.
     * @param lernmethode
     *            die zum Aktivieren verwendet werden soll.
     * @return die AP die der Held aufbringen muss um den Zauber zu aktivieren.
     */
    public final int getAktivierungskosten(
            final Held held,
            final ZauberWert wert,
            final Lernmethode lernmethode ) {
        return kosten.getAktivierungskosten( held, wert, lernmethode );
    }

    /**
     * Berechnet den hoechsten noch erlaubten Wert fuer die Stufe des Zaubers.
     *
     * @param held fuer den die Grenze gelten soll.
     * @return die hoechste noch erlaubte Stufe fuer den Zauber.
     */
    public final int getHoechststufe( final Held held ) {

        final int stufe;

        if ( 1 == repraesentationen.size() ) {
            // Der Zauber kommt nur in einer Repraesentation vor.
            // Besitzt der Held die Repraesentation?
            final Repraesentation repraesentation = repraesentationen.get( 0 );

            final SonderfertigkeitWert sfWert = new SonderfertigkeitWert(
                    Sonderfertigkeit.getSonderfertigkeit(
                            "Sonderfertigkeit.Repraesentation" ),
                    repraesentation );

            if ( held.hat( sfWert ) ) {

                stufe = findeHoechsteEigenschaft( probe.getEigenschaften(),
                                                  held );

            } else {

                stufe = findeNiedrigsteEigenschaft( probe.getEigenschaften(),
                                                    held );
            }

        } else {

            stufe = findeHoechsteEigenschaft( probe.getEigenschaften(),
                                              held );
        }

        return stufe + 3;
    }

    /**
	 * @return die Probe auf die bei dem Zauber gewuerfelt wird.
	 */
	public final Eigenschaftsprobe getProbe() {
		return probe;
	}

	/**
	 * @return die Repraesentationen in denen der Zauber vorkommt.
	 */
    public final List< Repraesentation > getRepraesentationen() {
        return new ArrayList< Repraesentation >( repraesentationen );
    }

    /**
     * @return der Steigerungsfaktor fuer den Zauber.
     */
    public final int getSteigerungsfaktor() {
        return SKT.getSteigerungsfaktor( steigerungsspalte );
    }

    /**
     * @return Die Steigerungsspalte nach der der Zauber gesteigert wird.
     */
    public final int getSteigerungsspalte() {
        return steigerungsspalte;
    }

	/**
     * Liefert die Steigerungsspalte fuer einen Zauber, auf eine bestimmten
     * Stufe.
     *
     * @param held
     *            fuer den die Spalte gelten soll.
     * @param wert
     *            dessen Steigerungsspalte ermittelt werden soll.
     * @param stufe
     *            von der aus der Zauber um einen Punkt gesteigert werden soll.
     * @param lernmethode
     *            die zum Steigern verwendet werden soll.
     * @return die Komplexitaet, nach der der Zauber gesteigert wird.
     */
	public final int getSteigerungsspalte(
	        final Held held,
	        final ZauberWert wert,
	        final int stufe,
	        final Lernmethode lernmethode ) {

	    return kosten.getSteigerungsspalte( held, wert, stufe, lernmethode );
	}

	/**
	 * Liefert die Steigerungskosten fuer einen Zauber, auf eine bestimmten
	 * Stufe.
	 *
	 * @param held
	 *            der den Zauber steigern will.
	 * @param wert
	 *            Der Zauber um dessen Steigerung es geht.
	 * @param stufe
	 *            von der aus der Zauber um einen Punkte gesteigert werden soll.
	 * @param lernmethode
	 *            die zum Steigern verwendet werden soll.
	 * @return die AP die der Held aufbringen muss um den Zauber um einen Punkt
	 *         zu steigern.
	 */
    public final int getSteigerungskosten(
            final Held held,
            final ZauberWert wert,
            final int stufe,
            final Lernmethode lernmethode ) {

        return kosten.getSteigerungskosten( held, wert, stufe, lernmethode );
    }

    /**
     * Liefert die Variante zu einem Objekt. Das Objekt kann eine Variante sein,
     * in dem Fall wird natürlich die Variante einfach gecastet wieder zurueck
     * gegeben. Es kann aber auch ein vom Benutzer eingegebenen Text, ein
     * String uebergeben werden. In dem Fall wird dann die zum String passende
     * Variante, die den String als Namen hat, zurueckgegeben. Wenn es keine
     * passende Variante gibt, wird sie angezeigt.
     * 
     * @param objekt Ein String oder eine Variante
     * @return Die zum Objekt passende Variante.
     */
    public final Variante getVariante( final Object objekt ) {
        
        return varianten.getVariante( objekt );
    }
    
	/**
     * @param held
     *            fuer den die Varianten gesucht sind.
     * @return die moeglichen Varianten des Zaubers fuer den Helden.
     */
	public final List< Variante > getVarianten( final Held held ) {
	    return varianten.getVarianten( held );
	}

	/**
	 * @return die Zaubergruppe zu welcher der Zauber gehoert.
	 */
    public Zaubergruppe getZaubergruppe() {
        return gruppe;
    }

    /**
     * @see #einfuegenVariante(String)
     * @return {@code true}, wenn die Varianten des Zaubers vom Benutzer
     *         bestimmt werden koennen, sonst {@code false}.
     */
    public final boolean hatFreieVarianten() {
        return varianten.istFreieVariante();
    }

    /**
     * @return {@code true}, wenn der Zauber in Varianten vorkommt, sonst
     *         {@code false}.
     */
    public final boolean hatVarianten() {
        return varianten.hatVarianten();
    }

	/** Prueft ob der Held den Zauber waehlen darf. */
	public final KonfliktBeschreibung pruefen(
	        final Held held,
	        final ZauberWert wert ) {

	    final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

	    for ( final Voraussetzung voraussetzung : voraussetzungen )  {

	        beschreibung.einfuegen( voraussetzung.pruefen( held, wert ) );
	    }

	    return beschreibung;
	}

	/**
	 * @param dieKosten
	 *            Das Objekt das zur Bestimmung des Steigerungsfaktors und der
	 *            Steigerungskosten verwendet wird.
	 */
	private void setKosten( final Kosten dieKosten ) {
	    kosten = dieKosten;
	}

	/**
     * @param dieVarianten
     *            fuer den Zauber.
     */
	private Zauber setVarianten( final Varianten dieVarianten ) {
	    varianten = dieVarianten;
	    return this;
	}

	/** @return den Namen des Zaubers. */
	@Override
	public final String toString() {
		return getName();
	}

}
