/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.varianten.VarianteListener;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.util.Beobachter;

public class ZauberWerte implements Cloneable {

    /** Der Held dem die Zauber gehoeren. */
    private final Held held;

	/** Alle gespeicherten Zauber. */
	private Map< Zauber, List< ZauberWert > > zauber;

    /**
     * Erzeugt eine leere Liste fuer Zauber
     */
    public ZauberWerte() {
        this( null );
    }

    /**
     * Erzeugt eine leere Liste fuer Zauber.
     *
     * @param derHeld dessen Zauber in dieser Liste gespeichert werden.
     *       Der Held wird ueber Aenderungen an der Liste und an den Werten
     *       informiert. Dazu wird bei jeder Veraenderung die Methode
     *       {@link Held#notifyObservers()} aufgerufen.
     *       Darf {@code null} sein.
     */
	public ZauberWerte( final Held derHeld ) {

        held = derHeld;

		zauber = new HashMap< Zauber, List< ZauberWert > >();
	}
	
	/**
	 * @return Eine Deep Copy der Collection. 
	 */
	@Override
	public ZauberWerte clone() {
	    
	    ZauberWerte clone = null;
	    
	    try {
            
	        clone = (ZauberWerte) super.clone();
	        
	        clone.zauber = cloneZauber();
	        
        } catch ( CloneNotSupportedException e ) {
            
            DialogFehlermeldung.zeigeDialog( e );
        }
	    
	    return clone;
	}

	private Map< Zauber, List< ZauberWert > > cloneZauber() {
	 
	    final Map< Zauber, List< ZauberWert > > clone =
	        new HashMap< Zauber, List<ZauberWert> >();
	    
	    for ( final Zauber z : zauber.keySet() ) {
	        
	        clone.put( z, ZauberWert.deepCopy( zauber.get( z ) ) );
	    }
	    
	    return clone;
	}
	
	/**
	 * Hebt die Zauberfertigkeit (Stufe) eines Zaubers.
     * Wenn der Zauber in der Liste noch nicht existiert wird er eingefuegt,
     * wenn der Zauber schon in der Liste vorhanden ist wird seine Stufe um die
     * Stufe von {@code wert} erhoeht.
	 *
	 * @param wert Der Bonus den der Zauber erhalten soll.
     * @throws NullPointerException wenn {@code null} uebergeben wird.
	 */
	public void einfuegen( final ZauberWert wert ) {

		if ( ! enthaelt( wert ) ) {

            final ZauberWert neuerWert = wert.clone();

            if ( null != held ) {
                
                neuerWert.anmelden( (Beobachter) held );
                neuerWert.anmelden( (VarianteListener) held );
            }

            final Zauber z = wert.getZauber();

            final List< ZauberWert > liste;
            if ( zauber.containsKey( z ) ) {

                liste = zauber.get( z );

            } else {

                liste = new ArrayList< ZauberWert >( 1 );

                zauber.put( z, liste );
            }

            liste.add( neuerWert );

            if ( null != held ) {

                held.notifyObservers( wert.getZauber() );
            }

		} else {

		    getWert( wert ).addiereStufe( wert.getStufe() );

		}

	}

	/**
	 * Entfernt einen Zauber aus der Liste, so vorhanden.
	 *
	 * @param wert der entfernt werden soll.
	 */
	public void entfernen( final ZauberWert wert ) {

    	if ( enthaelt( wert ) ) {

    	    final List< ZauberWert > werte = zauber.get( wert.getZauber() );

    	    for ( int i = 0; i < werte.size() ; ++i ) {
    	        
    	        if ( werte.get( i ).hatGleichenZauberUndVariante( wert ) ) {
    	            
    	            final ZauberWert entfernterWert = werte.remove( i );
    	            
    	            if ( held != null ) {
    	                
    	                entfernterWert.abmelden( (Beobachter) held );
    	                entfernterWert.abmelden( (VarianteListener) held );
    	            }
    	            
    	            break;
    	        }
    	    }

    	    if ( 0 == werte.size() ) {

    	        zauber.remove( wert.getZauber() );
    	    }

            if ( null != held ) {

                held.notifyObservers( wert.getZauber() );
            }

    	}
	}

    /**
     * Prueft ob ein Zauber in der Liste gespeichert ist.
     *
     * @param z der gesucht wird.
     * @return {@code true}, wenn der Zauber gespeichert ist, sonst
     *          {@code false}.
     */
    public boolean enthaelt( final Zauber z ) {
        return zauber.containsKey( z );
    }

	/**
	 * Prueft ob ein Zauber in der Liste gespeichert ist.
	 *
	 * @param wert der gesucht wird.
	 * @return {@code true}, wenn der Zauber gespeichert ist, sonst
	 * 			{@code false}.
	 */
	public boolean enthaelt( final ZauberWert wert ) {

	    final Zauber z = wert.getZauber();

	    if ( zauber.containsKey( z ) ) {

	        final List< ZauberWert > werte = zauber.get( z );
	        
	        for ( int i = 0 ; i < werte.size(); ++i ) {
	            
	            if ( werte.get( i ).hatGleichenZauberUndVariante( wert ) ) {
	                
	                return true;
	            }
	        }
	    }
	    
	    return false;
	}

	@Override
	public boolean equals( Object obj ) {
	    
	    if ( null == obj 
	            || ! obj.getClass().equals( getClass() ) ) {
	        
	        return false;
	    }
	    
	    final ZauberWerte werte = (ZauberWerte) obj;
	    
	    if ( zauber.size() != werte.zauber.size() ) {
	        
	        return false;
	    }
	    
	    for ( final Zauber z : werte.zauber.keySet() ) {
	        
	        if ( ! zauber.containsKey( z ) ) {
	            
	            return false;
	        }
	        
	        if ( ! zauber.get( z ).containsAll( werte.zauber.get( z ) ) )
	        {
	            return false;
	        }
	    }
	    
	    return true;
	}
	
    /**
     * Liefert eine Liste aller Zauber zu einer bestimmten Zaubergruppe.
     *
     * @param zaubergruppe
     *            deren Zauber gesucht werden.
     * @return eine Liste mit allen gespeicherten Zaubern zur gesuchten
     *         Zaubergruppe.
     */
    public List< ZauberWert > get( final Zaubergruppe zaubergruppe ) {

        final List< ZauberWert > liste = new ArrayList< ZauberWert >();

        for ( final Zauber z : zauber.keySet() ){

            if ( z.getZaubergruppe().equals( zaubergruppe ) ) {

                liste.addAll( zauber.get( z ) );
            }
        }

        return liste;
    }

	/**
	 * @return die Anzahl der gespeicherten Zauber.
	 * /
	public int getAnzahlZauber() {
		return zauber.size();
	}

	/**
     * @return eine Liste aller gespeicherten Zauber.
     */
	public List< ZauberWert > getZauber() {

	    final List< ZauberWert > werte = new ArrayList< ZauberWert >();

	    for ( final Zauber z : zauber.keySet() ) {

	        werte.addAll( zauber.get( z ) );
	    }

	    Collections.sort( werte, ZauberWert.NAME_COMPARATOR );

	    return werte;
	}

	/**
	 * @return den zum Zauber gehoerenden ZauberWert oder {@code null},
	 * 		wenn kein passender ZauberWert vorhanden ist. Es wird das
     *      Objekt geliefert, das in dieser Liste gespeichert ist. Aenderungen
     *      an diesem Objekt wirken sich also direkt aus.
	 */
	public ZauberWert getWert( final ZauberWert wert ) {

	    if ( enthaelt( wert ) ) {

	        for ( final ZauberWert w : zauber.get( wert.getZauber() ) ) {

	            if ( w.hatGleichenZauberUndVariante( wert ) ) {

	                return w;
	            }
	        }
	    }

	    return null;
	}

	/**
	 * Liefert alle Varianten eines Zaubers.
	 *
	 * @param z
	 *            dessen Varianten gesucht sind.
	 * @return eine Liste aller Varianten des Zaubers, die in diesem Objekt
	 *         gespeichert sind. Es handelt sich um die Wert-Objekte, die in
	 *         diesem Objekt gespeichert sind, Aenderungen an ihnen wirken sich
	 *         auf die in der Liste gespeicherten Objekte aus.
	 */
    public List< ZauberWert > getWerte( final Zauber z ) {

        final List< ZauberWert > werte = new ArrayList< ZauberWert >();

        if ( zauber.containsKey( z ) ) {

            for ( final ZauberWert wert : zauber.get( z ) ) {

                werte.add( wert );
            }
        }

        return werte;
    }

}
