/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber;

import java.text.Collator;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class Zaubergruppe
extends AbstractList< Zauber >
implements ElementMitName {

    /** Eine Liste aller Zaubergruppen. */
    private static final List< Zaubergruppe > GRUPPEN;

    /** Die normalen Zauber, fuer Elfen und Magier. */
    public static final Zaubergruppe ZAUBER =
        new Zaubergruppe( "Zaubergruppe.Zauber" );

    /** Die Zauber fuer den Vorteil uebernatuerliche Begabung. */
    public static final Zaubergruppe BEGABUNG =
        new Zaubergruppe( "Zaubergruppe.UebernatuerlicheBegabungen" );

    static {

        GRUPPEN = new ArrayList< Zaubergruppe >( 2 );

        GRUPPEN.add( ZAUBER );
        GRUPPEN.add( BEGABUNG );
        
        Collections.sort( GRUPPEN, NAME_COMPARATOR );
    }

    /**
     * @return eine sortierte Liste aller Zaubergruppen.
     */
    public static final List< Zaubergruppe > getZaubergruppen() {
        return new ArrayList< Zaubergruppe >( GRUPPEN );
    }

    /**
     * Sortiert die Zauber in den Gruppen neu, wenn die Sprache im Programm
     * geaendert wurde.
     */
    public static final void neuSortieren() {
        
        for ( final Zaubergruppe gruppe : GRUPPEN ) {
            
            gruppe.sortieren();
        }
        
        Collections.sort( GRUPPEN, NAME_COMPARATOR );
    }
    
    /** Die Id der Gruppe */
    private final String id;

    /** Liefert Name und Kuerzel der Gruppe. */
    private final ElementMitName name;

    /** Die Zauber die zu der Gruppe gehoeren. */
    private final List< Zauber > zauber;

    /**
     * @param dieId der Gruppe.
     */
    private Zaubergruppe( final String dieId ) {
        
        id = dieId;

        name = new ElementMitNameImpl( id );

        zauber = new ArrayList< Zauber >();
    }

    /** Sortiert die Zaubergruppen alphabtisch nach Namen. */
    public int compareTo( final Zaubergruppe o ) {
        
        final Collator collator =
            Collator.getInstance( Einstellungen.getInstanz().getSprache() );
        
        return collator.compare( getName(), o.getName() );
    }

    /**
     * Fuegt einen neuen Zauber in die Gruppe ein.
     * Macht nichts, wenn der Zauber schon in der Liste ist.
     *
     * @param neuerZauber fuer die Gruppe.
     */
    final void einfuegen( final Zauber neuerZauber ) {

        int index = Collections.binarySearch( 
                zauber, 
                neuerZauber,
                Zauber.NAME_COMPARATOR );

        if ( 0 > index ) {

            index = -( index + 1 );

            zauber.add( index, neuerZauber );
        }
    }

    /**
     * @return die Id der Gruppe.
     */
    public final String getId() {
        
        return name.getId();
    }
    
    /**
     * @return die Kurzform des Namens.
     */
    public final String getKuerzel() {
        return name.getKuerzel();
    }

    /**
     * @return einen Zauber aus der Liste.
     */
    @Override
    public final Zauber get( final int index ) {
        return zauber.get( index );
    }
    
    /**
     * @return der Schluessel ueber den das Kuerzel der Gruppe geladen wird.
     */
    public String getLokalisierungsschluesselKuerzel() {
        
        return name.getLokalisierungsschluesselKuerzel();
    }
    
    /**
     * @return den Lokalisierungsschluessel, mit dem der Name der Gruppe
     *          ermittelt wird.
     */
    public final String getLokalisierungsschluesselName() {
        
        return name.getLokalisierungsschluesselName();
    }

    /**
     * @return Den Namen der Gruppe.
     */
    public final String getName() {
        return name.getName();
    }

    /**
     * Aendert den Namen der Gruppe.
     */
    public final void setName( final String neuerName ) {
        
        name.setName( neuerName );
    }
    
    /**
     * @return die Anzahl von Zaubern in der Gruppe.
     */
    @Override
    public final int size() {
        return zauber.size();
    }
    
    /**
     * Sortiert die Zauber in der Gruppe, neu wenn die Sprache im Programm 
     * geaendert wurde.
     */
    private final void sortieren() {
        
        Collections.sort( zauber, Zauber.NAME_COMPARATOR );
    }
}
