/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

class KostenAdlerschwingeWolfsgestalt extends KostenStandard {

    /**
     * {@inheritDoc}
     */
    @Override
    public int getAktivierungskosten(
            final Held held,
            final ZauberWert wert,
            final Lernmethode lernmethode ) {

        int faktor = super.getSteigerungsspalte( held, wert, 0, lernmethode );

        final Zauber zauber = wert.getZauber();

        if ( held.hatVorgegeben( zauber ) ) {

            --faktor;

        } else {

            final ZauberWert hoechster = getHoechstenWert( held, zauber );

            if ( ! wert.equals( hoechster ) ) {

                --faktor;
            }
        }

        if ( held.wirdErschaffen() ) {

            return SKT.getAktivierungskostenErschaffung( faktor );

        } else {

            return SKT.getAktivierungskosten( faktor );
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getSteigerungsspalte(
            final Held held,
            final ZauberWert wert,
            final int stufe, final Lernmethode lernmethode ) {

        int faktor = super.getSteigerungsspalte( held, wert, stufe, lernmethode );

        final Zauber zauber = wert.getZauber();

        boolean verbilligungWirktSchon = false;

        if ( held.hatVorgegeben( zauber ) ) {

            final int vorgegebeneStufe =
                getHoechsteVorgegebeneStufe( held, zauber );

            if ( stufe < vorgegebeneStufe ) {

                --faktor;

                verbilligungWirktSchon = true;
            }
        }

        if ( ! verbilligungWirktSchon ) {

            final ZauberWert hoechsterWert = getHoechstenWert( held, zauber );

            if ( ( null != hoechsterWert )
              && ( ! wert.equals( hoechsterWert ) )
              && ( stufe < hoechsterWert.getStufe() )) {

                --faktor;
            }
        }

        return faktor;
    }

    /**
     * Sucht die Variante eines Zaubers mit der hoechsten Stufe, die ein Held
     * besitzt.
     *
     * @param held
     *            dessen hoechster Zauber gesucht ist.
     * @param zauber
     *            dessen Variante mit der hoechsten Stufe gesucht ist.
     * @return die Variante mit der hoechsten Stufe, die der Held besitzt, oder
     *         {@code null}, wenn der Held den Zauber nicht besitzt.
     */
    private ZauberWert getHoechstenWert(
            final Held held,
            final Zauber zauber ) {

        int hoechsteStufe = Integer.MIN_VALUE;

        ZauberWert hoechster = null;

        for ( final ZauberWert wert : held.getWerte( zauber ) ) {

            if ( hoechsteStufe < wert.getStufe() ) {

                hoechsteStufe = wert.getStufe();

                hoechster = wert;
            }
        }

        return hoechster;
    }

    /**
     * Liefert die hoechste vorgegebene Stufe in einem Zauber.
     *
     * @param held
     *            dessen Hoechststufe gesucht ist.
     * @param zauber
     *            in dem die Hoechststufe gesucht ist.
     * @return die hoechste Stufe, die der Held in dem Zauber besitzt oder
     *         {@link Integer#MIN_VALUE}, wenn der Zauber nicht vorgegeben ist.
     */
    private int getHoechsteVorgegebeneStufe(
            final Held held,
            final Zauber zauber ) {

        int hoechsteStufe = Integer.MIN_VALUE;

        for ( final ZauberWert wert : held.getVorgegeben( zauber ) ) {

            final int stufe = wert.getStufe();

            if ( hoechsteStufe < stufe ) {

                hoechsteStufe = stufe;
            }
        }

        return hoechsteStufe;
    }
}
