/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.zauber.Repraesentation;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

import java.util.ArrayList;
import java.util.List;

/**
 * Enthaelt den Standardalgorithmus zum Berechnen von Kosten fuer Zauber.
 */
class KostenStandard implements Kosten {

    /**
     * @return Den Vorteil Gutes Gedaechtnis.
     */
    public List<Object> getAbhaengigkeiten() {

        List<Object> abhaengigkeiten = new ArrayList<Object>(1);

        abhaengigkeiten.add(VorNachteil.GUTES_GEDAECHTNIS);

        return abhaengigkeiten;
    }

    /**
     * {@inheritDoc}
     */
    public int getAktivierungskosten(
            final Held held,
            final ZauberWert wert,
            final Lernmethode lernmethode ) {

        int kosten;

        final int faktor = getSteigerungsspalte( held, wert, 0, lernmethode );

        if ( held.wirdErschaffen() ) {

            kosten = SKT.getAktivierungskostenErschaffung( faktor );

        } else {

            kosten = SKT.getAktivierungskosten( faktor );
        }

        if (held.hat(VorNachteil.GUTES_GEDAECHTNIS))
            kosten = (int) Math.round( 0.75 * kosten );

        return kosten;
    }

    /**
     * {@inheritDoc}
     */
    public int getSteigerungsspalte(
            final Held held,
            final ZauberWert wert,
            final int stufe,
            final Lernmethode lernmethode ) {

        final Zauber zauber = wert.getZauber();

        int faktor = lernmethode.bestimmeSteigerungsfaktor( zauber );

        final List< Repraesentation > repraesentationen =
            zauber.getRepraesentationen();

        if ( 1 == repraesentationen.size() ) {
            // Der Zauber kommt nur in einer Repraesentation vor.
            // Besitzt der Held die Repraesentation?
            final Repraesentation repraesentation = repraesentationen.get( 0 );

            final SonderfertigkeitWert sfWert = new SonderfertigkeitWert(
                    Sonderfertigkeit.getSonderfertigkeit(
                            "Sonderfertigkeit.Repraesentation" ),
                    repraesentation );

            if ( ! held.hat( sfWert ) ) {
                faktor += 2;
            }
        }

        if ( held.hatHauszauber( zauber ) ) {

            --faktor;

        } else if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
               && ( ! held.hatLeitzauber( zauber ) ) ) {

            ++faktor;
        }

        return faktor;
    }

    /** {@inheritDoc} */
    public int getSteigerungskosten(
            final Held held,
            final ZauberWert wert,
            final int stufe,
            final Lernmethode lernmethode ) {

        final int faktor = getSteigerungsspalte( held, wert, stufe, lernmethode );

        int kosten = SKT.getKosten( 1 + stufe, faktor );

        if (held.hat(VorNachteil.GUTES_GEDAECHTNIS))
            kosten = (int) Math.round( 0.75 * kosten );

        return kosten;
    }

}
